@@ -33,6 +33,7 @@ extern "C" {
 #endif
 
 #include "systemimpl.h"
+#include "is_utf8.h"
 
 /*
  * Common includes
@@ -354,7 +355,7 @@ static char* SystemImpl__readFile(const char* filename)
   }
 
   /* adrpo: if size is larger than the max string, return a different string */
-#ifndef _LP64
+#if !(defined(_LP64) || defined(_LLP64) || defined(_WIN64) || defined(__MINGW64__))
   if (statstr.st_size > (pow((double)2, (double)22) * 4)) {
     const char *c_tokens[1]={filename};
     c_add_message(NULL,85, /* ERROR_OPENING_FILE */
@@ -416,6 +417,9 @@ int SystemImpl__writeFile(const char* filename, const char* data)
 #endif
   FILE * file = NULL;
   int len = strlen(data); /* MMC_HDRSTRLEN(MMC_GETHDR(rmlA1)); */
+#if defined(__APPLE_CC__)||defined(__MINGW32__)||defined(__MINGW64__)
+  unlink(filename);
+#endif
   /* adrpo: 2010-09-22 open the file in BINARY mode as otherwise \r\n becomes \r\r\n! */
   file = fopen(filename,fileOpenMode);
   if (file == NULL) {
@@ -2238,15 +2242,36 @@ char* SystemImpl__iconv__ascii(const char * str)
   return buf;
 }
 
+static int isUtf8Encoding(const char *str)
+{
+  return strcasecmp(str, "UTF-8") || strcasecmp(str, "UTF8");
+}
+
 extern char* SystemImpl__iconv(const char * str, const char *from, const char *to, int printError)
 {
-  char *in_str,*res;
+  char *in_str,*res=NULL;
   size_t sz,out_sz,buflen;
   iconv_t ic;
   int count;
   char *buf;
   sz = strlen(str);
-  buflen = sz*8;
+  if (isUtf8Encoding(from) && isUtf8Encoding(to))
+  {
+    is_utf8(str, sz, &res, &count);
+    if (res==NULL) {
+      /* Converting from UTF-8 to UTF-8 and the sequence is already UTF-8... */
+      return str;
+    }
+    /* Converting from UTF-8, but is not valid UTF-8. Just quit early. */
+    if (printError) {
+      char *ignore = SystemImpl__iconv__ascii(str);
+      const char *tokens[4] = {res,from,to,ignore};
+      c_add_message(NULL,-1,ErrorType_scripting,ErrorLevel_error,gettext("iconv(\"%s\",to=\"%s\",from=\"%s\") failed: %s"),tokens,4);
+      omc_alloc_interface.free_uncollectable(ignore);
+    }
+    return (char*) "";
+  }
+  buflen = sz*4;
   /* fprintf(stderr,"iconv(%s,to=%s,%s) of size %d, buflen %d\n",str,to,from,sz,buflen); */
   ic = iconv_open(to, from);
   if (ic == (iconv_t) -1) {
@@ -2259,7 +2284,13 @@ extern char* SystemImpl__iconv(const char * str, const char *from, const char *t
     return (char*) "";
   }
   buf = (char*) omc_alloc_interface.malloc_atomic(buflen);
-  assert(buf != 0);
+  if (0 == buf) {
+    if (printError) {
+      /* Make the error message small so we perhaps have a chance to recover */
+      c_add_message(NULL,-1,ErrorType_scripting,ErrorLevel_error,gettext("iconv() ran out of memory"),NULL,0);
+    }
+    return (char*) "";
+  }
   *buf = 0;
   in_str = (char*) str;
   out_sz = buflen-1;
@@ -2612,9 +2643,12 @@ int System_getTerminalWidth(void)
 
 #include "simulation_options.h"
 
+#define SB_SIZE 8192*2
+#define SB_SIZE_MINUS_ONE (SB_SIZE-1)
+
 char* System_getSimulationHelpTextSphinx(int detailed, int sphinx)
 {
-  static char buf[8192];
+  static char buf[SB_SIZE];
   int i,j;
   const char **desc = detailed ? FLAG_DETAILED_DESC : FLAG_DESC;
   char *cur = buf;
@@ -2622,13 +2656,13 @@ char* System_getSimulationHelpTextSphinx(int detailed, int sphinx)
   for(i=1; i<FLAG_MAX; ++i)
   {
     if (sphinx) {
-      cur += snprintf(cur, 8191-(buf-cur), "\n.. _simflag-%s :\n\n", FLAG_NAME[i]);
+      cur += snprintf(cur, SB_SIZE_MINUS_ONE-(buf-cur), "\n.. _simflag-%s :\n\n", FLAG_NAME[i]);
     }
     if (FLAG_TYPE[i] == FLAG_TYPE_FLAG) {
       if (sphinx) {
-        cur += snprintf(cur, 8191-(buf-cur), ":ref:`-%s <simflag-%s>`\n%s\n", FLAG_NAME[i], FLAG_NAME[i], desc[i]);
+        cur += snprintf(cur, SB_SIZE_MINUS_ONE-(buf-cur), ":ref:`-%s <simflag-%s>`\n%s\n", FLAG_NAME[i], FLAG_NAME[i], desc[i]);
       } else {
-        cur += snprintf(cur, 8191-(buf-cur), "<-%s>\n%s\n", FLAG_NAME[i], desc[i]);
+        cur += snprintf(cur, SB_SIZE_MINUS_ONE-(buf-cur), "<-%s>\n%s\n", FLAG_NAME[i], desc[i]);
       }
     } else if (FLAG_TYPE[i] == FLAG_TYPE_OPTION) {
       int numExtraFlags=0;
@@ -2636,9 +2670,9 @@ char* System_getSimulationHelpTextSphinx(int detailed, int sphinx)
       const char **flagName;
       const char **flagDesc;
       if (sphinx) {
-        cur += snprintf(cur, 8191-(buf-cur), ":ref:`-%s=value <simflag-%s>` *or* -%s value \n%s\n", FLAG_NAME[i], FLAG_NAME[i], FLAG_NAME[i], desc[i]);
+        cur += snprintf(cur, SB_SIZE_MINUS_ONE-(buf-cur), ":ref:`-%s=value <simflag-%s>` *or* -%s value \n%s\n", FLAG_NAME[i], FLAG_NAME[i], FLAG_NAME[i], desc[i]);
       } else {
-        cur += snprintf(cur, 8191-(buf-cur), "<-%s=value> or <-%s value>\n%s\n", FLAG_NAME[i], FLAG_NAME[i], desc[i]);
+        cur += snprintf(cur, SB_SIZE_MINUS_ONE-(buf-cur), "<-%s=value> or <-%s value>\n%s\n", FLAG_NAME[i], FLAG_NAME[i], desc[i]);
       }
 
       switch(i) {
@@ -2681,20 +2715,20 @@ char* System_getSimulationHelpTextSphinx(int detailed, int sphinx)
       }
 
       if (numExtraFlags) {
-        cur += snprintf(cur, 8191-(buf-cur), "\n");
+        cur += snprintf(cur, SB_SIZE_MINUS_ONE-(buf-cur), "\n");
         if (flagName) {
           for (j=firstExtraFlag; j<numExtraFlags; j++) {
-            cur += snprintf(cur, 8191-(buf-cur), "  * %s (%s)\n", flagName[j], flagDesc[j]);
+            cur += snprintf(cur, SB_SIZE_MINUS_ONE-(buf-cur), "  * %s (%s)\n", flagName[j], flagDesc[j]);
           }
         } else {
           for (j=firstExtraFlag; j<numExtraFlags; j++) {
-            cur += snprintf(cur, 8191-(buf-cur), "  * %s\n", flagDesc[j]);
+            cur += snprintf(cur, SB_SIZE_MINUS_ONE-(buf-cur), "  * %s\n", flagDesc[j]);
           }
         }
       }
 
     } else {
-      cur += snprintf(cur, 8191-(buf-cur), "[unknown flag-type] <-%s>\n", FLAG_NAME[i]);
+      cur += snprintf(cur, SB_SIZE_MINUS_ONE-(buf-cur), "[unknown flag-type] <-%s>\n", FLAG_NAME[i]);
     }
   }
   *cur = 0;
@@ -2759,7 +2793,7 @@ int SystemImpl__fileIsNewerThan(const char *file1, const char *file2)
 
 void SystemImpl__initGarbageCollector(void)
 {
-  static init=0;
+  static int init=0;
   if (!init) {
     GC_init();
     GC_register_displacement(0);
@@ -2911,6 +2945,9 @@ int SystemImpl__covertTextFileToCLiteral(const char *textFile, const char *outFi
     goto done;
   }
   errno = 0;
+#if defined(__APPLE_CC__)||defined(__MINGW32__)||defined(__MINGW64__)
+  unlink(outFile);
+#endif
   fout = fopen(outFile, "w");
   if (!fout) {
     const char *c_token[1]={strerror(errno)};
@@ -3017,10 +3054,10 @@ done:
 
 void SystemImpl__dladdr(void *symbol, const char **file, const char **name)
 {
-#if defined(__MINGW32__) || defined(_MSC_VER)
+#if defined(_MSC_VER)
   *file = "dladdr failed";
   *name = "not available on Windows";
-#else
+#else /* mingw & Linux */
   Dl_info info;
   void *ptr = (MMC_FETCH(MMC_OFFSET(MMC_UNTAGPTR(symbol), 1)));
   if (0 == dladdr(ptr, &info)) {
