@@ -176,6 +176,7 @@ typedef struct omc_ModelInput
   omc_ModelVariables    *rAlg; /* algebraic */
   omc_ModelVariables    *rPar; /* parameters */
   omc_ModelVariables    *rAli; /* aliases */
+  omc_ModelVariables    *rSen; /* sensitivities */
 
   omc_ModelVariables    *iAlg; /* int algebraic */
   omc_ModelVariables    *iPar; /* int parameters */
@@ -273,6 +274,8 @@ static void XMLCALL startElement(void *userData, const char *name, const char **
           mi->lastCT = &mi->rPar;
         } else if (0 == strcmp(ct+1,"Ali")) {
           mi->lastCT = &mi->rAli;
+        } else if (0 == strcmp(ct+1,"Sen")) {
+          mi->lastCT = &mi->rSen;
         } else {
           fail = 1;
         }
@@ -371,7 +374,6 @@ static void read_var_info(omc_ScalarVariable *v, VAR_INFO *info)
 
 static void read_var_attribute_real(omc_ScalarVariable *v, REAL_ATTRIBUTE *attribute)
 {
-  read_value_bool(findHashStringString(v,"useStart"), (modelica_boolean*)&(attribute->useStart));
   read_value_real(findHashStringStringEmpty(v,"start"), &(attribute->start), 0.0);
   read_value_bool(findHashStringString(v,"fixed"), (modelica_boolean*)&(attribute->fixed));
   read_value_bool(findHashStringString(v,"useNominal"), (modelica_boolean*)&(attribute->useNominal));
@@ -379,37 +381,34 @@ static void read_var_attribute_real(omc_ScalarVariable *v, REAL_ATTRIBUTE *attri
   read_value_real(findHashStringStringEmpty(v,"min"), &(attribute->min), REAL_MIN);
   read_value_real(findHashStringStringEmpty(v,"max"), &(attribute->max), REAL_MAX);
 
-  infoStreamPrint(LOG_DEBUG, 0, "Real %s(%sstart=%g%s, fixed=%s, %snominal=%g%s, min=%g, max=%g)", findHashStringString(v,"name"), (attribute->useStart)?"":"{", attribute->start, (attribute->useStart)?"":"}", (attribute->fixed)?"true":"false", (attribute->useNominal)?"":"{", attribute->nominal, attribute->useNominal?"":"}", attribute->min, attribute->max);
+  infoStreamPrint(LOG_DEBUG, 0, "Real %s(start=%g, fixed=%s, %snominal=%g%s, min=%g, max=%g)", findHashStringString(v,"name"), attribute->start, (attribute->fixed)?"true":"false", (attribute->useNominal)?"":"{", attribute->nominal, attribute->useNominal?"":"}", attribute->min, attribute->max);
 }
 
 static void read_var_attribute_int(omc_ScalarVariable *v, INTEGER_ATTRIBUTE *attribute)
 {
-  read_value_bool(findHashStringString(v,"useStart"), &attribute->useStart);
   read_value_long(findHashStringStringEmpty(v,"start"), &attribute->start, 0);
   read_value_bool(findHashStringString(v,"fixed"), &attribute->fixed);
   read_value_long(findHashStringStringEmpty(v,"min"), &attribute->min, INTEGER_MIN);
   read_value_long(findHashStringStringEmpty(v,"max"), &attribute->max, INTEGER_MAX);
 
-  infoStreamPrint(LOG_DEBUG, 0, "Integer %s(%sstart=%ld%s, fixed=%s, min=%ld, max=%ld)", findHashStringString(v,"name"), attribute->useStart?"":"{", attribute->start, attribute->useStart?"":"}", attribute->fixed?"true":"false", attribute->min, attribute->max);
+  infoStreamPrint(LOG_DEBUG, 0, "Integer %s(start=%ld, fixed=%s, min=%ld, max=%ld)", findHashStringString(v,"name"), attribute->start, attribute->fixed?"true":"false", attribute->min, attribute->max);
 }
 
 static void read_var_attribute_bool(omc_ScalarVariable *v, BOOLEAN_ATTRIBUTE *attribute)
 {
-  read_value_bool(findHashStringString(v,"useStart"), &attribute->useStart);
   read_value_bool(findHashStringStringEmpty(v,"start"), &attribute->start);
   read_value_bool(findHashStringString(v,"fixed"), &attribute->fixed);
 
-  infoStreamPrint(LOG_DEBUG, 0, "Boolean %s(%sstart=%s%s, fixed=%s)", findHashStringString(v,"name"), attribute->useStart?"":"{", attribute->start?"true":"false", attribute->useStart?"":"}", attribute->fixed?"true":"false");
+  infoStreamPrint(LOG_DEBUG, 0, "Boolean %s(start=%s, fixed=%s)", findHashStringString(v,"name"), attribute->start?"true":"false", attribute->fixed?"true":"false");
 }
 
 static void read_var_attribute_string(omc_ScalarVariable *v, STRING_ATTRIBUTE *attribute)
 {
   const char *start = NULL;
-  read_value_bool(findHashStringString(v,"useStart"), &attribute->useStart);
   read_value_string(findHashStringStringEmpty(v,"start"), &start);
   attribute->start = mmc_mk_scon_persist(start);
 
-  infoStreamPrint(LOG_DEBUG, 0, "String %s(%sstart=%s%s)", findHashStringString(v,"name"), attribute->useStart?"":"{", attribute->start, attribute->useStart?"":"}");
+  infoStreamPrint(LOG_DEBUG, 0, "String %s(start=%s)", findHashStringString(v,"name"), MMC_STRINGDATA(attribute->start));
 }
 
 /* \brief
@@ -425,10 +424,11 @@ void read_input_xml(MODEL_DATA* modelData,
   const char *filename, *guid, *override, *overrideFile;
   FILE* file = NULL;
   XML_Parser parser = NULL;
-  hash_string_long *mapAlias = NULL, *mapAliasParam = NULL;
+  hash_string_long *mapAlias = NULL, *mapAliasParam = NULL, *mapAliasSen = NULL;
   long *it, *itParam;
   mmc_sint_t i;
   int inputIndex = 0;
+  int k = 0;
 
   modelica_integer nxchk, nychk, npchk;
   modelica_integer nyintchk, npintchk;
@@ -604,18 +604,28 @@ void read_input_xml(MODEL_DATA* modelData,
     { \
       out[j].filterOutput = 1; \
     } \
-    else if (!omc_flag[FLAG_EMIT_PROTECTED] && 0 == strcmp(findHashStringString(v, "isProtected"), "true")) \
+    else if (!omc_flag[FLAG_EMIT_PROTECTED] && 0 == strcmp(findHashStringString(v, "isProtected"), "true") && 0 == strcmp(findHashStringString(v, "hideResult"), "true")) \
     { \
       infoStreamPrint(LOG_DEBUG, 0, "filtering protected variable %s", info->name); \
       out[j].filterOutput = 1; \
     } \
-    else if (!omc_flag[FLAG_IGNORE_HIDERESULT] && 0 == strcmp(findHashStringString(v, "hideResult"), "true")) \
+    else if (!omc_flag[FLAG_IGNORE_HIDERESULT] && 0 == strcmp(findHashStringString(v, "hideResult"), "true") && 0 == strcmp(findHashStringString(v, "isProtected"), "false")) \
     { \
       infoStreamPrint(LOG_DEBUG, 0, "filtering variable %s due to HideResult annotation", info->name); \
       out[j].filterOutput = 1; \
     } \
     addHashStringLong(&mapAlias, info->name, j); /* create a mapping for Alias variable to get the correct index */ \
     debugStreamPrint(LOG_DEBUG, 0, "real %s: mapAlias[%s] = %ld", debugName, info->name, (long) j); \
+    if (omc_flag[FLAG_IDAS] && 0 == strcmp(debugName, "real sensitivities")) \
+    { \
+      if (0 == strcmp(findHashStringString(v, "isValueChangeable"), "true")) \
+      { \
+        long *it = findHashStringLongPtr(mapAliasParam, info->name); \
+        simulationInfo->sensitivityParList[k] = *it; \
+        infoStreamPrint(LOG_SOLVER, 0, "%d. sensitivity parameter %s at index %d", k, info->name, simulationInfo->sensitivityParList[k]); \
+        k++; \
+      } \
+    } \
   } \
   messageClose(LOG_DEBUG);
 
@@ -632,6 +642,11 @@ void read_input_xml(MODEL_DATA* modelData,
   READ_VARIABLES(modelData->booleanParameterData,mi.bPar,BOOLEAN_ATTRIBUTE,read_var_attribute_bool,"boolean parameters",0,modelData->nParametersBoolean,mapAliasParam);
   READ_VARIABLES(modelData->stringParameterData,mi.sPar,STRING_ATTRIBUTE,read_var_attribute_string,"string parameters",0,modelData->nParametersString,mapAliasParam);
 
+  if (omc_flag[FLAG_IDAS])
+  {
+    READ_VARIABLES(modelData->realSensitivityData,mi.rSen,REAL_ATTRIBUTE,read_var_attribute_real,"real sensitivities",0, modelData->nSensitivityVars,mapAliasSen);
+  }
+
   /*
    * real all alias vars
    */
@@ -649,8 +664,19 @@ void read_input_xml(MODEL_DATA* modelData,
     }
     infoStreamPrint(LOG_DEBUG, 0, "read for %s negated %d from setup file", modelData->realAlias[i].info.name, modelData->realAlias[i].negate);
 
-    /* filter internal variables */
-    if(modelData->realAlias[i].info.name[0] == '$') {
+    if(modelData->realAlias[i].info.name[0] == '$')
+    {
+      /* filter internal variables */
+      modelData->realAlias[i].filterOutput = 1;
+    }
+    else if (!omc_flag[FLAG_EMIT_PROTECTED] && 0 == strcmp(findHashStringString(*findHashLongVar(mi.rAli,i), "isProtected"), "true") && 0 == strcmp(findHashStringString(*findHashLongVar(mi.rAli,i), "hideResult"), "true"))
+    {
+      infoStreamPrint(LOG_DEBUG, 0, "filtering protected variable %s", modelData->realAlias[i].info.name);
+      modelData->realAlias[i].filterOutput = 1;
+    }
+    else if (!omc_flag[FLAG_IGNORE_HIDERESULT] && 0 == strcmp(findHashStringString(*findHashLongVar(mi.rAli,i), "hideResult"), "true") && 0 == strcmp(findHashStringString(*findHashLongVar(mi.rAli,i), "isProtected"), "false"))
+    {
+      infoStreamPrint(LOG_DEBUG, 0, "filtering variable %s due to HideResult annotation", modelData->realAlias[i].info.name);
       modelData->realAlias[i].filterOutput = 1;
     }
 
@@ -696,7 +722,18 @@ void read_input_xml(MODEL_DATA* modelData,
     infoStreamPrint(LOG_DEBUG, 0, "read for %s negated %d from setup file",modelData->integerAlias[i].info.name,modelData->integerAlias[i].negate);
 
     /* filter internal variables */
-    if(modelData->integerAlias[i].info.name[0] == '$') {
+    if(modelData->integerAlias[i].info.name[0] == '$')
+    {
+      modelData->integerAlias[i].filterOutput = 1;
+    }
+    else if (!omc_flag[FLAG_EMIT_PROTECTED] && 0 == strcmp(findHashStringString(*findHashLongVar(mi.iAli,i), "isProtected"), "true") && 0 == strcmp(findHashStringString(*findHashLongVar(mi.iAli,i), "hideResult"), "true"))
+    {
+      infoStreamPrint(LOG_DEBUG, 0, "filtering protected variable %s", modelData->integerAlias[i].info.name);
+      modelData->integerAlias[i].filterOutput = 1;
+    }
+    else if (!omc_flag[FLAG_IGNORE_HIDERESULT] && 0 == strcmp(findHashStringString(*findHashLongVar(mi.iAli,i), "hideResult"), "true") && 0 == strcmp(findHashStringString(*findHashLongVar(mi.iAli,i), "isProtected"), "false"))
+    {
+      infoStreamPrint(LOG_DEBUG, 0, "filtering variable %s due to HideResult annotation", modelData->integerAlias[i].info.name);
       modelData->integerAlias[i].filterOutput = 1;
     }
     read_value_string(findHashStringString(*findHashLongVar(mi.iAli,i),"aliasVariable"), &aliasTmp);
@@ -738,8 +775,19 @@ void read_input_xml(MODEL_DATA* modelData,
 
     infoStreamPrint(LOG_DEBUG, 0, "read for %s negated %d from setup file", modelData->booleanAlias[i].info.name, modelData->booleanAlias[i].negate);
 
-    /* filter internal variables */
-    if(modelData->booleanAlias[i].info.name[0] == '$') {
+    if(modelData->booleanAlias[i].info.name[0] == '$')
+    {
+      /* filter internal variables */
+      modelData->booleanAlias[i].filterOutput = 1;
+    }
+    else if (!omc_flag[FLAG_EMIT_PROTECTED] && 0 == strcmp(findHashStringString(*findHashLongVar(mi.bAli,i), "isProtected"), "true") && 0 == strcmp(findHashStringString(*findHashLongVar(mi.bAli,i), "hideResult"), "true"))
+    {
+      infoStreamPrint(LOG_DEBUG, 0, "filtering protected variable %s", modelData->booleanAlias[i].info.name);
+      modelData->booleanAlias[i].filterOutput = 1;
+    }
+    else if (!omc_flag[FLAG_IGNORE_HIDERESULT] && 0 == strcmp(findHashStringString(*findHashLongVar(mi.bAli,i), "hideResult"), "true") && 0 == strcmp(findHashStringString(*findHashLongVar(mi.bAli,i), "isProtected"), "false"))
+    {
+      infoStreamPrint(LOG_DEBUG, 0, "filtering variable %s due to HideResult annotation", modelData->booleanAlias[i].info.name);
       modelData->booleanAlias[i].filterOutput = 1;
     }
     read_value_string(findHashStringString(*findHashLongVar(mi.bAli,i),"aliasVariable"), &aliasTmp);
@@ -780,8 +828,19 @@ void read_input_xml(MODEL_DATA* modelData,
     }
     infoStreamPrint(LOG_DEBUG, 0, "read for %s negated %d from setup file", modelData->stringAlias[i].info.name, modelData->stringAlias[i].negate);
 
-    /* filter internal variables */
-    if(modelData->stringAlias[i].info.name[0] == '$') {
+    if(modelData->stringAlias[i].info.name[0] == '$')
+    {
+      /* filter internal variables */
+      modelData->stringAlias[i].filterOutput = 1;
+    }
+    else if (!omc_flag[FLAG_EMIT_PROTECTED] && 0 == strcmp(findHashStringString(*findHashLongVar(mi.sAli,i), "isProtected"), "true") && 0 == strcmp(findHashStringString(*findHashLongVar(mi.sAli,i), "hideResult"), "true"))
+    {
+      infoStreamPrint(LOG_DEBUG, 0, "filtering protected variable %s", modelData->stringAlias[i].info.name);
+      modelData->stringAlias[i].filterOutput = 1;
+    }
+    else if (!omc_flag[FLAG_IGNORE_HIDERESULT] && 0 == strcmp(findHashStringString(*findHashLongVar(mi.sAli,i), "hideResult"), "true") && 0 == strcmp(findHashStringString(*findHashLongVar(mi.sAli,i), "isProtected"), "false"))
+    {
+      infoStreamPrint(LOG_DEBUG, 0, "filtering variable %s due to HideResult annotation", modelData->stringAlias[i].info.name);
       modelData->stringAlias[i].filterOutput = 1;
     }
 
@@ -970,8 +1029,6 @@ void doOverride(omc_ModelInput *mi, MODEL_DATA *modelData, const char *override,
       addHashStringString(&mOverrides, p, value);
       addHashStringLong(&mOverridesUses, p, OMC_OVERRIDE_UNUSED);
 
-      infoStreamPrint(LOG_SOLVER, 0, "override %s = %s", p, value);
-
       // move to next
       p = strtok(NULL, "!");
     }
@@ -985,55 +1042,64 @@ void doOverride(omc_ModelInput *mi, MODEL_DATA *modelData, const char *override,
       }
     }
 
-    #define CHECK_OVERRIDE(v) \
+    #define CHECK_OVERRIDE(v,b) \
       if (findHashStringStringNull(mOverrides, findHashStringString(*findHashLongVar(mi->v,i),"name"))) { \
-        addHashStringString(findHashLongVar(mi->v,i), "start", getOverrideValue(mOverrides, &mOverridesUses, findHashStringString(*findHashLongVar(mi->v,i),"name"))); \
+        if (0 == strcmp(findHashStringString(*findHashLongVar(mi->v,i), "isValueChangeable"), "true")){ \
+          infoStreamPrint(LOG_SOLVER, 0, "override %s = %s", findHashStringString(*findHashLongVar(mi->v,i),"name"), getOverrideValue(mOverrides, &mOverridesUses, findHashStringString(*findHashLongVar(mi->v,i),"name"))); \
+          if (b && fabs(atof(getOverrideValue(mOverrides, &mOverridesUses, findHashStringString(*findHashLongVar(mi->v,i),"name")))) < 1e-6) \
+            warningStreamPrint(LOG_STDOUT, 0, "You are overriding %s with a small value or zero.\nThis could lead to numerically dirty solutions or divisions by zero if not tearingStrictness=veryStrict.", findHashStringString(*findHashLongVar(mi->v,i),"name")); \
+          addHashStringString(findHashLongVar(mi->v,i), "start", getOverrideValue(mOverrides, &mOverridesUses, findHashStringString(*findHashLongVar(mi->v,i),"name"))); \
+        } \
+        else{ \
+          addHashStringLong(&mOverridesUses, findHashStringString(*findHashLongVar(mi->v,i),"name"), OMC_OVERRIDE_USED); \
+          warningStreamPrint(LOG_STDOUT, 0, "It is not possible to override the following quantity: %s\nIt seems to be structural, final, protected or evaluated or has a non-constant binding.", findHashStringString(*findHashLongVar(mi->v,i),"name")); \
+        } \
       }
 
     // override all found!
     for(i=0; i<modelData->nStates; i++) {
-      CHECK_OVERRIDE(rSta);
-      CHECK_OVERRIDE(rDer);
+      CHECK_OVERRIDE(rSta,0);
+      CHECK_OVERRIDE(rDer,0);
     }
     for(i=0; i<(modelData->nVariablesReal - 2*modelData->nStates); i++) {
-      CHECK_OVERRIDE(rAlg);
+      CHECK_OVERRIDE(rAlg,0);
     }
     for(i=0; i<modelData->nVariablesInteger; i++) {
-      CHECK_OVERRIDE(iAlg);
+      CHECK_OVERRIDE(iAlg,0);
     }
     for(i=0; i<modelData->nVariablesBoolean; i++) {
-      CHECK_OVERRIDE(bAlg);
+      CHECK_OVERRIDE(bAlg,0);
     }
     for(i=0; i<modelData->nVariablesString; i++) {
-      CHECK_OVERRIDE(sAlg);
+      CHECK_OVERRIDE(sAlg,0);
     }
     for(i=0; i<modelData->nParametersReal; i++) {
       // TODO: only allow to override primary parameters
-      CHECK_OVERRIDE(rPar);
+      CHECK_OVERRIDE(rPar,1);
     }
     for(i=0; i<modelData->nParametersInteger; i++) {
       // TODO: only allow to override primary parameters
-      CHECK_OVERRIDE(iPar);
+      CHECK_OVERRIDE(iPar,1);
     }
     for(i=0; i<modelData->nParametersBoolean; i++) {
       // TODO: only allow to override primary parameters
-      CHECK_OVERRIDE(bPar);
+      CHECK_OVERRIDE(bPar,0);
     }
     for(i=0; i<modelData->nParametersString; i++) {
       // TODO: only allow to override primary parameters
-      CHECK_OVERRIDE(sPar);
+      CHECK_OVERRIDE(sPar,0);
     }
     for(i=0; i<modelData->nAliasReal; i++) {
-      CHECK_OVERRIDE(rAli);
+      CHECK_OVERRIDE(rAli,0);
     }
     for(i=0; i<modelData->nAliasInteger; i++) {
-      CHECK_OVERRIDE(iAli);
+      CHECK_OVERRIDE(iAli,0);
     }
     for(i=0; i<modelData->nAliasBoolean; i++) {
-      CHECK_OVERRIDE(bAli);
+      CHECK_OVERRIDE(bAli,0);
     }
     for(i=0; i<modelData->nAliasString; i++) {
-      CHECK_OVERRIDE(sAli);
+      CHECK_OVERRIDE(sAli,0);
     }
 
     // give a warning if an override is not used #3204
