@@ -75,6 +75,9 @@ allocateUmfPackData(int n_row, int n_col, int nz, void** voiddata)
   data->Ax = (double*) calloc(nz,sizeof(double));
   data->work = (double*) calloc(n_col,sizeof(double));
 
+  data->Wi = (int*) malloc(n_row * sizeof(int));
+  data->W = (double*) malloc(5*n_row * sizeof(double));
+
   data->numberSolving=0;
   umfpack_di_defaults(data->control);
 
@@ -106,6 +109,9 @@ freeUmfPackData(void **voiddata)
   free(data->Ax);
   free(data->work);
 
+  free(data->Wi);
+  free(data->W);
+
   if(data->symbolic)
     umfpack_di_free_symbolic (&data->symbolic);
   if(data->numeric)
@@ -144,11 +150,8 @@ int getAnalyticalJacobianUmfPack(DATA* data, threadData_t *threadData, int sysNu
     {
       if(data->simulationInfo->analyticJacobians[index].seedVars[j] == 1)
       {
-        if(j==0)
-          ii = 0;
-        else
-          ii = data->simulationInfo->analyticJacobians[index].sparsePattern.leadindex[j-1];
-        while(ii < data->simulationInfo->analyticJacobians[index].sparsePattern.leadindex[j])
+        ii = data->simulationInfo->analyticJacobians[index].sparsePattern.leadindex[j];
+        while(ii < data->simulationInfo->analyticJacobians[index].sparsePattern.leadindex[j+1])
         {
           l  = data->simulationInfo->analyticJacobians[index].sparsePattern.index[ii];
           /* infoStreamPrint(LOG_LS_V, 0, "set on Matrix A (%d, %d)(%d) = %f", i, l, nth, -data->simulationInfo->analyticJacobians[index].resultVars[l]); */
@@ -193,6 +196,8 @@ solveUmfPack(DATA *data, threadData_t *threadData, int sysNumber)
   DATA_UMFPACK* solverData = (DATA_UMFPACK*)systemData->solverData;
 
   int i, j, status = UMFPACK_OK, success = 0, ni=0, n = systemData->size, eqSystemNumber = systemData->equationIndex, indexes[2] = {1,eqSystemNumber};
+  int casualTearingSet = systemData->strictTearingFunctionCall != NULL;
+  double tmpJacEvalTime;
 
   infoStreamPrintWithEquationIndexes(LOG_LS, 0, indexes, "Start solving Linear System %d (size %d) at time %g with UMFPACK Solver",
    eqSystemNumber, (int) systemData->size,
@@ -231,25 +236,22 @@ solveUmfPack(DATA *data, threadData_t *threadData, int sysNumber)
     memcpy(solverData->work, systemData->x, sizeof(double)*solverData->n_row);
     wrapper_fvec_umfpack(solverData->work, systemData->b, dataAndThreadData, sysNumber);
   }
-
-  infoStreamPrint(LOG_LS, 0, "###  %f  time to set Matrix A and vector b.", rt_ext_tp_tock(&(solverData->timeClock)));
+  tmpJacEvalTime = rt_ext_tp_tock(&(solverData->timeClock));
+  systemData->jacobianTime += tmpJacEvalTime;
+  infoStreamPrint(LOG_LS_V, 0, "###  %f  time to set Matrix A and vector b.", tmpJacEvalTime);
 
   if (ACTIVE_STREAM(LOG_LS_V))
   {
-    if (ACTIVE_STREAM(LOG_LS_V))
-    {
-      infoStreamPrint(LOG_LS_V, 1, "Old solution x:");
-      for(i = 0; i < solverData->n_row; ++i)
-        infoStreamPrint(LOG_LS_V, 0, "[%d] %s = %g", i+1, modelInfoGetEquation(&data->modelData->modelDataXml,eqSystemNumber).vars[i], systemData->x[i]);
+    infoStreamPrint(LOG_LS_V, 1, "Old solution x:");
+    for(i = 0; i < solverData->n_row; ++i)
+      infoStreamPrint(LOG_LS_V, 0, "[%d] %s = %g", i+1, modelInfoGetEquation(&data->modelData->modelDataXml,eqSystemNumber).vars[i], systemData->x[i]);
+    messageClose(LOG_LS_V);
 
-      messageClose(LOG_LS_V);
-    }
     infoStreamPrint(LOG_LS_V, 1, "Matrix A n_rows = %d", solverData->n_row);
     for (i=0; i<solverData->n_row; i++){
       infoStreamPrint(LOG_LS_V, 0, "%d. Ap => %d -> %d", i, solverData->Ap[i], solverData->Ap[i+1]);
       for (j=solverData->Ap[i]; j<solverData->Ap[i+1]; j++){
         infoStreamPrint(LOG_LS_V, 0, "A[%d,%d] = %f", i, solverData->Ai[j], solverData->Ax[j]);
-
       }
     }
     messageClose(LOG_LS_V);
@@ -267,32 +269,32 @@ solveUmfPack(DATA *data, threadData_t *threadData, int sysNumber)
 
   /* compute the LU factorization of A */
   if (0 == status){
+    umfpack_di_free_numeric(&(solverData->numeric));
     status = umfpack_di_numeric(solverData->Ap, solverData->Ai, solverData->Ax, solverData->symbolic, &(solverData->numeric), solverData->control, solverData->info);
   }
 
   if (0 == status){
     if (1 == systemData->method){
-      status = umfpack_di_solve(UMFPACK_A, solverData->Ap, solverData->Ai, solverData->Ax, systemData->x, systemData->b, solverData->numeric, solverData->control, solverData->info);
+      status = umfpack_di_wsolve(UMFPACK_A, solverData->Ap, solverData->Ai, solverData->Ax, systemData->x, systemData->b, solverData->numeric, solverData->control, solverData->info, solverData->Wi, solverData->W);
     } else {
-      status = umfpack_di_solve(UMFPACK_Aat, solverData->Ap, solverData->Ai, solverData->Ax, systemData->x, systemData->b, solverData->numeric, solverData->control, solverData->info);
+      status = umfpack_di_wsolve(UMFPACK_Aat, solverData->Ap, solverData->Ai, solverData->Ax, systemData->x, systemData->b, solverData->numeric, solverData->control, solverData->info, solverData->Wi, solverData->W);
     }
   }
 
   if (status == UMFPACK_OK){
     success = 1;
   }
-  else if (status == UMFPACK_WARNING_singular_matrix)
+  else if ((status == UMFPACK_WARNING_singular_matrix) && (casualTearingSet==0))
   {
     if (!solveSingularSystem(systemData))
     {
       success = 1;
     }
   }
-  infoStreamPrint(LOG_LS, 0, "Solve System: %f", rt_ext_tp_tock(&(solverData->timeClock)));
+  infoStreamPrint(LOG_LS_V, 0, "Solve System: %f", rt_ext_tp_tock(&(solverData->timeClock)));
 
   /* print solution */
   if (1 == success){
-
     if (1 == systemData->method){
       /* take the solution */
       for(i = 0; i < solverData->n_row; ++i)
@@ -403,9 +405,9 @@ int solveSingularSystem(LINEAR_SYSTEM_DATA* systemData)
   }
 
   /* solve L * y = P * R * b  <=>  P^T * L * y = R * b */
-  status = umfpack_di_solve(UMFPACK_Pt_L, solverData->Ap, solverData->Ai,
+  status = umfpack_di_wsolve(UMFPACK_Pt_L, solverData->Ap, solverData->Ai,
       solverData->Ax, y, b, solverData->numeric, solverData->control,
-      solverData->info);
+      solverData->info, solverData->Wi, solverData->W);
 
   switch (status)
   {
