@@ -29,13 +29,19 @@
  */
 
 /*! \file radau.c
- * author: team Bielefeld :)
+ * author: Ru(n)ge, wbraun
+ * description: This file contains implicit Runge-Kutta methods of different order,
+ * based on Radau IIA and Lobatto IIA methods. The method with order one is
+ * corresponding to the implicit euler method. Further orders 2 to 6.
  */
 
 #include <string.h>
+#include <math.h>
 
 #include "radau.h"
 #include "external_input.h"
+
+#include "simulation/options.h"
 #ifdef WITH_SUNDIALS
 
 #include <kinsol/kinsol.h>
@@ -55,10 +61,9 @@ extern int KINSetInfoHandlerFn(void *kinmem, KINInfoHandlerFn ihfun, void *ih_da
 }
 #endif
 
-static int allocateNlpOde(KINODE *kinOde);
+static int allocateNlpOde(KINODE *kinOde, int order);
 static int allocateKINSOLODE(KINODE *kinOde);
 
-
 static void kinsol_errorHandler(int error_code, const char* module, const char* function, char* msg, void* user_data);
 
 static void kinsol_infoHandler(const char* module, const char* function, char* msg, void* user_data);
@@ -81,16 +86,17 @@ static int lobatto2Res(N_Vector z, N_Vector f, void* user_data);
 static int lobatto4Res(N_Vector z, N_Vector f, void* user_data);
 static int lobatto6Res(N_Vector z, N_Vector f, void* user_data);
 
-int allocateKinOde(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo, int flag, int N)
+int allocateKinOde(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo, int order)
 {
+  int i;
   KINODE *kinOde = (KINODE*) solverInfo->solverData;
   kinOde->kData = (KDATAODE*) malloc(sizeof(KDATAODE));
   kinOde->nlp = (NLPODE*) malloc(sizeof(NLPODE));
-  kinOde->N = N;
-  kinOde->flag = flag;
+  kinOde->order = order;
+  kinOde->N = ceil((double)order/2.0);
   kinOde->data = data;
   kinOde->threadData = threadData;
-  allocateNlpOde(kinOde);
+  allocateNlpOde(kinOde, order);
   allocateKINSOLODE(kinOde);
   kinOde->solverInfo = solverInfo;
   kinOde->kData->mset = 50;
@@ -104,69 +110,109 @@ int allocateKinOde(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo
   if (ACTIVE_STREAM(LOG_SOLVER)) {
     KINSetPrintLevel(kinOde->kData->kmem,2);
   }
-  //KINSetEtaForm(kinOde->kData->kmem, KIN_ETACHOICE2);
+  /* KINSetEtaForm(kinOde->kData->kmem, KIN_ETACHOICE2); */
   KINSetMaxSetupCalls(kinOde->kData->kmem, kinOde->kData->mset);
-  kinOde->nlp->currentStep = &kinOde->solverInfo->currentStepSize;
+
   KINSetErrHandlerFn(kinOde->kData->kmem, kinsol_errorHandler, NULL);
   KINSetInfoHandlerFn(kinOde->kData->kmem, kinsol_infoHandler, NULL);
-  switch(kinOde->flag)
+  switch(kinOde->order)
   {
-    case S_RADAU5:
+    case 5:
       KINInit(kinOde->kData->kmem, radau5Res, kinOde->kData->x);
       break;
-    case S_RADAU3:
+    case 3:
       KINInit(kinOde->kData->kmem, radau3Res, kinOde->kData->x);
       break;
-    case S_RADAU1:
+    case 1:
       KINInit(kinOde->kData->kmem, radau1Res, kinOde->kData->x);
       break;
-    case S_LOBATTO2:
+    case 2:
       KINInit(kinOde->kData->kmem, lobatto2Res, kinOde->kData->x);
       break;
-    case S_LOBATTO4:
+    case 4:
       KINInit(kinOde->kData->kmem, lobatto4Res, kinOde->kData->x);
       break;
-    case S_LOBATTO6:
+    case 6:
       KINInit(kinOde->kData->kmem, lobatto6Res, kinOde->kData->x);
       break;
 
     default:
       assert(0);
   }
-  /* Call KINDense to specify the linear solver */
- /*KINSpbcg*/
-  if(kinOde->nlp->nStates < 10)
-    KINSpgmr(kinOde->kData->kmem, N*kinOde->nlp->nStates+1);
+  /* if FLAG_IMPRK_LS is set, choose ida linear solver method */
+  if (omc_flag[FLAG_IMPRK_LS])
+  {
+    for(i=1; i< IMPRK_LS_MAX;i++)
+    {
+      if (!strcmp((const char*)omc_flagValue[FLAG_IMPRK_LS], IMPRK_LS_METHOD[i])){
+        kinOde->lsMethod = (int)i;
+        break;
+      }
+    }
+    if (kinOde->lsMethod == IMPRK_LS_UNKNOWN)
+    {
+      if (ACTIVE_WARNING_STREAM(LOG_SOLVER))
+      {
+        warningStreamPrint(LOG_SOLVER, 1, "unrecognized linear solver method %s, current options are:", (const char*)omc_flagValue[FLAG_IMPRK_LS]);
+        for(i=1; i < IMPRK_LS_MAX; ++i)
+        {
+          warningStreamPrint(LOG_SOLVER, 0, "%-15s [%s]", IMPRK_LS_METHOD[i], IMPRK_LS_METHOD_DESC[i]);
+        }
+        messageClose(LOG_SOLVER);
+      }
+      throwStreamPrint(threadData,"unrecognized linear solver method %s", (const char*)omc_flagValue[FLAG_IMPRK_LS]);
+    }
+  }
   else
-    KINSpbcg(kinOde->kData->kmem, N*kinOde->nlp->nStates+1);
+  {
+    kinOde->lsMethod = IMPRK_LS_ITERATIVE;
+  }
+
   kinOde->kData->glstr = KIN_LINESEARCH;
 
+  switch (kinOde->lsMethod){
+    case IMPRK_LS_ITERATIVE:
+      /*KINSpbcg*/
+      if(kinOde->nlp->nStates < 10)
+        KINSpgmr(kinOde->kData->kmem, kinOde->N*kinOde->nlp->nStates+1);
+      else
+        KINSpbcg(kinOde->kData->kmem, kinOde->N*kinOde->nlp->nStates+1);
+      break;
+    case IMPRK_LS_DENSE:
+      KINDense(kinOde->kData->kmem, kinOde->N*kinOde->nlp->nStates);
+      break;
+    default:
+      throwStreamPrint(threadData,"unrecognized linear solver method %s", (const char*)omc_flagValue[FLAG_IMPRK_LS]);
+    break;
+  }
+  KINSetNoInitSetup(kinOde->kData->kmem, FALSE);
+
   return 0;
 }
 
-static int allocateNlpOde(KINODE *kinOde)
+static int allocateNlpOde(KINODE *kinOde, int order)
 {
   NLPODE * nlp = (NLPODE*) kinOde->nlp;
   nlp->nStates = kinOde->data->modelData->nStates;
 
-  switch(kinOde->flag)
+  switch(order)
   {
-  case S_RADAU5:
+  case 5:
     radau5Coeff(kinOde);
     break;
-  case S_RADAU3:
+  case 3:
     radau3Coeff(kinOde);
     break;
-  case S_RADAU1:
+  case 1:
     radau1Coeff(kinOde);
     break;
-  case S_LOBATTO2:
+  case 2:
     radau1Coeff(kinOde); /* TODO: Is this right? */
     break;
-  case S_LOBATTO4:
+  case 4:
     lobatto4Coeff(kinOde);
     break;
-  case S_LOBATTO6:
+  case 6:
     lobatto6Coeff(kinOde);
     break;
   default:
@@ -178,16 +224,233 @@ static int allocateNlpOde(KINODE *kinOde)
 }
 
 static void kinsol_errorHandler(int error_code, const char* module, const char* function, char* msg, void* user_data)
-  {
-      warningStreamPrint(LOG_SOLVER, 0, "[module] %s | [function] %s | [error_code] %d", module, function, error_code);
-      if (msg) warningStreamPrint(LOG_SOLVER, 0, "%s", msg);
-  }
+{
+    warningStreamPrint(LOG_SOLVER, 0, "[module] %s | [function] %s | [error_code] %d", module, function, error_code);
+    if (msg) warningStreamPrint(LOG_SOLVER, 0, "%s", msg);
+}
 
 static void kinsol_infoHandler(const char* module, const char* function, char* msg, void* user_data)
-  {
-    infoStreamPrint(LOG_SOLVER, 0, " %s: %s ", module, function);
-    if (msg) infoStreamPrint(LOG_SOLVER, 0, "%s", msg);
+{
+  infoStreamPrint(LOG_SOLVER, 0, " %s: %s ", module, function);
+  if (msg) infoStreamPrint(LOG_SOLVER, 0, "%s", msg);
+}
+
+static int allocateKINSOLODE(KINODE *kinOde)
+{
+  int m;
+  DATA *data = kinOde->data;
+  int n = data->modelData->nStates;
+  int i, j, k;
+  double* c;
+  KDATAODE * kData = (kinOde)->kData;
+  m =  kinOde->N*n;
+  kData->x = N_VNew_Serial(m);
+  kData->sVars = N_VNew_Serial(m);
+  kData->sEqns = N_VNew_Serial(m);
+  kData->c = N_VNew_Serial(m);
+  kData->kmem = KINCreate();
+  c = NV_DATA_S(kData->c);
+
+  for(j=0, k=0; j< kinOde->N; ++j)
+    for(i=0; i<n; ++i, ++k)
+      c[k] = 0;
+
+  KINSetUserData(kinOde->kData->kmem, (void*) kinOde);
+  KINSetConstraints(kinOde->kData->kmem, kData->c);
+  return 0;
+}
+
+int freeKinOde(DATA* data, SOLVER_INFO* solverInfo)
+{
+  KINODE *kinOde = (KINODE*) solverInfo->solverData;
+  freeImOde((void*) kinOde->nlp, kinOde->N);
+  freeKinsol((void*) kinOde->kData);
+  free(kinOde);
+  return 0;
+}
+
+static int freeImOde(void *nlpode, int N)
+{
+  int i;
+  NLPODE *nlp = (NLPODE*) nlpode;
+  free(nlp->min);
+  free(nlp->max);
+  free(nlp->s);
+
+  for(i=0; i<N; i++) {
+    free(nlp->c[i]);
   }
+  free(nlp->c);
+
+  free(nlp->a);
+  return 0;
+}
+
+static int freeKinsol(void * kOde)
+{
+  KDATAODE *kData = (KDATAODE*) kOde;
+  N_VDestroy_Serial(kData->x);
+  N_VDestroy_Serial(kData->sVars);
+  N_VDestroy_Serial(kData->sEqns);
+  N_VDestroy_Serial(kData->c);
+  KINFree(&kData->kmem);
+  return 0;
+}
+
+static int initKinsol(KINODE *kinOde)
+{
+  int i,j,k,n;
+  double *scal_eq, *scal_var, *x, *f2;
+  long double tmp, h, hf, hf_min;
+  DATA *data;
+  NLPODE *nlp;
+  KDATAODE * kData;
+
+  n = kinOde->nlp->nStates;
+  data= kinOde->data;
+  kData = kinOde->kData;
+  x = NV_DATA_S(kData->x);
+  nlp = kinOde->nlp;
+
+  nlp->currentStep = kinOde->solverInfo->currentStepSize;
+  f2 = data->localData[2]->realVars + n;
+  nlp->dt = nlp->currentStep;
+  nlp->derx = data->localData[0]->realVars + n;
+  nlp->x0 = data->localData[1]->realVars;
+  nlp->f0 = data->localData[1]->realVars + n;
+  nlp->t0 = data->localData[1]->timeValue;
+
+
+  scal_var = NV_DATA_S(kData->sVars);
+  scal_eq = NV_DATA_S(kData->sEqns);
+
+  hf_min = 1e-6;
+  for(j=0, k=0; j<kinOde->N; ++j)
+  {
+    for(i=0;i<n;++i,++k)
+    {
+      hf = 0.5*nlp->dt*nlp->a[j]*(3*nlp->f0[i]-f2[i]);
+      if(fabsl(hf) < hf_min) hf_min = fabsl(hf);
+      x[k] = (nlp->x0[i] + hf);
+      tmp = fabs(x[k] + nlp->x0[i]) + 1e-12;
+      tmp = (tmp < 1e-9) ? nlp->s[i] : 2.0/tmp;
+      scal_var[k] = tmp + 1e-9;
+      scal_eq[k] = 1.0/(scal_var[k])+ 1e-12;
+    }
+  }
+  KINSetMaxNewtonStep(kinOde->kData->kmem, hf_min);
+  return 0;
+}
+
+int kinsolOde(SOLVER_INFO* solverInfo)
+{
+  KINODE *kinOde = (KINODE*) solverInfo->solverData;
+  KDATAODE *kData = kinOde->kData;
+  int flag, dense=0, try_again = 1;
+  int retries = 0;
+
+  infoStreamPrint(LOG_SOLVER, 1, "##IMPRK## new step from %.15g to %.15g", solverInfo->currentTime, solverInfo->currentTime + solverInfo->currentStepSize);
+  initKinsol(kinOde);
+
+  do
+  {
+    kData->error_code = KINSol(kData->kmem,           /* KINSol memory block */
+                               kData->x,              /* initial guess on input; solution vector */
+                               kData->glstr,          /* global strategy choice */
+                               kData->sVars,          /* scaling vector, for the variable cc */
+                               kData->sEqns           /* scaling vector, for the residual eqns */
+                               );
+
+    switch(kinOde->lsMethod){
+      retries++;
+      case IMPRK_LS_ITERATIVE:
+        if(retries == 0)
+        {
+         KINDense(kinOde->kData->kmem, kinOde->N*kinOde->nlp->nStates);
+         dense=1;
+         warningStreamPrint(LOG_SOLVER,0,"Restart Kinsol: change linear solver to KINDense.");
+        }
+        else if(retries == 1)
+        {
+          KINSptfqmr(kinOde->kData->kmem, kinOde->N*kinOde->nlp->nStates);
+          dense=0;
+          warningStreamPrint(LOG_SOLVER,0,"Restart Kinsol: change linear solver to KINSptfqmr.");
+        }
+        else if(retries == 2)
+        {
+          KINSpbcg(kinOde->kData->kmem, kinOde->N*kinOde->nlp->nStates);
+          warningStreamPrint(LOG_SOLVER,0,"Restart Kinsol: change linear solver to KINSpbcg.");
+        }
+        else
+        {
+          try_again= 0;
+        }
+        break;
+      case IMPRK_LS_DENSE:
+        dense=1;
+        if (retries== 1)
+        {
+          kinOde->kData->glstr = KIN_NONE;
+        }
+        else
+        {
+          try_again= 0;
+        }
+        break;
+      default:
+        kData->error_code = -42;
+        try_again = 0;
+        break;
+    }
+
+  }while(kData->error_code < 0 && try_again);
+
+  /* save stats */
+  /* steps */
+  solverInfo->solverStatsTmp[0] += 1;
+  /* functionODE evaluations */
+  {
+    long int tmp = 0;
+    flag = KINGetNumFuncEvals(kData->kmem, &tmp);
+    if (flag == KIN_SUCCESS)
+    {
+      solverInfo->solverStatsTmp[1] += tmp;
+    }
+  }
+
+  /* Jacobians evaluations */
+  {
+    long int tmp = 0;
+    if (dense)
+    {
+      flag = KINDlsGetNumJacEvals(kData->kmem, &tmp);
+    }
+    else
+    {
+      flag = KINSpilsGetNumJtimesEvals(kData->kmem, &tmp);
+    }
+    if (flag == KIN_SUCCESS)
+    {
+      solverInfo->solverStatsTmp[2] += tmp;
+    }
+  }
+
+  /* beta-condition failures evaluations */
+  {
+    long int tmp = 0;
+    flag = KINGetNumBetaCondFails(kData->kmem, &tmp);
+    if (flag == KIN_SUCCESS)
+    {
+      solverInfo->solverStatsTmp[4] += tmp;
+    }
+  }
+  infoStreamPrint(LOG_SOLVER, 0, "##IMPRK## Integration step finished .");
+  /* closing new step message */
+  messageClose(LOG_SOLVER);
+
+  return (kData->error_code<0) ? -1 : 0;
+}
+
 
 static int boundsVars(KINODE *kinOde)
 {
@@ -214,14 +477,13 @@ static int boundsVars(KINODE *kinOde)
 static int radau5Coeff(KINODE *kinOde)
 {
   int i;
-  const int N = 3;
   NLPODE * nlp = (NLPODE*) kinOde->nlp;
 
-  nlp->c = (long double**) malloc(N * sizeof(long double*));
-  for(i = 0; i < N; i++)
-    nlp->c[i] = (long double*) calloc(N+1, sizeof(long double));
+  nlp->c = (long double**) malloc(kinOde->N * sizeof(long double*));
+  for(i = 0; i < kinOde->N; i++)
+    nlp->c[i] = (long double*) calloc(kinOde->N+1, sizeof(long double));
 
-  nlp->a = (double*) malloc(N * sizeof(double));
+  nlp->a = (double*) malloc(kinOde->N * sizeof(double));
 
   nlp->c[0][0] = 4.1393876913398137178367408896470696703591369767880;
   nlp->c[0][1] = 3.2247448713915890490986420373529456959829737403284;
@@ -247,14 +509,13 @@ static int radau5Coeff(KINODE *kinOde)
 static int radau3Coeff(KINODE *kinOde)
 {
   int i;
-  const int N = 2;
   NLPODE * nlp = (NLPODE*) kinOde->nlp;
 
-  nlp->c = (long double**) malloc(N * sizeof(long double*));
-  for(i = 0; i < N; i++)
-    nlp->c[i] = (long double*) calloc(N+1, sizeof(long double));
+  nlp->c = (long double**) malloc(kinOde->N  * sizeof(long double*));
+  for(i = 0; i < kinOde->N; i++)
+    nlp->c[i] = (long double*) calloc(kinOde->N+1, sizeof(long double));
 
-  nlp->a = (double*) malloc(N * sizeof(double));
+  nlp->a = (double*) malloc(kinOde->N * sizeof(double));
 
   nlp->c[0][0] = 2.0;
   nlp->c[0][1] = 1.50;
@@ -294,15 +555,14 @@ static int lobatto4Coeff(KINODE *kinOde)
 static int lobatto6Coeff(KINODE *kinOde)
 {
   int i;
-  const int N = 3;
   NLPODE * nlp = (NLPODE*) kinOde->nlp;
 
 
-  nlp->c = (long double**) malloc(N * sizeof(long double*));
-  for(i = 0; i < N; ++i)
-    nlp->c[i] = (long double*) malloc((N+2)* sizeof(long double));
+  nlp->c = (long double**) malloc(kinOde->N * sizeof(long double*));
+  for(i = 0; i < kinOde->N; ++i)
+    nlp->c[i] = (long double*) malloc((kinOde->N+2)* sizeof(long double));
 
-  nlp->a = (double*) malloc(N * sizeof(double));
+  nlp->a = (double*) malloc(kinOde->N * sizeof(double));
 
   nlp->c[0][0] = 4.3013155617496424838955952368431696002490512113396;
   nlp->c[0][1] = 3.6180339887498948482045868343656381177203091798058;
@@ -328,110 +588,6 @@ static int lobatto6Coeff(KINODE *kinOde)
   return 0;
 }
 
-static int allocateKINSOLODE(KINODE *kinOde)
-{
-  int m;
-  DATA *data = kinOde->data;
-  int n = data->modelData->nStates;
-  int i, j, k;
-  double* c;
-  KDATAODE * kData = (kinOde)->kData;
-  m =  kinOde->N*n;
-  kData->x = N_VNew_Serial(m);
-  kData->sVars = N_VNew_Serial(m);
-  kData->sEqns = N_VNew_Serial(m);
-  kData->c = N_VNew_Serial(m);
-  kData->kmem = KINCreate();
-  c = NV_DATA_S(kData->c);
-
-  for(j=0, k=0; j< kinOde->N; ++j)
-    for(i=0; i<n; ++i, ++k)
-      c[k] = 0;
-
-  KINSetUserData(kinOde->kData->kmem, (void*) kinOde);
-  KINSetConstraints(kinOde->kData->kmem, kData->c);
-  return 0;
-}
-
-int freeKinOde(DATA* data, SOLVER_INFO* solverInfo, int N)
-{
-  KINODE *kinOde = (KINODE*) solverInfo->solverData;
-  freeImOde((void*) kinOde->nlp ,N);
-  freeKinsol((void*) kinOde->kData);
-  free(kinOde);
-  return 0;
-}
-
-static int freeImOde(void *nlpode, int N)
-{
-  int i;
-  NLPODE *nlp = (NLPODE*) nlpode;
-  free(nlp->min);
-  free(nlp->max);
-  free(nlp->s);
-
-  for(i=0; i<N; i++) {
-    free(nlp->c[i]);
-  }
-  free(nlp->c);
-
-  free(nlp->a);
-  return 0;
-}
-
-static int freeKinsol(void * kOde)
-{
-  KDATAODE *kData = (KDATAODE*) kOde;
-  N_VDestroy_Serial(kData->x);
-  N_VDestroy_Serial(kData->sVars);
-  N_VDestroy_Serial(kData->sEqns);
-  N_VDestroy_Serial(kData->c);
-  KINFree(&kData->kmem);
-  return 0;
-}
-
-static int initKinsol(KINODE *kinOde)
-{
-  int i,j,k, n;
-  double *scal_eq, *scal_var, *x, *f2;
-  long double tmp, h, hf, hf_min;
-  DATA *data;
-  NLPODE *nlp;
-  KDATAODE * kData;
-
-  n = kinOde->nlp->nStates;
-  data= kinOde->data;
-  kData = kinOde->kData;
-  x = NV_DATA_S(kData->x);
-  nlp = kinOde->nlp;
-  f2 = data->localData[2]->realVars + n;
-  nlp->dt = *(nlp->currentStep);
-  nlp->derx = data->localData[0]->realVars + n;
-  nlp->x0 = data->localData[1]->realVars;
-  nlp->f0 = data->localData[1]->realVars + n;
-  nlp->t0 = data->localData[1]->timeValue;
-
-  scal_var = NV_DATA_S(kData->sVars);
-  scal_eq = NV_DATA_S(kData->sEqns);
-
-  hf_min = 1e-6;
-  for(j=0, k=0; j<kinOde->N; ++j)
-  {
-    for(i=0;i<n;++i,++k)
-    {
-      hf = 0.5*nlp->dt*nlp->a[j]*(3*nlp->f0[i]-f2[i]);
-      if(fabsl(hf) < hf_min) hf_min = fabsl(hf);
-      x[k] = (nlp->x0[i] + hf);
-      tmp = fabs(x[k] + nlp->x0[i]) + 1e-12;
-      tmp = (tmp < 1e-9) ? nlp->s[i] : 2.0/tmp;
-      scal_var[k] = tmp + 1e-9;
-      scal_eq[k] = 1.0/(scal_var[k])+ 1e-12;
-    }
-  }
-  KINSetMaxNewtonStep(kinOde->kData->kmem, hf_min);
-  return 0;
-}
-
 static int refreshModell(DATA* data, threadData_t *threadData, double* x, double time)
 {
   SIMULATION_DATA *sData = (SIMULATION_DATA*)data->localData[0];
@@ -651,45 +807,9 @@ static int lobatto6Res(N_Vector x, N_Vector f, void* user_data)
 
   return 0;
 }
-
-int kinsolOde(void* ode)
-{
-  KINODE *kinOde = (KINODE*) ode;
-  KDATAODE *kData = kinOde->kData;
-  int i;
-  initKinsol(kinOde);
-  for(i = 0;i <3; ++i)
-  {
-
-    kData->error_code = KINSol(kData->kmem,           /* KINSol memory block */
-                               kData->x,              /* initial guess on input; solution vector */
-                               kData->glstr,          /* global stragegy choice */
-                               kData->sVars,          /* scaling vector, for the variable cc */
-                               kData->sEqns );
-    if(kData->error_code>=0)
-      return 0;
-
-    if(i == 0)
-    {
-     KINDense(kinOde->kData->kmem, kinOde->N*kinOde->nlp->nStates);
-     infoStreamPrint(LOG_SOLVER,0,"Restart Kinsol: change linear solver to KINDense.");
-    }
-    else if(i == 1)
-    {
-      KINSptfqmr(kinOde->kData->kmem, kinOde->N*kinOde->nlp->nStates);
-      infoStreamPrint(LOG_SOLVER,0,"Restart Kinsol: change linear solver to KINSptfqmr.");
-    }
-    else if(i == 2)
-    {
-      KINSpbcg(kinOde->kData->kmem, kinOde->N*kinOde->nlp->nStates);
-      infoStreamPrint(LOG_SOLVER,0,"Restart Kinsol: change linear solver to KINSpbcg.");
-    }
-  }
-  return (kData->error_code<0) ? -1 : 0;
-}
 #else
 
-int kinsolOde(void* ode)
+int kinsolOde(SOLVER_INFO* solverInfo)
 {
   assert(0);
   return -1;
