@@ -50,15 +50,25 @@ namespace ublas = boost::numeric::ublas;
 
 #if !defined(USE_CPP_03) && !defined(__vxworks)
   #include <array>
-  #include <thread>
-  #include <atomic>
-  #include <mutex>
-  #include <condition_variable>
   #include <tuple>
   #include <memory>
   #include <unordered_map>
   #include <unordered_set>
 
+  #if defined(USE_THREAD)
+    #include <thread>
+    #include <atomic>
+    #include <mutex>
+    #include <condition_variable>
+    using std::thread;
+    using std::atomic;
+    using std::mutex;
+    using std::memory_order_release;
+    using std::memory_order_relaxed;
+    using std::condition_variable;
+    using std::unique_lock;
+  #endif //USE_THREAD
+
   // builtin range based for loop
   #define FOREACH(element, range) for(element : range)
 
@@ -81,15 +91,9 @@ namespace ublas = boost::numeric::ublas;
   }
   using std::bind;
   using std::function;
-  using std::thread;
-  using std::atomic;
-  using std::mutex;
-  using std::memory_order_release;
-  using std::memory_order_relaxed;
-  using std::condition_variable;
-  using std::unique_lock;
   using std::make_tuple;
   using std::array;
+  using std::isfinite;
   using std::minmax_element;
   using std::get;
   using std::tuple;
@@ -118,6 +122,7 @@ namespace ublas = boost::numeric::ublas;
   #include <boost/lexical_cast.hpp>
   #include <boost/assign/list_of.hpp>
   #include <boost/array.hpp>
+  #include <boost/math/special_functions/fpclassify.hpp>
   #include <boost/math/special_functions/trunc.hpp>
   #include <boost/unordered_map.hpp>
   #include <boost/unordered_set.hpp>
@@ -139,7 +144,6 @@ namespace ublas = boost::numeric::ublas;
     using boost::memory_order_relaxed;
     using boost::condition_variable;
     using boost::unique_lock;
-    using boost::dynamic_pointer_cast;
   #endif //USE_THREAD
 
   // boost range based for loop
@@ -166,6 +170,7 @@ namespace ublas = boost::numeric::ublas;
     }
   }
   using boost::array;
+  using boost::math::isfinite;
   using boost::unordered_map;
   using boost::unordered_set;
   using boost::shared_ptr;
@@ -195,7 +200,7 @@ typedef ublas::matrix<double, ublas::column_major> matrix_t;
 #include <Core/SimulationSettings/ISettingsFactory.h>
 #include <SimCoreFactory/Policies/FactoryConfig.h>
 #include <Core/Utils/Modelica/ModelicaSimulationError.h>
-#include <Core/Utils/Modelica/ModelicaUtilities.h>
+#include <ModelicaUtilities.h>
 #include <Core/Math/Array.h>
 #include <Core/System/IStateSelection.h>
 #include <Core/System/ISystemProperties.h>
@@ -209,7 +214,9 @@ typedef ublas::matrix<double, ublas::column_major> matrix_t;
 #include <Core/Solver/ILinSolverSettings.h>
 #include <Core/DataExchange/IHistory.h>
 #include <Core/System/IMixedSystem.h>
-#include <Core/System/IAlgLoop.h>
+#include <Core/System/ILinearAlgLoop.h>
+#include <Core/System/INonLinearAlgLoop.h>
+#include <Core/System/ISystemTypes.h>
 #include <Core/Solver/ISolver.h>
 #include <Core/Solver/IAlgLoopSolver.h>
 #include <Core/System/IAlgLoopSolverFactory.h>
