@@ -10,11 +10,13 @@ class ITime
 public:
   virtual ~ITime() {};
   virtual int getDimTimeEvent() const = 0;
-  // gibt die Time events (Startzeit und Frequenz) zurück
-  virtual void getTimeEvent(time_event_type& time_events) = 0;
-  // Wird vom Solver zur Behandlung der Time events aufgerufen (wenn zero_sign[i] = 0  kein time event,zero_sign[i] = n  Anzahl von vorgekommen time events )
-  virtual void handleTimeEvent(int* time_events) = 0;
+  virtual std::pair<double, double>* getTimeEventData() const = 0;
+  virtual void initTimeEventData() = 0;// sets the start time and interval length in the timeEventData
+  virtual void computeTimeEventConditions(double currTime) = 0;
+  virtual double computeNextTimeEvents(double currTime) = 0;
+  virtual void resetTimeConditions() = 0;
+
   /// Set current integration time
   virtual void setTime(const double& time) = 0;
 };
-/** @} */ // end of coreSystem
\ No newline at end of file
+/** @} */ // end of coreSystem
