@@ -36,6 +36,26 @@
 
 #include "FMU2Wrapper.h"
 
+/* FMI extension for multi-rate sampled data systems */
+typedef fmi2Status fmi2GetClockTYPE(fmi2Component, const int fmi2Integer[],
+                                    size_t, fmi2Boolean[]);
+typedef fmi2Status fmi2SetClockTYPE(fmi2Component, const int fmiInteger[],
+                                    size_t, const fmi2Boolean[],
+                                    const fmi2Boolean*);
+typedef fmi2Status fmi2GetIntervalTYPE(fmi2Component, const int fmiInteger[],
+                                       size_t, fmi2Real[]);
+typedef fmi2Status fmi2SetIntervalTYPE(fmi2Component, const int fmiInteger[],
+                                       size_t, const fmi2Real[]);
+
+extern "C"
+{
+  FMI2_Export fmi2GetClockTYPE fmi2GetClock;
+  FMI2_Export fmi2SetClockTYPE fmi2SetClock;
+  FMI2_Export fmi2GetIntervalTYPE fmi2GetInterval;
+  FMI2_Export fmi2SetIntervalTYPE fmi2SetInterval;
+}
+
+/* Common definitions */
 #define LOG_CALL(w, ...) \
   FMU2_LOG(w, fmi2OK, logFmi2Call, __VA_ARGS__)
 
@@ -228,12 +248,12 @@ extern "C"
 
   fmi2Status fmi2GetClock(fmi2Component c,
                           const fmi2Integer clockIndex[],
-                          size_t nClockIndex, fmi2Boolean active[])
+                          size_t nClockIndex, fmi2Boolean tick[])
   {
     FMU2Wrapper *w = reinterpret_cast<FMU2Wrapper*>(c);
     LOG_CALL(w, "fmi2GetClock(nClockIndex = %d)", nClockIndex);
     try {
-      return w->getClock(clockIndex, nClockIndex, active);
+      return w->getClock(clockIndex, nClockIndex, tick);
     }
     CATCH_EXCEPTION(w);
   }
@@ -300,12 +320,13 @@ extern "C"
 
   fmi2Status fmi2SetClock(fmi2Component c,
                           const fmi2Integer clockIndex[],
-                          size_t nClockIndex, const fmi2Boolean active[])
+                          size_t nClockIndex, const fmi2Boolean tick[],
+                          const fmi2Boolean subactive[])
   {
     FMU2Wrapper *w = reinterpret_cast<FMU2Wrapper*>(c);
     LOG_CALL(w, "fmi2SetClock(nClockIndex = %d)", nClockIndex);
     try {
-      return w->setClock(clockIndex, nClockIndex, active);
+      return w->setClock(clockIndex, nClockIndex, tick, subactive);
     }
     CATCH_EXCEPTION(w);
   }
