@@ -34,6 +34,8 @@
 
 
 #include "FMUWrapper.h"
+#include "Modeling/MessagesWidget.h"
+#include "Util/Helper.h"
 
 SimSettingsFMU::SimSettingsFMU()
                 : _callEventUpdate(fmi1_false),
@@ -43,7 +45,8 @@ SimSettingsFMU::SimSettingsFMU()
                   _hdef(0.1),
                   _tend(0.1),
                   _relativeTolerance(0.001),
-                  _solver(Solver::EULER_FORWARD)
+                  _solver(Solver::EULER_FORWARD),
+                  mIterateEvents(true)
 {
 }
 
@@ -107,6 +110,15 @@ int SimSettingsFMU::getIntermediateResults()
   return _intermediateResults;
 }
 
+void SimSettingsFMU::setIterateEvents(bool iE)
+{
+  mIterateEvents = iE;
+}
+
+bool SimSettingsFMU::getIterateEvents()
+{
+  return mIterateEvents;
+}
 //-------------------------------
 // Abstract FMU class
 //-------------------------------
@@ -145,7 +157,7 @@ FMUWrapper_ME_1::~FMUWrapper_ME_1()
 
 void FMUWrapper_ME_1::fmi_get_real(unsigned int* valueRef, double* res)
 {
-	fmi1_import_get_real(mpFMU, valueRef, 1, res);
+  fmi1_import_get_real(mpFMU, valueRef, 1, res);
 }
 
 unsigned int FMUWrapper_ME_1::fmi_get_variable_by_name(const char* name)
@@ -199,6 +211,23 @@ void FMUWrapper_ME_1::initialize(const std::shared_ptr<SimSettingsFMU> simSettin
   mFMUdata._statesDer = (fmi1_real_t*) calloc(mFMUdata._nStates, sizeof(double));
   mFMUdata._eventIndicators = (fmi1_real_t*) calloc(mFMUdata._nEventIndicators, sizeof(double));
   mFMUdata._eventIndicatorsPrev = (fmi1_real_t*) calloc(mFMUdata._nEventIndicators, sizeof(double));
+  mFMUdata._stateVRs = (unsigned int*) malloc(mFMUdata._nStates* sizeof(unsigned int));
+
+  // get states to manipulate them
+  mFMUdata._fmiStatus = fmi1_import_get_state_value_references(mpFMU, mFMUdata._stateVRs, mFMUdata._nStates);
+  if (!mFMUdata._fmiStatus)
+  {
+  for (unsigned int i=0; i<mFMUdata._nStates; i++)
+  {
+    fmi1_import_variable_t * stateVar = fmi1_import_get_variable_by_vr(mpFMU, fmi1_base_type_enu_t::fmi1_base_type_real, mFMUdata._stateVRs[i]);
+    mFMUdata._stateNames.push_back(fmi1_import_get_variable_name(stateVar));
+  }
+  }
+  else
+  {
+    MessagesWidget::instance()->addGUIMessage(MessageItem(MessageItem::Modelica, "", false, 0, 0, 0, 0, QObject::tr("fmi1_import_get_state_value_references returned failure code"+mFMUdata._fmiStatus),
+                                                              Helper::scriptingKind, Helper::errorLevel));
+  }
 
   // Instantiate model
   jm_status_enu_t jmstatus = fmi1_import_instantiate_model(mpFMU, "Test ME model instance");
@@ -231,6 +260,11 @@ const FMUData* FMUWrapper_ME_1::getFMUData()
   return &mFMUdata;
 }
 
+fmi1_import_t* FMUWrapper_ME_1::getFMU()
+{
+  return mpFMU;
+}
+
 void FMUWrapper_ME_1::setContinuousStates()
 {
   mFMUdata._fmiStatus = fmi1_import_set_continuous_states(mpFMU, mFMUdata._states, mFMUdata._nStates);
@@ -321,7 +355,7 @@ FMUWrapper_ME_2::FMUWrapper_ME_2()
       mCallBackFunctions(),
       mFMUdata()
 {
-	mFMUdata.terminateSimulation = fmi2_false;
+  mFMUdata.terminateSimulation = fmi2_false;
 }
 
 FMUWrapper_ME_2::~FMUWrapper_ME_2()
@@ -342,7 +376,7 @@ FMUWrapper_ME_2::~FMUWrapper_ME_2()
 
 void FMUWrapper_ME_2::fmi_get_real(unsigned int* valueRef, double* res)
 {
-	fmi2_import_get_real(mpFMU, valueRef, 1, res);
+  fmi2_import_get_real(mpFMU, valueRef, 1, res);
 }
 
 void FMUWrapper_ME_2::load(const std::string& modelFile, const std::string& path, fmi_import_context_t* context)
@@ -389,6 +423,27 @@ void FMUWrapper_ME_2::initialize(const std::shared_ptr<SimSettingsFMU> simSettin
   mFMUdata._statesDer = (double*)calloc(mFMUdata._nStates, sizeof(double));
   mFMUdata._eventIndicators = (double*)calloc(mFMUdata._nEventIndicators, sizeof(double));
   mFMUdata._eventIndicatorsPrev = (double*)calloc(mFMUdata._nEventIndicators, sizeof(double));
+  mFMUdata._stateVRs = (unsigned int*) malloc(mFMUdata._nStates*sizeof(unsigned int));
+  mFMUdata._stateNames = {};
+
+  // initialize variables
+  std::string stateVarName = "";
+  std::string derVarName = "";
+
+  //set state VRs and Names
+  fmi2_import_variable_list_t* derVariables = fmi2_import_get_derivatives_list(mpFMU);
+  mFMUdata._stateVRs =  (unsigned int*)fmi2_import_get_value_referece_list(derVariables);
+  for (unsigned int i=0;i<mFMUdata._nStates;i++){
+    fmi2_import_variable_t *derVar = fmi2_import_get_variable_by_vr(mpFMU, fmi2_base_type_enu_t::fmi2_base_type_real, mFMUdata._stateVRs[i]);
+    derVarName = std::string(fmi2_import_get_variable_name(derVar));
+    if (derVarName.size() >= 5){
+      fmi2_import_variable_t * stateVar = (fmi2_import_variable_t*)fmi2_import_get_real_variable_derivative_of(fmi2_import_get_variable_as_real(derVar));
+      stateVarName = std::string(fmi2_import_get_variable_name(stateVar));
+      //std::cout<<" state "<<i<<" : "<<stateVarName<<std::endl;
+      mFMUdata._stateNames.push_back(stateVarName);
+      mFMUdata._stateVRs[i] = (unsigned int)fmi2_import_get_variable_vr(fmi2_import_get_variable_by_name(mpFMU, stateVarName.c_str()));
+    }
+  }
 
   // Instantiate model
   jm_status_enu_t jmstatus = fmi2_import_instantiate(mpFMU, "Test ME model instance",fmi2_model_exchange,0,0);
@@ -403,7 +458,7 @@ void FMUWrapper_ME_2::initialize(const std::shared_ptr<SimSettingsFMU> simSettin
   try
   {
     mFMUdata.fmiStatus2 = fmi2_import_enter_initialization_mode(mpFMU);
-	mFMUdata.fmiStatus2 = fmi2_import_exit_initialization_mode(mpFMU);
+  mFMUdata.fmiStatus2 = fmi2_import_exit_initialization_mode(mpFMU);
   }
   catch (std::exception &ex)
   {
@@ -451,6 +506,10 @@ const FMUData* FMUWrapper_ME_2::getFMUData()
   return &mFMUdata;
 }
 
+fmi2_import_t* FMUWrapper_ME_2::getFMU()
+{
+  return mpFMU;
+}
 
 void FMUWrapper_ME_2::setContinuousStates()
 {
@@ -480,7 +539,7 @@ void FMUWrapper_ME_2::updateNextTimeStep(const double hdef)
   double tlast = mFMUdata._tcur;
   mFMUdata._tcur += hdef;
   if (mFMUdata.eventInfo2.nextEventTimeDefined && (mFMUdata._tcur >= mFMUdata.eventInfo2.nextEventTime)) {
-	  mFMUdata._tcur = mFMUdata.eventInfo2.nextEventTime;
+    mFMUdata._tcur = mFMUdata.eventInfo2.nextEventTime;
   }
   mFMUdata._hcur = mFMUdata._tcur - tlast;
 }
@@ -488,11 +547,11 @@ void FMUWrapper_ME_2::updateNextTimeStep(const double hdef)
 void FMUWrapper_ME_2::handleEvents(const int intermediateResults)
 {
   //std::cout<<"Handle event at "<<std::to_string(mFMUdata._tcur)<<std::endl;
-	mFMUdata.fmiStatus2 = fmi2_import_enter_event_mode(mpFMU);
-	do_event_iteration(mpFMU, &mFMUdata.eventInfo2);
-	mFMUdata.fmiStatus2 = fmi2_import_enter_continuous_time_mode(mpFMU);
-	mFMUdata.fmiStatus2 = fmi2_import_get_continuous_states(mpFMU, mFMUdata._states, mFMUdata._nStates);
-	mFMUdata.fmiStatus2 = fmi2_import_get_event_indicators(mpFMU, mFMUdata._eventIndicators, mFMUdata._nEventIndicators);
+  mFMUdata.fmiStatus2 = fmi2_import_enter_event_mode(mpFMU);
+  do_event_iteration(mpFMU, &mFMUdata.eventInfo2);
+  mFMUdata.fmiStatus2 = fmi2_import_enter_continuous_time_mode(mpFMU);
+  mFMUdata.fmiStatus2 = fmi2_import_get_continuous_states(mpFMU, mFMUdata._states, mFMUdata._nStates);
+  mFMUdata.fmiStatus2 = fmi2_import_get_event_indicators(mpFMU, mFMUdata._eventIndicators, mFMUdata._nEventIndicators);
 }
 
 void FMUWrapper_ME_2::prepareSimulationStep(const double time)
