@@ -34,6 +34,15 @@
 
 #include "Visualizer.h"
 
+#if (QT_VERSION < QT_VERSION_CHECK(5, 2, 0))
+#include <QGLWidget>
+#endif
+#include <osg/Image>
+#include <osg/Shape>
+#include <osg/Node>
+#include <osgDB/Export>
+#include <osgDB/Registry>
+#include <osgDB/WriteFile>
 
 OMVisualBase::OMVisualBase(const std::string& modelFile, const std::string& path)
   : _shapes(),
@@ -44,6 +53,42 @@ OMVisualBase::OMVisualBase(const std::string& modelFile, const std::string& path
 {
 }
 
+/*!
+ * \brief OMVisualBase::getShapeObjectByID
+ * get the shapeObject with the same shapeID
+ *\param the name of the shape
+ *\return the selected shape
+ */
+ShapeObject* OMVisualBase::getShapeObjectByID(std::string shapeID)
+{
+  for(std::vector<ShapeObject>::iterator shape =_shapes.begin() ; shape < _shapes.end(); ++shape )
+  {
+      if(shape->_id == shapeID) {
+        return &(*shape);
+      }
+  }
+  return 0;
+}
+
+/*!
+ * \brief OMVisualBase::getShapeObjectIndexByID
+ * get the shapeObjectIndex with the same shapeID
+ *\param the name of the shape
+ *\return the selected shape
+ */
+int OMVisualBase::getShapeObjectIndexByID(std::string shapeID)
+{
+  int i = 0;
+  for(std::vector<ShapeObject>::iterator shape =_shapes.begin() ; shape < _shapes.end(); ++shape )
+  {
+      if(shape->_id == shapeID) {
+        return i;
+      }
+   i +=1;
+  }
+  return -1;
+}
+
 void OMVisualBase::initXMLDoc()
 {
   // Check if the XML file is available.
@@ -264,6 +309,18 @@ TimeManager* VisualizerAbstract::getTimeManager() const
   return mpTimeManager;
 }
 
+void VisualizerAbstract::modifyShape(std::string shapeName)
+{
+  int shapeIdx = getBaseData()->getShapeObjectIndexByID(shapeName);
+  ShapeObject* shape = getBaseData()->getShapeObjectByID(shapeName);
+  shape->setStateSetAction(stateSetAction::modify);
+  mpUpdateVisitor->_shape = *shape;
+  osg::ref_ptr<osg::Node> child = mpOMVisScene->getScene().getRootNode()->getChild(shapeIdx);  // the transformation
+  child->accept(*mpUpdateVisitor);
+  shape->setStateSetAction(stateSetAction::update);
+}
+
+
 void VisualizerAbstract::sceneUpdate()
 {
   //measure realtime
@@ -329,8 +386,6 @@ void VisualizerAbstract::pauseVisualization()
 }
 
 
-
-
 OMVisScene::OMVisScene()
   : _scene()
 {
@@ -358,8 +413,7 @@ OSGScene::OSGScene()
 int OSGScene::setUpScene(std::vector<ShapeObject> allShapes)
 {
   int isOk(0);
-  for (std::vector<ShapeObject>::size_type i = 0; i != allShapes.size(); i++)
-  {
+  for (std::vector<ShapeObject>::size_type i = 0; i != allShapes.size(); i++) {
     ShapeObject shape = allShapes[i];
     osg::ref_ptr<osg::Geode> geode;
     osg::ref_ptr<osg::StateSet> ss;
@@ -372,26 +426,22 @@ int OSGScene::setUpScene(std::vector<ShapeObject> allShapes)
     osg::ref_ptr<osg::MatrixTransform> transf = new osg::MatrixTransform();
 
     //cad node
-    if (shape._type.compare("stl") == 0)
-    {
+    if (shape._type.compare("stl") == 0) {
       //std::cout<<"Its a CAD and the filename is "<<shape._fileName<<std::endl;
       osg::ref_ptr<osg::Node> node = osgDB::readNodeFile(shape._fileName);
-      osg::ref_ptr<osg::StateSet> ss = node->getOrCreateStateSet();
-
-      ss->setAttribute(material.get());
-      node->setStateSet(ss);
-      transf->addChild(node.get());
-    }
-    else if ((shape._type.compare("dxf") == 0)) {
+      if (node) {
+        osg::ref_ptr<osg::StateSet> ss = node->getOrCreateStateSet();
+        ss->setAttribute(material.get());
+        node->setStateSet(ss);
+        transf->addChild(node.get());
+      }
+    } else if ((shape._type.compare("dxf") == 0)) {
       std::string name = shape._fileName;
       DXFile* shape = new DXFile(name);
       geode = new osg::Geode();
       geode->addDrawable(shape);
       transf->addChild(geode);
-    }
-    //geode with shape drawable
-    else
-    {
+    } else { //geode with shape drawable
       osg::ref_ptr<osg::ShapeDrawable> shapeDraw = new osg::ShapeDrawable();
       shapeDraw->setColor(osg::Vec4(1.0, 1.0, 1.0, 1.0));
       geode = new osg::Geode();
@@ -443,16 +493,18 @@ void UpdateVisitor::apply(osg::MatrixTransform& node)
  */
 void UpdateVisitor::apply(osg::Geode& node)
 {
-  //std::cout<<"GEODE "<< _shape._id<<" "<<std::endl;
+  //std::cout<<"GEODE "<< _shape._id<<" "<<_shape.getTransparency()<<std::endl;
   osg::ref_ptr<osg::StateSet> ss = node.getOrCreateStateSet();
-
-  //its a drawable and not a cad file so we have to create a new drawable
-  if (_shape._type.compare("dxf") != 0 and (_shape._type.compare("stl") != 0))
+  node.setName(_shape._id);
+  switch(_shape.getStateSetAction())
   {
+  case(stateSetAction::update):
+   {
+    //its a drawable and not a cad file so we have to create a new drawable
+    if (_shape._type.compare("dxf") != 0 and (_shape._type.compare("stl") != 0))
+    {
     osg::ref_ptr<osg::Drawable> draw = node.getDrawable(0);
     draw->dirtyDisplayList();
-    //osg::ref_ptr<osg::ShapeDrawable> shapeDraw = dynamic_cast<osg::ShapeDrawable*>(draw.get());
-    //shapeDraw->setColor(osg::Vec4(visAttr.color,1.0));
     if (_shape._type == "pipe")
     {
       node.removeDrawable(draw);
@@ -487,23 +539,137 @@ void UpdateVisitor::apply(osg::Geode& node)
     else
     {
       std::cout<<"Unknown type "<<_shape._type<<", we make a capsule."<<std::endl;
-      //string id = string(visAttr.type.begin(), visAttr.type.begin()+11);
       draw->setShape(new osg::Capsule(osg::Vec3f(0.0, 0.0, 0.0), 0.1, 0.5));
     }
     //std::cout<<"SHAPE "<<draw->getShape()->className()<<std::endl;
     node.addDrawable(draw.get());
-  }
+    }
+    break;
+   }//end case
+
+  case(stateSetAction::modify):
+   {
+     //apply texture
+     applyTexture(ss, _shape.getTextureImagePath());
+     break;
+   }//end case
+
+   default:
+   {break;}
+
+  }//end switch
+
+  //set color
   if (_shape._type.compare("dxf") != 0)
-  {
-    //osg::Material *material = dynamic_cast<osg::Material*>(ss->getAttribute(osg::StateAttribute::MATERIAL));
-    osg::ref_ptr<osg::Material> material = new osg::Material;
-    material->setDiffuse(osg::Material::FRONT, osg::Vec4f(_shape._color[0].exp / 255, _shape._color[1].exp / 255, _shape._color[2].exp / 255, 1.0));
-    ss->setAttribute(material);
-    node.setStateSet(ss);
-  }
+    changeColor(ss, _shape._color[0].exp, _shape._color[1].exp, _shape._color[2].exp);
+
+  //set transparency
+  makeTransparent(node, _shape.getTransparency());
+
+  node.setStateSet(ss);
   traverse(node);
 }
 
+/*!
+ * \brief UpdateVisitor::changeColor
+ * changes color for a geode
+ */
+void UpdateVisitor::changeColor(osg::StateSet* ss, float r, float g, float b)
+{
+  osg::Material *material;
+  if (!ss->getAttribute(osg::StateAttribute::MATERIAL))
+    material = new osg::Material();
+  else
+    material = dynamic_cast<osg::Material*>(ss->getAttribute(osg::StateAttribute::MATERIAL));
+  material->setDiffuse(osg::Material::FRONT, osg::Vec4f(r / 255, g / 255, b / 255, 1.0));
+  ss->setAttribute(material);
+}
+
+
+/*!
+ * \brief UpdateVisitor::applyTexture
+ * sets a texture for a geode
+ */
+void UpdateVisitor::applyTexture(osg::StateSet* ss, std::string imagePath)
+{
+  if (imagePath.compare(""))
+  {
+    osg::Image *image = nullptr;
+    std::string resIdent = ":/Resources";
+    if(!imagePath.compare(0,resIdent.length(),resIdent))
+    {
+      QImage* qim = new QImage(QString::fromStdString(imagePath));
+      image = convertImage(*qim);
+      image->setInternalTextureFormat(GL_RGBA);
+    }
+    else
+    {
+      image = osgDB::readImageFile(imagePath);
+    }
+    if (image)
+    {
+    osg::Texture2D *texture = new osg::Texture2D;
+    texture->setDataVariance(osg::Object::DYNAMIC);
+    texture->setFilter(osg::Texture::MIN_FILTER, osg::Texture::LINEAR_MIPMAP_LINEAR);
+    texture->setFilter(osg::Texture::MAG_FILTER, osg::Texture::LINEAR);
+    texture->setWrap(osg::Texture::WRAP_S, osg::Texture::CLAMP);
+    texture->setImage(image);
+    texture->setResizeNonPowerOfTwoHint(false);// dont output console message about scaling
+    ss->setTextureAttributeAndModes(0, texture, osg::StateAttribute::ON);
+    }
+  }
+  else
+  {
+    ss->getTextureAttributeList().clear();
+    ss->getTextureModeList().clear();
+  }
+}
+
+osg::Image* UpdateVisitor::convertImage(const QImage& iImage)
+{
+  osg::Image* osgImage = new osg::Image();
+  if (false == iImage.isNull()) {
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 2, 0))
+    QImage glImage = iImage.convertToFormat(QImage::Format_RGBA8888_Premultiplied);
+#else
+    QImage glImage = QGLWidget::convertToGLFormat(iImage);
+#endif
+    if (false == glImage.isNull()) {
+      unsigned char* data = new unsigned char[glImage.byteCount()];
+      for(int i=0; i < glImage.byteCount(); ++i) {
+        data[i] = glImage.bits()[i];
+      }
+      osgImage->setImage(glImage.width(), glImage.height(), 1, 4, GL_RGBA, GL_UNSIGNED_BYTE, data, osg::Image::USE_NEW_DELETE, 1);
+    }
+  }
+  return osgImage;
+}
+
+
+/*!
+ * \brief UpdateVisitor::makeTransparent
+ * makes a geode transparent
+ */
+void UpdateVisitor::makeTransparent(osg::Geode& node, float transpCoeff)
+{
+  if (_shape.getTransparency())
+      {
+      node.getStateSet()->setMode( GL_BLEND, osg::StateAttribute::ON );
+      node.getStateSet()->setRenderingHint(osg::StateSet::TRANSPARENT_BIN);
+      osg::Material *material;
+      if (NULL == node.getStateSet()->getAttribute(osg::StateAttribute::MATERIAL))
+      {
+        material = new osg::Material();
+      }
+      else
+      {
+        material = dynamic_cast<osg::Material*>(node.getStateSet()->getAttribute(osg::StateAttribute::MATERIAL));
+      }
+      material->setTransparency(osg::Material::FRONT_AND_BACK, transpCoeff);
+      node.getStateSet()->setAttributeAndModes(material, osg::StateAttribute::OVERRIDE);
+  }
+}
+
 InfoVisitor::InfoVisitor()
   : _level(0)
 {
