@@ -29,28 +29,33 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- *
  */
 
 #ifndef LINEANNOTATION_H
 #define LINEANNOTATION_H
 
 #include "ShapeAnnotation.h"
-#include "Component.h"
 
+#include <QTreeView>
+#include <QSortFilterProxyModel>
+#include <QSpinBox>
+#include <QUndoCommand>
+
+class Label;
 class Component;
+class TextAnnotation;
 
 class LineAnnotation : public ShapeAnnotation
 {
   Q_OBJECT
 public:
   enum LineType {
-    ComponentType,  /* Line is within Component. */
-    ConnectionType,  /* Line is a connection. */
-    ShapeType  /* Line is a custom shape. */
+    ComponentType,    /* Line is within Component. */
+    ConnectionType,   /* Line is a connection. */
+    TransitionType,   /* Line is a transition. */
+    InitialStateType, /* Line is an initial state. */
+    ShapeType         /* Line is a custom shape. */
   };
   // Used for icon/diagram shape
   LineAnnotation(QString annotation, GraphicsView *pGraphicsView);
@@ -58,10 +63,15 @@ public:
   LineAnnotation(ShapeAnnotation *pShapeAnnotation, Component *pParent);
   // Used for icon/diagram inherited shape
   LineAnnotation(ShapeAnnotation *pShapeAnnotation, GraphicsView *pGraphicsView);
-  // Used for creating connection
-  LineAnnotation(Component *pStartComponent, GraphicsView *pGraphicsView);
+  // Used for creating connection/transition
+  LineAnnotation(LineAnnotation::LineType lineType, Component *pStartComponent, GraphicsView *pGraphicsView);
   // Used for reading a connection
   LineAnnotation(QString annotation, Component *pStartComponent, Component *pEndComponent, GraphicsView *pGraphicsView);
+  // Used for reading a transition
+  LineAnnotation(QString annotation, QString text, Component *pStartComponent, Component *pEndComponent, QString condition, QString immediate,
+                 QString reset, QString synchronize, QString priority, GraphicsView *pGraphicsView);
+  // Used for reading an initial state
+  LineAnnotation(QString annotation, Component *pComponent, GraphicsView *pGraphicsView);
   // Used for non-exisiting component
   LineAnnotation(Component *pParent);
   // Used for non-existing class
@@ -72,16 +82,18 @@ public:
   QPainterPath shape() const;
   void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget = 0);
   void drawLineAnnotaion(QPainter *painter);
-  QPolygonF drawArrow(QPointF startPos, QPointF endPos, qreal size, int arrowType) const;
+  void drawArrow(QPainter *painter, QPointF startPos, QPointF endPos, qreal size, int arrowType) const;
+  QPolygonF perpendicularLine(QPointF startPos, QPointF endPos, qreal size) const;
   QString getOMCShapeAnnotation();
   QString getShapeAnnotation();
-  QString getTLMShapeAnnotation();
+  QString getCompositeModelShapeAnnotation();
   void addPoint(QPointF point);
   void removePoint(int index);
   void clearPoints();
   void updateStartPoint(QPointF point);
   void updateEndPoint(QPointF point);
   void moveAllPoints(qreal offsetX, qreal offsetY);
+  void updateTransitionTextPosition();
   void setLineType(LineType lineType) {mLineType = lineType;}
   LineType getLineType() {return mLineType;}
   void setStartComponent(Component *pStartComponent) {mpStartComponent = pStartComponent;}
@@ -92,17 +104,58 @@ public:
   Component* getEndComponent() {return mpEndComponent;}
   void setEndComponentName(QString name) {mEndComponentName = name;}
   QString getEndComponentName() {return mEndComponentName;}
+  void setCondition(QString condition) {mCondition = condition;}
+  QString getCondition() {return mCondition;}
+  void setImmediate(bool immediate) {mImmediate = immediate;}
+  bool getImmediate() {return mImmediate;}
+  void setReset(bool reset) {mReset = reset;}
+  bool getReset() {return mReset;}
+  void setSynchronize(bool synchronize) {mSynchronize = synchronize;}
+  bool getSynchronize() {return mSynchronize;}
+  void setPriority(int priority) {mPriority = priority;}
+  int getPriority() {return mPriority;}
+  TextAnnotation* getTextAnnotation() {return mpTextAnnotation;}
+  void setOldAnnotation(QString oldAnnotation) {mOldAnnotation = oldAnnotation;}
+  QString getOldAnnotation() {return mOldAnnotation;}
+  void setDelay(QString delay) {mDelay = delay;}
+  QString getDelay() {return mDelay;}
+  void setZf(QString zf) {mZf = zf;}
+  QString getZf() {return mZf;}
+  void setZfr(QString zfr) {mZfr = zfr;}
+  QString getZfr() {return mZfr;}
+  void setAlpha(QString alpha) {mAlpha = alpha;}
+  QString getAlpha() {return mAlpha;}
   void setShapeFlags(bool enable);
   void updateShape(ShapeAnnotation *pShapeAnnotation);
-private:
+  void setAligned(bool aligned);
+protected:
+  QVariant itemChange(GraphicsItemChange change, const QVariant &value);
+
+  private:
   LineType mLineType;
   Component *mpStartComponent;
   QString mStartComponentName;
   Component *mpEndComponent;
   QString mEndComponentName;
+  QString mCondition;
+  bool mImmediate;
+  bool mReset;
+  bool mSynchronize;
+  int mPriority;
+  TextAnnotation *mpTextAnnotation;
+  // MetaModel attributes
+  QString mOldAnnotation;
+  // CompositeModel attributes
+  QString mDelay;
+  QString mZf;
+  QString mZfr;
+  QString mAlpha;
 public slots:
   void handleComponentMoved();
   void updateConnectionAnnotation();
+  void updateConnectionTransformation(QUndoCommand *pUndoCommand);
+  void updateTransitionAnnotation(QString oldCondition, bool oldImmediate, bool oldReset, bool oldSynchronize, int oldPriority);
+  void updateInitialStateAnnotation();
   void duplicate();
 };
 
@@ -226,7 +279,33 @@ private:
 public slots:
   void startConnectorChanged(const QModelIndex &current, const QModelIndex &previous);
   void endConnectorChanged(const QModelIndex &current, const QModelIndex &previous);
-  void createArrayConnection();
+  void createConnection();
+};
+
+class CreateOrEditTransitionDialog : public QDialog
+{
+  Q_OBJECT
+public:
+  CreateOrEditTransitionDialog(GraphicsView *pGraphicsView, LineAnnotation *pTransitionLineAnnotation, bool editCase, QWidget *pParent = 0);
+private:
+  GraphicsView *mpGraphicsView;
+  LineAnnotation *mpTransitionLineAnnotation;
+  bool mEditCase;
+  Label *mpHeading;
+  QFrame *mpHorizontalLine;
+  QGroupBox *mpPropertiesGroupBox;
+  Label *mpConditionLabel;
+  QLineEdit *mpConditionTextBox;
+  QCheckBox *mpImmediateCheckBox;
+  QCheckBox *mpResetCheckBox;
+  QCheckBox *mpSynchronizeCheckBox;
+  Label *mpPriorityLabel;
+  QSpinBox *mpPrioritySpinBox;
+  QPushButton *mpOkButton;
+  QPushButton *mpCancelButton;
+  QDialogButtonBox *mpButtonBox;
+public slots:
+  void createOrEditTransition();
 };
 
 #endif // LINEANNOTATION_H
