@@ -29,14 +29,11 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- *
  */
 
 #include "RectangleAnnotation.h"
-#include "Commands.h"
+#include "Modeling/Commands.h"
 
 RectangleAnnotation::RectangleAnnotation(QString annotation, GraphicsView *pGraphicsView)
   : ShapeAnnotation(false, pGraphicsView, 0)
@@ -55,6 +52,7 @@ RectangleAnnotation::RectangleAnnotation(ShapeAnnotation *pShapeAnnotation, Comp
   : ShapeAnnotation(pParent)
 {
   updateShape(pShapeAnnotation);
+  initUpdateVisible(); // DynamicSelect for visible attribute
   setPos(mOrigin);
   setRotation(mRotation);
   connect(pShapeAnnotation, SIGNAL(updateReferenceShapes()), pShapeAnnotation, SIGNAL(changed()));
@@ -135,8 +133,9 @@ void RectangleAnnotation::paint(QPainter *painter, const QStyleOptionGraphicsIte
 {
   Q_UNUSED(option);
   Q_UNUSED(widget);
-  if (mVisible)
+  if (mVisible || !mDynamicVisible.isEmpty()) {
     drawRectangleAnnotaion(painter);
+  }
 }
 
 void RectangleAnnotation::drawRectangleAnnotaion(QPainter *painter)
