@@ -102,6 +102,29 @@ TextAnnotation::TextAnnotation(Component *pParent)
   setRotation(mRotation);
 }
 
+TextAnnotation::TextAnnotation(QString annotation, LineAnnotation *pLineAnnotation)
+  : ShapeAnnotation(pLineAnnotation)
+{
+  mpComponent = 0;
+  // set the default values
+  GraphicItem::setDefaults();
+  FilledShape::setDefaults();
+  ShapeAnnotation::setDefaults();
+  parseShapeAnnotation(annotation);
+  updateTextString();
+  /* From Modelica Spec 33revision1,
+   * The extent of the Text is interpreted relative to either the first point of the Line, in the case of immediate=false,
+   * or the last point (immediate=true).
+   */
+  if (pLineAnnotation->getPoints().size() > 0) {
+    if (pLineAnnotation->getImmediate()) {
+      setPos(pLineAnnotation->getPoints().last());
+    } else {
+      setPos(pLineAnnotation->getPoints().first());
+    }
+  }
+}
+
 /*!
  * \brief TextAnnotation::parseShapeAnnotation
  * Parses the text annotation string
@@ -244,25 +267,29 @@ void TextAnnotation::drawTextAnnotaion(QPainter *painter)
   mTextString = StringHandler::removeFirstLastQuotes(mTextString);
   mTextString = StringHandler::unparse(QString("\"").append(mTextString).append("\""));
   QFont font;
+  font = QFont(mFontName, mFontSize, StringHandler::getFontWeight(mTextStyles), StringHandler::getFontItalic(mTextStyles));
+  // set font underline
+  if (StringHandler::getFontUnderline(mTextStyles)) {
+    font.setUnderline(true);
+  }
+  painter->setFont(font);
+  QRect fontBoundRect = painter->fontMetrics().boundingRect(boundingRect().toRect(), Qt::TextDontClip, mTextString);
+  bool calculateFontSize = true;
   if (mFontSize > 0) {
-    font = QFont(mFontName, mFontSize, StringHandler::getFontWeight(mTextStyles), StringHandler::getFontItalic(mTextStyles));
-    // set font underline
-    if(StringHandler::getFontUnderline(mTextStyles)) {
-      font.setUnderline(true);
+    if (boundingRect().width() != 0 && fontBoundRect.width() <= boundingRect().width()) {
+      calculateFontSize = false;
+    } else if (boundingRect().height() != 0 && fontBoundRect.height() <= boundingRect().height()) {
+      calculateFontSize = false;
     }
-    font.setPointSizeF(mFontSize/4);
-    painter->setFont(font);
-  } else {
-    font = QFont(mFontName, mFontSize, StringHandler::getFontWeight(mTextStyles), StringHandler::getFontItalic(mTextStyles));
-    // set font underline
-    if(StringHandler::getFontUnderline(mTextStyles)) {
-      font.setUnderline(true);
-    }
-    painter->setFont(font);
-    QRect fontBoundRect = painter->fontMetrics().boundingRect(boundingRect().toRect(), Qt::TextDontClip, mTextString);
+  }
+  if (calculateFontSize) {
     float xFactor = boundingRect().width() / fontBoundRect.width();
     float yFactor = boundingRect().height() / fontBoundRect.height();
-    float factor = xFactor < yFactor ? xFactor : yFactor;
+    /* Ticket:4256
+       * Text aspect when x1=x2 i.e, width is 0.
+       * Use height.
+       */
+    float factor = (boundingRect().width() != 0 && xFactor < yFactor) ? xFactor : yFactor;
     QFont f = painter->font();
     qreal fontSizeFactor = f.pointSizeF()*factor;
     if ((fontSizeFactor < 12) && mpComponent) {
@@ -319,10 +346,12 @@ void TextAnnotation::drawTextAnnotaion(QPainter *painter)
     }
   }
   // draw the font
-  if (mpComponent) {
-    painter->drawText(boundingRect(), StringHandler::getTextAlignment(mHorizontalAlignment) | Qt::AlignVCenter | Qt::TextDontClip, mTextString);
-  } else if (boundingRect().width() > 0 && boundingRect().height() > 0) {
+  if (mpComponent || boundingRect().width() > 0 || boundingRect().height() > 0) {
     painter->drawText(boundingRect(), StringHandler::getTextAlignment(mHorizontalAlignment) | Qt::AlignVCenter | Qt::TextDontClip, mTextString);
+    mExportBoundingRect = painter->boundingRect(boundingRect(), StringHandler::getTextAlignment(mHorizontalAlignment) | Qt::AlignVCenter | Qt::TextDontClip, mTextString);
+    if (mpComponent) {
+      mExportBoundingRect = sceneTransform().mapRect(mExportBoundingRect);
+    }
   }
 }
 
@@ -512,26 +541,38 @@ void TextAnnotation::updateTextString()
    * - %name replaced by the name of the component (i.e. the identifier for it in in the enclosing class).
    * - %class replaced by the name of the class.
    */
-  mTextString = mOriginalTextString;
-  if (!mTextString.contains("%")) {
-    return;
-  }
-  if (mOriginalTextString.toLower().contains("%name")) {
-    mTextString.replace(QRegExp("%name"), mpComponent->getName());
-  }
-  if (mOriginalTextString.toLower().contains("%class")) {
-    mTextString.replace(QRegExp("%class"), mpComponent->getLibraryTreeItem()->getNameStructure());
-  }
-  if (!mTextString.contains("%")) {
-    return;
-  }
-  /* handle variables now */
-  updateTextStringHelper(QRegExp("(%%|%\\w*)"));
-  /* call again with non-word characters so invalid % can be removed. */
-  updateTextStringHelper(QRegExp("(%%|%\\W*)"));
-  /* handle %% */
-  if (mOriginalTextString.toLower().contains("%%")) {
-    mTextString.replace(QRegExp("%%"), "%");
+  LineAnnotation *pLineAnnotation = dynamic_cast<LineAnnotation*>(parentItem());
+  if (pLineAnnotation) {
+    if (mOriginalTextString.toLower().contains("%condition")) {
+      if (!pLineAnnotation->getCondition().isEmpty()) {
+        mTextString.replace(QRegExp("%condition"), pLineAnnotation->getCondition());
+      }
+      if (pLineAnnotation->getPriority() > 1) {
+        mTextString.prepend(QString("%1: ").arg(pLineAnnotation->getPriority()));
+      }
+    }
+  } else if (mpComponent) {
+    mTextString = mOriginalTextString;
+    if (!mTextString.contains("%")) {
+      return;
+    }
+    if (mOriginalTextString.toLower().contains("%name")) {
+      mTextString.replace(QRegExp("%name"), mpComponent->getName());
+    }
+    if (mOriginalTextString.toLower().contains("%class")) {
+      mTextString.replace(QRegExp("%class"), mpComponent->getLibraryTreeItem()->getNameStructure());
+    }
+    if (!mTextString.contains("%")) {
+      return;
+    }
+    /* handle variables now */
+    updateTextStringHelper(QRegExp("(%%|%\\w*)"));
+    /* call again with non-word characters so invalid % can be removed. */
+    updateTextStringHelper(QRegExp("(%%|%\\W*)"));
+    /* handle %% */
+    if (mOriginalTextString.toLower().contains("%%")) {
+      mTextString.replace(QRegExp("%%"), "%");
+    }
   }
 }
 
