@@ -29,26 +29,23 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- *
  */
 
 #ifndef COMPONENT_H
 #define COMPONENT_H
 
-#include "ShapeAnnotation.h"
-#include "CornerItem.h"
-#include "CoOrdinateSystem.h"
-#include "ModelWidgetContainer.h"
-#include "OMCProxy.h"
-#include "LineAnnotation.h"
-#include "PolygonAnnotation.h"
-#include "RectangleAnnotation.h"
-#include "EllipseAnnotation.h"
-#include "TextAnnotation.h"
-#include "BitmapAnnotation.h"
+#include "Annotations/ShapeAnnotation.h"
+#include "Component/CornerItem.h"
+#include "Modeling/CoOrdinateSystem.h"
+#include "Modeling/ModelWidgetContainer.h"
+#include "OMC/OMCProxy.h"
+#include "Annotations/LineAnnotation.h"
+#include "Annotations/PolygonAnnotation.h"
+#include "Annotations/RectangleAnnotation.h"
+#include "Annotations/EllipseAnnotation.h"
+#include "Annotations/TextAnnotation.h"
+#include "Annotations/BitmapAnnotation.h"
 
 class OMCProxy;
 class GraphicsScene;
@@ -59,7 +56,7 @@ class RectangleAnnotation;
 class EllipseAnnotation;
 class TextAnnotation;
 class BitmapAnnotation;
-class TLMInterfacePointInfo;
+class LibraryTreeItem;
 
 class ComponentInfo : public QObject
 {
@@ -69,8 +66,9 @@ public:
   ComponentInfo(ComponentInfo *pComponentInfo, QObject *pParent = 0);
   void updateComponentInfo(const ComponentInfo *pComponentInfo);
   void parseComponentInfoString(QString value);
-  void fetchModifiers(OMCProxy *pOMCProxy, QString className);
+  void fetchModifiers(OMCProxy *pOMCProxy, QString className, Component *pComponent);
   void fetchParameterValue(OMCProxy *pOMCProxy, QString className);
+  void applyDefaultPrefixes(QString defaultPrefixes);
   void setClassName(QString className) {mClassName = className;}
   QString getClassName() const {return mClassName;}
   void setName(QString name) {mName = name;}
@@ -101,22 +99,30 @@ public:
   bool isModifiersLoaded() const {return mModifiersLoaded;}
   void setModifiersMap(QMap<QString, QString> modifiersMap) {mModifiersMap = modifiersMap;}
   QMap<QString, QString> getModifiersMapWithoutFetching() const {return mModifiersMap;}
-  QMap<QString, QString> getModifiersMap(OMCProxy *pOMCProxy, QString className);
+  QMap<QString, QString> getModifiersMap(OMCProxy *pOMCProxy, QString className, Component *pComponent);
   bool isParameterValueLoaded() const {return mParameterValueLoaded;}
   void setParameterValue(QString parameterValue) {mParameterValue = parameterValue;}
   QString getParameterValueWithoutFetching() const {return mParameterValue;}
   QString getParameterValue(OMCProxy *pOMCProxy, QString className);
-  // MetaModel attributes
+  // CompositeModel attributes
   void setStartCommand(QString startCommand) {mStartCommand = startCommand;}
   QString getStartCommand() const {return mStartCommand;}
   void setExactStep(bool exactStep) {mExactStep = exactStep;}
   bool getExactStep() const {return mExactStep;}
   void setModelFile(QString modelFile) {mModelFile = modelFile;}
   QString getModelFile() const {return mModelFile;}
+  void setGeometryFile(QString geometryFile) {mGeometryFile = geometryFile;}
+  QString getGeometryFile() const {return mGeometryFile;}
   void setPosition(QString position) {mPosition = position;}
   QString getPosition() const {return mPosition;}
   void setAngle321(QString angle321) {mAngle321 = angle321;}
   QString getAngle321() const {return mAngle321;}
+  void setDimensions(int dimensions) {mDimensions = dimensions;}
+  int getDimensions() const {return mDimensions;}
+  void setTLMCausality(QString causality) {mTLMCausality = causality;}
+  QString getTLMCausality() const {return mTLMCausality;}
+  void setDomain(QString domain) {mDomain = domain;}
+  QString getDomain() const {return mDomain;}
   // operator overloading
   bool operator==(const ComponentInfo &componentInfo) const;
   bool operator!=(const ComponentInfo &componentInfo) const;
@@ -141,12 +147,18 @@ private:
   QMap<QString, QString> mModifiersMap;
   bool mParameterValueLoaded;
   QString mParameterValue;
-  // MetaModel attributes
+  // CompositeModel attributes
   QString mStartCommand;
   bool mExactStep;
   QString mModelFile;
+  QString mGeometryFile;
   QString mPosition;
   QString mAngle321;
+  int mDimensions;
+  QString mTLMCausality;
+  QString mDomain;
+
+  bool isModiferClassRecord(QString modifierName, Component *pComponent);
 };
 
 class Component : public QObject, public QGraphicsItem
@@ -159,15 +171,18 @@ public:
     Extend,  /* Inherited Component. */
     Port  /* Port Component. */
   };
-  Component(QString name, LibraryTreeItem *pLibraryTreeItem, QString transformation, QPointF position, QStringList dialogAnnotation,
-            ComponentInfo *pComponentInfo, GraphicsView *pGraphicsView);
+  Component(QString name, LibraryTreeItem *pLibraryTreeItem, QString annotation, QPointF position, ComponentInfo *pComponentInfo,
+            GraphicsView *pGraphicsView);
   Component(LibraryTreeItem *pLibraryTreeItem, Component *pParentComponent);
   Component(Component *pComponent, Component *pParentComponent, Component *pRootParentComponent);
   Component(Component *pComponent, GraphicsView *pGraphicsView);
+  // used for interface point
+  Component(ComponentInfo *pComponentInfo, Component *pParentComponent);
   bool isInheritedComponent() {return mIsInheritedComponent;}
   bool hasShapeAnnotation(Component *pComponent);
   bool hasNonExistingClass();
   QRectF boundingRect() const;
+  QRectF itemsBoundingRect();
   void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget = 0);
   LibraryTreeItem* getLibraryTreeItem() {return mpLibraryTreeItem;}
   QString getName() {return mpComponentInfo->getName();}
@@ -179,18 +194,20 @@ public:
   QString getTransformationString() {return mTransformationString;}
   void setDialogAnnotation(QStringList dialogAnnotation) {mDialogAnnotation = dialogAnnotation;}
   QStringList getDialogAnnotation() {return mDialogAnnotation;}
+  void setChoicesAnnotation(QStringList choicesAnnotation) {mChoicesAnnotation = choicesAnnotation;}
+  QStringList getChoicesAnnotation() {return mChoicesAnnotation;}
   CoOrdinateSystem getCoOrdinateSystem() const;
   OriginItem* getOriginItem() {return mpOriginItem;}
   QAction* getParametersAction() {return mpParametersAction;}
+  QAction* getFetchInterfaceDataAction() {return mpFetchInterfaceDataAction;}
   QAction* getAttributesAction() {return mpAttributesAction;}
   QAction* getOpenClassAction() {return mpOpenClassAction;}
   QAction* getViewDocumentationAction() {return mpViewDocumentationAction;}
-  QAction* getTLMAttributesAction() {return mpTLMAttributesAction;}
+  QAction* getSubModelAttributesAction() {return mpSubModelAttributesAction;}
   ComponentInfo* getComponentInfo() {return mpComponentInfo;}
   QList<ShapeAnnotation*> getShapesList() {return mShapesList;}
   QList<Component*> getInheritedComponentsList() {return mInheritedComponentsList;}
   QList<Component*> getComponentsList() {return mComponentsList;}
-  QList<TLMInterfacePointInfo*> getInterfacepointsList() {return mInterfacePointsList;}
   void setOldScenePosition(QPointF oldScenePosition) {mOldScenePosition = oldScenePosition;}
   QPointF getOldScenePosition() {return mOldScenePosition;}
   void setOldPosition(QPointF oldPosition) {mOldPosition = oldPosition;}
@@ -206,9 +223,13 @@ public:
   void applyRotation(qreal angle);
   void addConnectionDetails(LineAnnotation *pConnectorLineAnnotation);
   void removeConnectionDetails(LineAnnotation *pConnectorLineAnnotation);
+  void setHasTransition(bool hasTransition);
+  void setIsInitialState(bool isInitialState);
+  void removeChildren();
   void emitAdded();
   void emitTransformChange() {emit transformChange();}
   void emitTransformHasChanged();
+  void emitTransformChanging(QUndoCommand *pUndoCommand) {emit transformChanging(pUndoCommand);}
   void emitChanged();
   void emitDeleted();
   void componentParameterHasChanged();
@@ -217,9 +238,9 @@ public:
   void shapeUpdated();
   void shapeDeleted();
   void renameComponentInConnections(QString newName);
-  void addInterfacePoint(TLMInterfacePointInfo *pTLMInterfacePointInfo);
-  void removeInterfacePoint(TLMInterfacePointInfo *pTLMInterfacePointInfo);
-  void renameInterfacePoint(TLMInterfacePointInfo *pTLMInterfacePointInfo, QString interfacePoint);
+  void insertInterfacePoint(QString interfaceName, QString position, QString angle321, int dimensions, QString causality, QString domain);
+  void removeInterfacePoint(QString interfaceName);
+  void adjustInterfacePoints();
 
   Transformation mTransformation;
   Transformation mOldTransformation;
@@ -233,16 +254,19 @@ private:
   ComponentType mComponentType;
   QString mTransformationString;
   QStringList mDialogAnnotation;
+  QStringList mChoicesAnnotation;
   QString mParameterValue;
   QGraphicsRectItem *mpResizerRectangle;
   LineAnnotation *mpNonExistingComponentLine;
   RectangleAnnotation *mpDefaultComponentRectangle;
   TextAnnotation *mpDefaultComponentText;
+  RectangleAnnotation *mpStateComponentRectangle;
   QAction *mpParametersAction;
+  QAction *mpFetchInterfaceDataAction;
   QAction *mpAttributesAction;
   QAction *mpOpenClassAction;
   QAction *mpViewDocumentationAction;
-  QAction *mpTLMAttributesAction;
+  QAction *mpSubModelAttributesAction;
   ResizerItem *mpBottomLeftResizerItem;
   ResizerItem *mpTopLeftResizerItem;
   ResizerItem *mpTopRightResizerItem;
@@ -259,16 +283,18 @@ private:
   QList<ShapeAnnotation*> mShapesList;
   QList<Component*> mComponentsList;
   QPointF mOldScenePosition;
-  QList<TLMInterfacePointInfo*> mInterfacePointsList;
   QPointF mOldPosition;
+  bool mHasTransition;
+  bool mIsInitialState;
   void createNonExistingComponent();
   void createDefaultComponent();
+  void createStateComponent();
+  void drawInterfacePoints();
   void drawComponent();
   void drawInheritedComponentsAndShapes();
   void showNonExistingOrDefaultComponentIfNeeded();
   void createClassInheritedComponents();
   void createClassShapes();
-  void removeChildren();
   void createActions();
   void createResizerItems();
   void getResizerItemsPositions(qreal *x1, qreal *y1, qreal *x2, qreal *y2);
@@ -278,12 +304,15 @@ private:
   void setOriginAndExtents();
   void updateConnections();
   QString getParameterDisplayStringFromExtendsModifiers(QString parameterName);
-  QString getParameterDisplayStringFromExtendsParameters(QString parameterName);
+  QString getParameterDisplayStringFromExtendsParameters(QString parameterName, QString modifierString);
+  bool checkEnumerationDisplayString(QString &displayString, const QString &typeName);
   void updateToolTip();
+  bool canUseDiagramAnnotation();
 signals:
   void added();
   void transformChange();
   void transformHasChanged();
+  void transformChanging(QUndoCommand *pUndoCommand);
   void displayTextChanged();
   void changed();
   void deleted();
@@ -325,11 +354,11 @@ public slots:
   void moveCtrlRight();
   void showParameters();
   void showAttributes();
+  void fetchInterfaceData();
   void openClass();
   void viewDocumentation();
-  void showTLMAttributes();
+  void showSubModelAttributes();
 protected:
-  virtual void mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event);
   virtual void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
   virtual QVariant itemChange(GraphicsItemChange change, const QVariant &value);
 };
