@@ -44,6 +44,8 @@
 #include <QMessageBox>
 #include <QDesktopServices>
 #include <QDesktopWidget>
+#include <QList>
+#include <QStringList>
 
 /*!
  * \class Parameter
@@ -69,7 +71,12 @@ Parameter::Parameter(Component *pComponent, bool showStartAttribute, QString tab
   // set the value type based on component type.
   OMCProxy *pOMCProxy = MainWindow::instance()->getOMCProxy();
   if (mpComponent->getComponentInfo()->getClassName().compare("Boolean") == 0) {
-    mValueType = Parameter::Boolean;
+    if (mpComponent->getChoicesAnnotation().size() > 0 &&
+        (mpComponent->getChoicesAnnotation().at(0).compare("true") == 0 || mpComponent->getChoicesAnnotation().at(1).compare("true") == 0)) {
+      mValueType = Parameter::CheckBox;
+    } else {
+      mValueType = Parameter::Boolean;
+    }
   } else if (pOMCProxy->isBuiltinType(mpComponent->getComponentInfo()->getClassName())) {
     mValueType = Parameter::Normal;
   } else if (pOMCProxy->isWhat(StringHandler::Enumeration, mpComponent->getComponentInfo()->getClassName())) {
@@ -77,6 +84,16 @@ Parameter::Parameter(Component *pComponent, bool showStartAttribute, QString tab
   } else {
     mValueType = Parameter::Normal;
   }
+  mValueCheckBoxModified = false;
+  mDefaultValue = "";
+  mpFileSelectorButton = new QToolButton;
+  mpFileSelectorButton->setText("...");
+  mpFileSelectorButton->setToolButtonStyle(Qt::ToolButtonTextOnly);
+  connect(mpFileSelectorButton, SIGNAL(clicked()), SLOT(fileSelectorButtonClicked()));
+  setLoadSelectorFilter("-");
+  setLoadSelectorCaption("-");
+  setSaveSelectorFilter("-");
+  setSaveSelectorCaption("-");
   createValueWidget();
   /* Get unit value
    * First check if unit is defined with in the component modifier.
@@ -141,8 +158,9 @@ void Parameter::updateNameLabel()
  * \param defaultValue
  * \param fromUnit
  * \param valueChanged
+ * \param adjustSize
  */
-void Parameter::setValueWidget(QString value, bool defaultValue, QString fromUnit, bool valueModified)
+void Parameter::setValueWidget(QString value, bool defaultValue, QString fromUnit, bool valueModified, bool adjustSize)
 {
   // convert the value to display unit
   if (!fromUnit.isEmpty() && mpUnitComboBox->currentText().compare(fromUnit) != 0) {
@@ -157,6 +175,7 @@ void Parameter::setValueWidget(QString value, bool defaultValue, QString fromUni
       }
     }
   }
+  mDefaultValue = defaultValue;
   QFontMetrics fm = QFontMetrics(QFont());
   switch (mValueType) {
     case Parameter::Boolean:
@@ -167,9 +186,14 @@ void Parameter::setValueWidget(QString value, bool defaultValue, QString fromUni
         mpValueComboBox->lineEdit()->setText(value);
         mpValueComboBox->lineEdit()->setModified(valueModified);
       }
-      /* Set the minimum width so that the value text will be readable */
-      fm = QFontMetrics(mpValueComboBox->lineEdit()->font());
-      mpValueComboBox->setMinimumWidth(fm.width(value) + 50);
+      if (adjustSize) {
+        /* Set the minimum width so that the value text will be readable */
+        fm = QFontMetrics(mpValueComboBox->lineEdit()->font());
+        mpValueComboBox->setMinimumWidth(fm.width(value) + 50);
+      }
+      break;
+    case Parameter::CheckBox:
+      mpValueCheckBox->setChecked(value.compare("true") == 0);
       break;
     case Parameter::Normal:
     default:
@@ -179,9 +203,11 @@ void Parameter::setValueWidget(QString value, bool defaultValue, QString fromUni
         mpValueTextBox->setText(value);
         mpValueTextBox->setModified(valueModified);
       }
-      /* Set the minimum width so that the value text will be readable */
-      fm = QFontMetrics(mpValueTextBox->font());
-      mpValueTextBox->setMinimumWidth(fm.width(value) + 50);
+      if (adjustSize) {
+        /* Set the minimum width so that the value text will be readable */
+        fm = QFontMetrics(mpValueTextBox->font());
+        mpValueTextBox->setMinimumWidth(fm.width(value) + 50);
+      }
       mpValueTextBox->setCursorPosition(0); /* move the cursor to start so that parameter value will show up from start instead of end. */
       break;
   }
@@ -193,18 +219,27 @@ QWidget* Parameter::getValueWidget()
     case Parameter::Boolean:
     case Parameter::Enumeration:
       return mpValueComboBox;
+    case Parameter::CheckBox:
+      return mpValueCheckBox;
     case Parameter::Normal:
     default:
       return mpValueTextBox;
   }
 }
 
+/*!
+ * \brief Parameter::isValueModified
+ * Returns true if value widget is changed.
+ * \return
+ */
 bool Parameter::isValueModified()
 {
   switch (mValueType) {
     case Parameter::Boolean:
     case Parameter::Enumeration:
       return mpValueComboBox->lineEdit()->isModified();
+    case Parameter::CheckBox:
+      return mValueCheckBoxModified;
     case Parameter::Normal:
     default:
       return mpValueTextBox->isModified();
@@ -222,6 +257,8 @@ QString Parameter::getValue()
     case Parameter::Boolean:
     case Parameter::Enumeration:
       return mpValueComboBox->lineEdit()->text().trimmed();
+    case Parameter::CheckBox:
+      return mpValueCheckBox->isChecked() ? "true" : "false";
     case Parameter::Normal:
     default:
       return mpValueTextBox->text().trimmed();
@@ -235,14 +272,7 @@ QString Parameter::getValue()
  */
 QString Parameter::getDefaultValue()
 {
-  switch (mValueType) {
-    case Parameter::Boolean:
-    case Parameter::Enumeration:
-      return mpValueComboBox->lineEdit()->placeholderText().trimmed();
-    case Parameter::Normal:
-    default:
-      return mpValueTextBox->placeholderText().trimmed();
-  }
+  return mDefaultValue;
 }
 
 void Parameter::setFixedState(QString fixed, bool defaultValue)
@@ -308,6 +338,9 @@ void Parameter::setEnabled(bool enable)
     case Parameter::Enumeration:
       mpValueComboBox->setEnabled(enable);
       break;
+    case Parameter::CheckBox:
+      mpValueCheckBox->setEnabled(enable);
+      break;
     case Parameter::Normal:
     default:
       mpValueTextBox->setEnabled(enable);
@@ -340,6 +373,10 @@ void Parameter::createValueWidget()
       }
       connect(mpValueComboBox, SIGNAL(currentIndexChanged(int)), SLOT(valueComboBoxChanged(int)));
       break;
+    case Parameter::CheckBox:
+      mpValueCheckBox = new QCheckBox;
+      connect(mpValueCheckBox, SIGNAL(toggled(bool)), SLOT(valueCheckBoxChanged(bool)));
+      break;
     case Parameter::Normal:
     default:
       mpValueTextBox = new QLineEdit;
@@ -347,6 +384,45 @@ void Parameter::createValueWidget()
   }
 }
 
+/*!
+ * \brief Parameter::fileSelectorButtonClicked
+ * Slot activated when mpFileSelectorButton clicked SIGNAL is raised.
+ * Opens a QFileDialog::getOpenFileName or QFileDialog::getSaveFileName so user can select a file.
+ */
+void Parameter::fileSelectorButtonClicked()
+{
+  QString filter, caption = "";
+  QString fileName;
+  /* saveSelector is given precedence if present.
+   * There is nothing about it in the specification but we do so to keep consistant with Dymola.
+   */
+  if (mSaveSelectorFilter.compare("-") != 0 && mSaveSelectorCaption.compare("-") != 0) {
+    if (mSaveSelectorFilter.compare("-") != 0) {
+      filter = mSaveSelectorFilter;
+    }
+    caption = tr("Save");
+    if (mSaveSelectorCaption.compare("-") != 0) {
+      caption = mSaveSelectorCaption;
+    }
+    fileName = StringHandler::getSaveFileName(MainWindow::instance(), caption, NULL, filter, NULL);
+  } else {
+    filter = "";
+    if (mLoadSelectorFilter.compare("-") != 0) {
+      filter = mLoadSelectorFilter;
+    }
+    caption = tr("Load");
+    if (mLoadSelectorCaption.compare("-") != 0) {
+      caption = mLoadSelectorCaption;
+    }
+    fileName = StringHandler::getOpenFileName(MainWindow::instance(), caption, NULL, filter, NULL);
+  }
+  // if user press ESC
+  if (fileName.isEmpty()) {
+    return;
+  }
+  setValueWidget(QString("\"%1\"").arg(fileName), false, mUnit, true, false);
+}
+
 /*!
  * \brief Parameter::unitComboBoxChanged
  * SLOT activated when mpUnitComboBox currentIndexChanged(QString) SIGNAL is raised.\n
@@ -378,6 +454,17 @@ void Parameter::valueComboBoxChanged(int index)
   mpValueComboBox->lineEdit()->setModified(true);
 }
 
+/*!
+ * \brief Parameter::valueCheckBoxChanged
+ * SLOT activated when mpValueCheckBox toggled(bool) SIGNAL is raised.\n
+ * Marks the item modified.
+ * \param toggle
+ */
+void Parameter::valueCheckBoxChanged(bool toggle)
+{
+  mValueCheckBoxModified = true;
+}
+
 void Parameter::showFixedMenu()
 {
   // create a menu
@@ -648,8 +735,21 @@ void ComponentParameters::setUpDialog()
             pGroupBoxGridLayout->addItem(new QSpacerItem(1, 1), layoutIndex, columnIndex++);
           }
           pGroupBoxGridLayout->addWidget(pParameter->getValueWidget(), layoutIndex, columnIndex++);
+          if (pParameter->getLoadSelectorFilter().compare("-") != 0 || pParameter->getLoadSelectorCaption().compare("-") != 0 ||
+              pParameter->getSaveSelectorFilter().compare("-") != 0 || pParameter->getSaveSelectorCaption().compare("-") != 0) {
+            pGroupBoxGridLayout->addWidget(pParameter->getFileSelectorButton(), layoutIndex, columnIndex++);
+          } else {
+            pGroupBoxGridLayout->addItem(new QSpacerItem(1, 1), layoutIndex, columnIndex++);
+          }
           if (pParameter->getUnitComboBox()->count() > 0) { // only add the unit combobox if we really have a unit
-            pGroupBoxGridLayout->addWidget(pParameter->getUnitComboBox(), layoutIndex, columnIndex++);
+            /* ticket:4421
+             * Show a fixed value when there is only one unit.
+             */
+            if (pParameter->getUnitComboBox()->count() == 1) {
+              pGroupBoxGridLayout->addWidget(new Label(pParameter->getUnitComboBox()->currentText()), layoutIndex, columnIndex++);
+            } else {
+              pGroupBoxGridLayout->addWidget(pParameter->getUnitComboBox(), layoutIndex, columnIndex++);
+            }
           } else {
             pGroupBoxGridLayout->addItem(new QSpacerItem(1, 1), layoutIndex, columnIndex++);
           }
@@ -776,7 +876,8 @@ void ComponentParameters::createTabsGroupBoxesAndParametersHelper(LibraryTreeIte
     QString groupBox = "";
     bool enable = true;
     bool showStartAttribute = false;
-    QString start, fixed = "";
+    QString loadSelectorFilter = "-", loadSelectorCaption = "-", saveSelectorFilter = "-", saveSelectorCaption = "-";
+    QString start = "", fixed = "";
     bool isParameter = (pComponent->getComponentInfo()->getVariablity().compare("parameter") == 0);
     // If not a parameter then check for start and fixed bindings. See Modelica.Electrical.Analog.Basic.Resistor parameter R.
     if (!isParameter) {
@@ -811,6 +912,12 @@ void ComponentParameters::createTabsGroupBoxesAndParametersHelper(LibraryTreeIte
       if (dialogAnnotation.at(3).compare("-") != 0) {
         showStartAttribute = (dialogAnnotation.at(3).compare("true") == 0);
       }
+      // get the loadSelector
+      loadSelectorFilter = StringHandler::removeFirstLastQuotes(dialogAnnotation.at(5));
+      loadSelectorCaption = StringHandler::removeFirstLastQuotes(dialogAnnotation.at(6));
+      // get the saveSelector
+      saveSelectorFilter = StringHandler::removeFirstLastQuotes(dialogAnnotation.at(7));
+      saveSelectorCaption = StringHandler::removeFirstLastQuotes(dialogAnnotation.at(8));
       // get the group image
       groupImage = StringHandler::removeFirstLastQuotes(dialogAnnotation.at(9));
       if (!groupImage.isEmpty()) {
@@ -844,6 +951,10 @@ void ComponentParameters::createTabsGroupBoxesAndParametersHelper(LibraryTreeIte
     // create the Parameter
     Parameter *pParameter = new Parameter(pComponent, showStartAttribute, tab, groupBox);
     pParameter->setEnabled(enable);
+    pParameter->setLoadSelectorFilter(loadSelectorFilter);
+    pParameter->setLoadSelectorCaption(loadSelectorCaption);
+    pParameter->setSaveSelectorFilter(saveSelectorFilter);
+    pParameter->setSaveSelectorCaption(saveSelectorCaption);
     QString componentDefinedInClass = pComponent->getGraphicsView()->getModelWidget()->getLibraryTreeItem()->getNameStructure();
     QString value = pComponent->getComponentInfo()->getParameterValue(pOMCProxy, componentDefinedInClass);
     pParameter->setValueWidget(value, true, pParameter->getUnit());
@@ -1338,6 +1449,28 @@ void ComponentAttributes::updateComponentAttributes()
       return;
     }
   }
+  // check for spaces
+  if (StringHandler::containsSpace(mpNameTextBox->text())) {
+    QMessageBox::critical(MainWindow::instance(), QString("%1 - %2").arg(Helper::applicationName).arg(Helper::error),
+                          tr("A component name should not have spaces. Please choose another name."), Helper::ok);
+    return;
+  }
+  // check for comma
+  if (mpNameTextBox->text().contains(',')) {
+    QMessageBox::critical(MainWindow::instance(), QString("%1 - %2").arg(Helper::applicationName).arg(Helper::error),
+                          GUIMessages::getMessage(GUIMessages::INVALID_INSTANCE_NAME).arg(mpNameTextBox->text()), Helper::ok);
+    return;
+  }
+  // check for invalid names
+  MainWindow::instance()->getOMCProxy()->setLoggingEnabled(false);
+  QList<QString> result = MainWindow::instance()->getOMCProxy()->parseString(QString("model M N %1; end M;").arg(mpNameTextBox->text()),
+                                                                             "M", false);
+  MainWindow::instance()->getOMCProxy()->setLoggingEnabled(true);
+  if (result.isEmpty()) {
+    QMessageBox::critical(MainWindow::instance(), QString("%1 - %2").arg(Helper::applicationName).arg(Helper::error),
+                          GUIMessages::getMessage(GUIMessages::INVALID_INSTANCE_NAME).arg(mpNameTextBox->text()), Helper::ok);
+    return;
+  }
   QString variability;
   if (mpConstantRadio->isChecked()) {
     variability = "constant";
@@ -1385,15 +1518,15 @@ void ComponentAttributes::updateComponentAttributes()
 }
 
 /*!
- * \class MetaModelSubModelAttributes
+ * \class CompositeModelSubModelAttributes
  * \brief A dialog for displaying SubModel attributes.
  */
 /*!
- * \brief MetaModelSubModelAttributes::MetaModelSubModelAttributes
+ * \brief CompositeModelSubModelAttributes::CompositeModelSubModelAttributes
  * \param pComponent - pointer to Component
  * \param pParent
  */
-MetaModelSubModelAttributes::MetaModelSubModelAttributes(Component *pComponent, QWidget *pParent)
+CompositeModelSubModelAttributes::CompositeModelSubModelAttributes(Component *pComponent, QWidget *pParent)
   : QDialog(pParent)
 {
   setWindowTitle(QString(Helper::applicationName).append(" - ").append(tr("SubModel Attributes")));
@@ -1404,10 +1537,10 @@ MetaModelSubModelAttributes::MetaModelSubModelAttributes(Component *pComponent,
 }
 
 /*!
- * \brief MetaModelSubModelAttributes::setUpDialog
- * Creates the dialog and set up submodel attributes of the metamodel.
+ * \brief CompositeModelSubModelAttributes::setUpDialog
+ * Creates the dialog and set up submodel attributes of the CompositeModel.
  */
-void MetaModelSubModelAttributes::setUpDialog()
+void CompositeModelSubModelAttributes::setUpDialog()
 {
   // Create the name label and text box
   mpNameLabel = new Label(Helper::name);
@@ -1438,7 +1571,17 @@ void MetaModelSubModelAttributes::setUpDialog()
   mpGeometryFileLabel = new Label(tr("Geometry File:"));
   mpGeometryFileTextBox = new QLineEdit;
   mpGeometryFileBrowseButton = new QPushButton(Helper::browse);
+  mpGeometryFileBrowseButton->setAutoDefault(false);
   connect(mpGeometryFileBrowseButton, SIGNAL(clicked()), this, SLOT(browseGeometryFile()));
+  // Model parameters
+  mpParametersLayout = new QGridLayout;
+  mpParametersLabel = new QLabel("Parameters:");
+  mpParametersLayout->addWidget(mpParametersLabel,0,0,1,2);
+  mpParametersScrollArea = new QScrollArea;
+  mpParametersScrollArea->setWidgetResizable(true);
+  mpParametersScrollWidget = new QWidget;
+  mpParametersScrollWidget->setLayout(mpParametersLayout);
+  mpParametersScrollArea->setWidget(mpParametersScrollWidget);
   // Create the buttons
   mpOkButton = new QPushButton(Helper::ok);
   mpOkButton->setAutoDefault(true);
@@ -1465,14 +1608,15 @@ void MetaModelSubModelAttributes::setUpDialog()
   pMainLayout->addWidget(mpGeometryFileLabel, 5, 0);
   pMainLayout->addWidget(mpGeometryFileTextBox, 5, 1);
   pMainLayout->addWidget(mpGeometryFileBrowseButton, 5, 2);
-  pMainLayout->addWidget(mpButtonBox, 6, 0, 1, 3, Qt::AlignRight);
+  pMainLayout->addWidget(mpParametersScrollArea,6,0,1,3);
+  pMainLayout->addWidget(mpButtonBox, 7, 0, 1, 3, Qt::AlignRight);
   setLayout(pMainLayout);
 }
 
 /*!
   Initialize the fields with values.
   */
-void MetaModelSubModelAttributes::initializeDialog()
+void CompositeModelSubModelAttributes::initializeDialog()
 {
   // set Name
   mpNameTextBox->setText(mpComponent->getName());
@@ -1486,37 +1630,49 @@ void MetaModelSubModelAttributes::initializeDialog()
   mpModelFileTextBox->setText(mpComponent->getComponentInfo()->getModelFile());
   // set the geometry file name
   mpGeometryFileTextBox->setText(mpComponent->getComponentInfo()->getGeometryFile());
-
+  // update parameter widgets
+  CompositeModelEditor *pEditor = dynamic_cast<CompositeModelEditor*>(mpComponent->getGraphicsView()->getModelWidget()->getEditor());
+  QStringList parameters = pEditor->getParameterNames(mpComponent->getName());
+  mParameterLabels.clear();
+  mParameterLineEdits.clear();
+  for(int i=0; i<parameters.size(); ++i) {
+      mParameterLabels.append(new QLabel(parameters[i]));
+      mParameterLineEdits.append(new QLineEdit(pEditor->getParameterValue(mpComponent->getName(), parameters[i])));
+      mpParametersLayout->addWidget(mParameterLabels.last(),i+1,0);
+      mpParametersLayout->addWidget(mParameterLineEdits.last(),i+1,1);
+  }
+  mpParametersScrollWidget->setVisible(!parameters.isEmpty());
+  mpParametersLabel->setVisible(!parameters.isEmpty());
 }
 
 /*!
- * \brief MetaModelSubModelAttributes::changeSimulationToolStartCommand
+ * \brief CompositeModelSubModelAttributes::changeSimulationToolStartCommand
  * Updates the simulation tool start command.\n
  * Slot activated when mpSimulationToolComboBox currentIndexChanged signal is raised.
  * \param tool
  */
-void MetaModelSubModelAttributes::changeSimulationToolStartCommand(QString tool)
+void CompositeModelSubModelAttributes::changeSimulationToolStartCommand(QString tool)
 {
   mpStartCommandTextBox->setText(StringHandler::getSimulationToolStartCommand(tool, mpStartCommandTextBox->text()));
 }
 
 /*!
- * \brief MetaModelSubModelAttributes::changeSimulationTool
+ * \brief CompositeModelSubModelAttributes::changeSimulationTool
  * Updates the simulation tool.\n
  * Slot activated when mpStartCommandTextBox textChanged signal is raised.
  * \param simulationToolStartCommand
  */
-void MetaModelSubModelAttributes::changeSimulationTool(QString simulationToolStartCommand)
+void CompositeModelSubModelAttributes::changeSimulationTool(QString simulationToolStartCommand)
 {
   mpSimulationToolComboBox->setCurrentIndex(StringHandler::getSimulationTool(simulationToolStartCommand));
 }
 
 /*!
- * \brief MetaModelSubModelAttributes::browseGeometryFile
+ * \brief CompositeModelSubModelAttributes::browseGeometryFile
  * Updates subModel parameters.\n
  * Slot activated when mpGeometryFileBrowseButton clicked signal is raised.
  */
-void MetaModelSubModelAttributes::browseGeometryFile()
+void CompositeModelSubModelAttributes::browseGeometryFile()
 {
   QString geometryFile = StringHandler::getOpenFileName(this, QString("%1 - %2").arg(Helper::applicationName).arg(Helper::chooseFile),
                                                        NULL, "", NULL);
@@ -1527,11 +1683,11 @@ void MetaModelSubModelAttributes::browseGeometryFile()
 }
 
 /*!
- * \brief MetaModelSubModelAttributes::updateSubModelParameters
+ * \brief CompositeModelSubModelAttributes::updateSubModelParameters
  * Updates subModel parameters.\n
  * Slot activated when mpOkButton clicked signal is raised.
  */
-void MetaModelSubModelAttributes::updateSubModelParameters()
+void CompositeModelSubModelAttributes::updateSubModelParameters()
 {
   // save the old ComponentInfo
   ComponentInfo oldComponentInfo(mpComponent->getComponentInfo());
@@ -1540,29 +1696,54 @@ void MetaModelSubModelAttributes::updateSubModelParameters()
   newComponentInfo.setStartCommand(mpStartCommandTextBox->text());
   newComponentInfo.setExactStep(mpExactStepCheckBox->isChecked());
   newComponentInfo.setGeometryFile(mpGeometryFileTextBox->text());
+
+
+  QStringList parameterNames, oldParameterValues, newParameterValues;
+  if(mpComponent->getGraphicsView()->getModelWidget()->getLibraryTreeItem()->getLibraryType() == LibraryTreeItem::CompositeModel) {
+    BaseEditor *pBaseEditor = mpComponent->getGraphicsView()->getModelWidget()->getEditor();
+    CompositeModelEditor *pEditor = qobject_cast<CompositeModelEditor*>(pBaseEditor);
+
+    parameterNames = pEditor->getParameterNames(mpComponent->getName());  //Assume submodel; otherwise returned list is empty
+    foreach(QString parName, parameterNames) {
+      oldParameterValues.append(pEditor->getParameterValue(mpComponent->getName(), parName));
+    }
+
+    for(int i=0; i<mParameterLineEdits.size(); ++i) {
+      newParameterValues.append(mParameterLineEdits[i]->text());
+    }
+  }
+
   // If user has really changed the Component's attributes then push that change on the stack.
-  if (oldComponentInfo != newComponentInfo) {
+  if (oldComponentInfo != newComponentInfo || oldParameterValues != newParameterValues) {
     UpdateSubModelAttributesCommand *pUpdateSubModelAttributesCommand = new UpdateSubModelAttributesCommand(mpComponent, oldComponentInfo,
-                                                                                                            newComponentInfo);
+                                                                                                            newComponentInfo,
+                                                                                                            parameterNames,
+                                                                                                            oldParameterValues,
+                                                                                                            newParameterValues);
     ModelWidget *pModelWidget = mpComponent->getGraphicsView()->getModelWidget();
     pModelWidget->getUndoStack()->push(pUpdateSubModelAttributesCommand);
     pModelWidget->updateModelText();
   }
+
   accept();
 }
 
 /*!
-  \class MetaModelConnectionAttributes
-  \brief A dialog for displaying MetaModel Connection Attributes
+  \class CompositeModelConnectionAttributes
+  \brief A dialog for displaying CompositeModel Connection Attributes
   */
 /*!
   \param pConnectionLineAnnotation - pointer to LineAnnotation
   \param pMainWindow - pointer to MainWindow
   */
-MetaModelConnectionAttributes::MetaModelConnectionAttributes(GraphicsView *pGraphicsView, LineAnnotation *pConnectionLineAnnotation,
+CompositeModelConnectionAttributes::CompositeModelConnectionAttributes(GraphicsView *pGraphicsView, LineAnnotation *pConnectionLineAnnotation,
                                                              bool edit, QWidget *pParent)
   : QDialog(pParent), mpGraphicsView(pGraphicsView), mpConnectionLineAnnotation(pConnectionLineAnnotation), mEdit(edit)
 {
+  ComponentInfo *pInfo = mpConnectionLineAnnotation->getStartComponent()->getComponentInfo();
+  bool tlm = (pInfo->getTLMCausality() == "Bidirectional");
+  int dimensions = pInfo->getDimensions();
+
   setWindowTitle(QString(Helper::applicationName).append(" - ").append(Helper::connectionAttributes));
   setAttribute(Qt::WA_DeleteOnClose);
   // heading
@@ -1588,7 +1769,7 @@ MetaModelConnectionAttributes::MetaModelConnectionAttributes(GraphicsView *pGrap
   // Create the buttons
   mpOkButton = new QPushButton(Helper::ok);
   mpOkButton->setAutoDefault(true);
-  connect(mpOkButton, SIGNAL(clicked()), this, SLOT(createMetaModelConnection()));
+  connect(mpOkButton, SIGNAL(clicked()), this, SLOT(createCompositeModelConnection()));
   mpCancelButton = new QPushButton(Helper::cancel);
   mpCancelButton->setAutoDefault(false);
   connect(mpCancelButton, SIGNAL(clicked()), this, SLOT(reject()));
@@ -1606,41 +1787,58 @@ MetaModelConnectionAttributes::MetaModelConnectionAttributes(GraphicsView *pGrap
   pMainLayout->addWidget(mpConnectionEndLabel, 3, 1);
   pMainLayout->addWidget(mpDelayLabel, 4, 0);
   pMainLayout->addWidget(mpDelayTextBox, 4, 1);
-  pMainLayout->addWidget(mpZfLabel,5, 0);
-  pMainLayout->addWidget(mpZfTextBox, 5, 1);
-  pMainLayout->addWidget(mpZfrLabel, 6, 0);
-  pMainLayout->addWidget(mpZfrTextBox, 6, 1);
-  pMainLayout->addWidget(mpAlphapLabel, 7, 0);
-  pMainLayout->addWidget(mpAlphaTextBox, 7, 1);
+  if(tlm) {     //Only show TLM parameter widgets if it is a TLM connection
+    pMainLayout->addWidget(mpZfLabel,5, 0);
+    pMainLayout->addWidget(mpZfTextBox, 5, 1);
+    if(dimensions > 1) {        //Only show rotational impedance box for 3D connections
+      pMainLayout->addWidget(mpZfrLabel, 6, 0);
+      pMainLayout->addWidget(mpZfrTextBox, 6, 1);
+    }
+    pMainLayout->addWidget(mpAlphapLabel, 7, 0);
+    pMainLayout->addWidget(mpAlphaTextBox, 7, 1);
+  }
   pMainLayout->addWidget(mpButtonBox, 8, 0, 1, 2, Qt::AlignRight);
   setLayout(pMainLayout);
 }
 
 /*!
- * \brief MetaModelConnectionAttributes::createMetaModelConnection
+ * \brief CompositeModelConnectionAttributes::createCompositeModelConnection
  * Slot activated when mpOkButton clicked signal is raised.\n
  * Creates a connection
  */
-void MetaModelConnectionAttributes::createMetaModelConnection()
+void CompositeModelConnectionAttributes::createCompositeModelConnection()
 {
+  ComponentInfo *pInfo = mpConnectionLineAnnotation->getStartComponent()->getComponentInfo();
+  bool tlm = (pInfo->getTLMCausality() == "Bidirectional");
+  int dimensions = pInfo->getDimensions();
   if (mEdit) {
-    MetaModelConnection oldMetaModelConnection;
-    oldMetaModelConnection.mDelay = mpConnectionLineAnnotation->getDelay();
-    oldMetaModelConnection.mZf = mpConnectionLineAnnotation->getZf();
-    oldMetaModelConnection.mZfr = mpConnectionLineAnnotation->getZfr();
-    oldMetaModelConnection.mAlpha = mpConnectionLineAnnotation->getAlpha();
-    MetaModelConnection newMetaModelConnection;
-    newMetaModelConnection.mDelay = mpDelayTextBox->text();
-    newMetaModelConnection.mZf = mpZfTextBox->text();
-    newMetaModelConnection.mZfr = mpZfrTextBox->text();
-    newMetaModelConnection.mAlpha = mpAlphaTextBox->text();
-    mpGraphicsView->getModelWidget()->getUndoStack()->push(new UpdateMetaModelConnection(mpConnectionLineAnnotation, oldMetaModelConnection,
-                                                                                         newMetaModelConnection));
+    CompositeModelConnection oldCompositeModelConnection;
+    oldCompositeModelConnection.mDelay = mpConnectionLineAnnotation->getDelay();
+    oldCompositeModelConnection.mZf = mpConnectionLineAnnotation->getZf();
+    oldCompositeModelConnection.mZfr = mpConnectionLineAnnotation->getZfr();
+    oldCompositeModelConnection.mAlpha = mpConnectionLineAnnotation->getAlpha();
+    CompositeModelConnection newCompositeModelConnection;
+    newCompositeModelConnection.mDelay = mpDelayTextBox->text();
+    if(tlm) { //Only update TLM parameters if this is a TLM connection
+      newCompositeModelConnection.mZf = mpZfTextBox->text();
+      if(dimensions>1) { //Only update rotational impedance if this is a 3D connection
+        newCompositeModelConnection.mZfr = mpZfrTextBox->text();
+      }
+      newCompositeModelConnection.mAlpha = mpAlphaTextBox->text();
+    }
+
+    mpGraphicsView->getModelWidget()->getUndoStack()->push(new UpdateCompositeModelConnection(mpConnectionLineAnnotation,
+                                                                                              oldCompositeModelConnection,
+                                                                                              newCompositeModelConnection));
   } else {
     mpConnectionLineAnnotation->setDelay(mpDelayTextBox->text());
-    mpConnectionLineAnnotation->setZf(mpZfTextBox->text());
-    mpConnectionLineAnnotation->setZfr(mpZfrTextBox->text());
-    mpConnectionLineAnnotation->setAlpha(mpAlphaTextBox->text());
+    if(tlm) { //Only update TLM parameters if this is a TLM connection
+      mpConnectionLineAnnotation->setZf(mpZfTextBox->text());
+      if(dimensions > 1) { //Only update rotational impedance if this is a 3D connection
+        mpConnectionLineAnnotation->setZfr(mpZfrTextBox->text());
+      }
+      mpConnectionLineAnnotation->setAlpha(mpAlphaTextBox->text());
+    }
     mpGraphicsView->getModelWidget()->getUndoStack()->push(new AddConnectionCommand(mpConnectionLineAnnotation, true));
     mpGraphicsView->getModelWidget()->getLibraryTreeItem()->emitConnectionAdded(mpConnectionLineAnnotation);
   }
