@@ -29,10 +29,7 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- *
  */
 
 #ifndef COMPONENTPROPERTIES_H
@@ -40,6 +37,8 @@
 
 #include "Component.h"
 
+#include <QRadioButton>
+
 class Parameter : public QObject
 {
   Q_OBJECT
@@ -47,6 +46,7 @@ public:
   enum ValueType {
     Normal,  /* Integer, Real etc. */
     Boolean,
+    CheckBox,
     Enumeration
   };
   Parameter(Component *pComponent, bool showStartAttribute, QString tab, QString groupBox);
@@ -62,16 +62,29 @@ public:
   FixedCheckBox* getFixedCheckBox() {return mpFixedCheckBox;}
   QString getOriginalFixedValue() {return mOriginalFixedValue;}
   void setValueType(ValueType valueType) {mValueType = valueType;}
-  void setValueWidget(QString value, bool defaultValue);
+  void setValueWidget(QString value, bool defaultValue, QString fromUnit, bool valueModified = false, bool adjustSize = true);
   ValueType getValueType() {return mValueType;}
   QWidget* getValueWidget();
   bool isValueModified();
   QString getValue();
-  Label* getUnitLabel() {return mpUnitLabel;}
+  QString getDefaultValue();
+  QToolButton* getFileSelectorButton() {return mpFileSelectorButton;}
+  void setLoadSelectorFilter(QString loadSelectorFilter) {mLoadSelectorFilter = loadSelectorFilter;}
+  QString getLoadSelectorFilter() {return mLoadSelectorFilter;}
+  void setLoadSelectorCaption(QString loadSelectorCaption) {mLoadSelectorCaption = loadSelectorCaption;}
+  QString getLoadSelectorCaption() {return mLoadSelectorCaption;}
+  void setSaveSelectorFilter(QString saveSelectorFilter) {mSaveSelectorFilter = saveSelectorFilter;}
+  QString getSaveSelectorFilter() {return mSaveSelectorFilter;}
+  void setSaveSelectorCaption(QString saveSelectorCaption) {mSaveSelectorCaption = saveSelectorCaption;}
+  QString getSaveSelectorCaption() {return mSaveSelectorCaption;}
+  QString getUnit() {return mUnit;}
+  void setDisplayUnit(QString displayUnit) {mDisplayUnit = displayUnit;}
+  QString getDisplayUnit() {return mDisplayUnit;}
+  QComboBox* getUnitComboBox() {return mpUnitComboBox;}
   Label* getCommentLabel() {return mpCommentLabel;}
   void setFixedState(QString fixed, bool defaultValue);
   QString getFixedState();
-  QString getUnitFromDerivedClass(Component *pComponent);
+  QString getModifierValueFromDerivedClass(Component *pComponent, QString modifierName);
   void setEnabled(bool enable);
 private:
   Component *mpComponent;
@@ -82,14 +95,28 @@ private:
   FixedCheckBox *mpFixedCheckBox;
   QString mOriginalFixedValue;
   ValueType mValueType;
+  bool mValueCheckBoxModified;
+  QString mDefaultValue;
   QComboBox *mpValueComboBox;
   QLineEdit *mpValueTextBox;
-  Label *mpUnitLabel;
+  QCheckBox *mpValueCheckBox;
+  QToolButton *mpFileSelectorButton;
+  QString mLoadSelectorFilter;
+  QString mLoadSelectorCaption;
+  QString mSaveSelectorFilter;
+  QString mSaveSelectorCaption;
+  QString mUnit;
+  QString mDisplayUnit;
+  QString mPreviousUnit;
+  QComboBox *mpUnitComboBox;
   Label *mpCommentLabel;
 
   void createValueWidget();
 public slots:
+  void fileSelectorButtonClicked();
+  void unitComboBoxChanged(QString text);
   void valueComboBoxChanged(int index);
+  void valueCheckBoxChanged(bool toggle);
   void showFixedMenu();
   void trueFixedClicked();
   void falseFixedClicked();
@@ -128,11 +155,10 @@ class ComponentParameters : public QDialog
 {
   Q_OBJECT
 public:
-  ComponentParameters(Component *pComponent, MainWindow *pMainWindow);
+  ComponentParameters(Component *pComponent, QWidget *pParent = 0);
   ~ComponentParameters();
 private:
   Component *mpComponent;
-  MainWindow *mpMainWindow;
   Label *mpParametersHeading;
   QFrame *mHorizontalLine;
   QTabWidget *mpParametersTabWidget;
@@ -170,12 +196,11 @@ class ComponentAttributes : public QDialog
 {
   Q_OBJECT
 public:
-  ComponentAttributes(Component *pComponent, MainWindow *pMainWindow);
+  ComponentAttributes(Component *pComponent, QWidget *pParent = 0);
   void setUpDialog();
   void initializeDialog();
 private:
   Component *mpComponent;
-  MainWindow *mpMainWindow;
   Label *mpAttributesHeading;
   QFrame *mHorizontalLine;
   QGroupBox *mpTypeGroupBox;
@@ -213,16 +238,15 @@ public slots:
   void updateComponentAttributes();
 };
 
-class SubModelAttributes : public QDialog
+class CompositeModelSubModelAttributes : public QDialog
 {
   Q_OBJECT
 public:
-  SubModelAttributes(Component *pComponent, MainWindow *pMainWindow);
+  CompositeModelSubModelAttributes(Component *pComponent, QWidget *pParent = 0);
   void setUpDialog();
   void initializeDialog();
 private:
   Component *mpComponent;
-  MainWindow *mpMainWindow;
   Label *mpNameLabel;
   QLineEdit *mpNameTextBox;
   Label *mpSimulationToolLabel;
@@ -232,49 +256,42 @@ private:
   Label *mpModelFileLabel;
   QLineEdit *mpModelFileTextBox;
   Label *mpExactStepFlagLabel;
-  QCheckBox *mpExactStepFlagCheckBox;
+  QCheckBox *mpExactStepCheckBox;
+  Label *mpGeometryFileLabel;
+  QLineEdit *mpGeometryFileTextBox;
+  QPushButton *mpGeometryFileBrowseButton;
+  QScrollArea *mpParametersScrollArea;
+  QWidget *mpParametersScrollWidget;
+  QGridLayout *mpParametersLayout;
+  QLabel *mpParametersLabel;
+  QList<QLabel*> mParameterLabels;
+  QList<QLineEdit*> mParameterLineEdits;
   QPushButton *mpOkButton;
   QPushButton *mpCancelButton;
   QDialogButtonBox *mpButtonBox;
 private slots:
   void changeSimulationToolStartCommand(QString tool);
   void changeSimulationTool(QString simulationToolStartCommand);
+  void browseGeometryFile();
   void updateSubModelParameters();
 };
 
-class Component;
-class TLMInterfacePointInfo
-{
-public:
-  TLMInterfacePointInfo(QString name, QString className, QString interfaceName);
-  QString getName();
-  QString getClassName();
-  QString getInterfaceName();
-  void setInterfaceName(QString interfaceName);
-private:
-  QString mName;
-  QString mClassName;
-  QString mInterfaceName;
-};
-
 class LineAnnotation;
-class TLMConnectionAttributes : public QDialog
+class CompositeModelConnectionAttributes : public QDialog
 {
   Q_OBJECT
 public:
-  TLMConnectionAttributes(LineAnnotation *pConnectionLineAnnotation, MainWindow *pMainWindow);
-  void setUpDialog();
-  void initializeDialog();
+  CompositeModelConnectionAttributes(GraphicsView *pGraphicsView, LineAnnotation *pConnectionLineAnnotation, bool edit, QWidget *pParent = 0);
 private:
+  GraphicsView *mpGraphicsView;
   LineAnnotation *mpConnectionLineAnnotation;
-  MainWindow *mpMainWindow;
-  QList<TLMInterfacePointInfo*> mInterfacepointsList;
-  Label *mpStartSubModelNameLabel;
-  QLineEdit *mpStartSubModelNameTextBox;
-  QComboBox *mpStartSubModelInterfacePointComboBox;
-  QComboBox *mpEndSubModelInterfacePointComboBox;
-  Label *mpEndSubModelNameLabel;
-  QLineEdit *mpEndSubModelNameTextBox;
+  bool mEdit;
+  Label *mpHeading;
+  QFrame *mpHorizontalLine;
+  Label *mpFromLabel;
+  Label *mpConnectionStartLabel;
+  Label *mpToLabel;
+  Label *mpConnectionEndLabel;
   Label *mpDelayLabel;
   QLineEdit *mpDelayTextBox;
   Label *mpZfLabel;
@@ -284,9 +301,10 @@ private:
   Label *mpAlphapLabel;
   QLineEdit *mpAlphaTextBox;
   QPushButton *mpOkButton;
+  QPushButton *mpCancelButton;
   QDialogButtonBox *mpButtonBox;
 public slots:
-  void createTLMConnection();
+  void createCompositeModelConnection();
 };
 
 #endif // COMPONENTPROPERTIES_H
