@@ -28,25 +28,31 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- *
  */
 
 #include "BreakpointDialog.h"
+#include "MainWindow.h"
+#include "BreakpointsWidget.h"
+#include "Util/Helper.h"
+#include "Util/Utilities.h"
+#include "Modeling/LibraryTreeWidget.h"
+#include "Modeling/ModelicaClassDialog.h"
+
 #include <limits>
+#include <QMessageBox>
 
 /*!
-  \class BreakpointDialog
-  \brief Interface for adding and editing breakpoint.
-  */
+ * \class BreakpointDialog
+ * \brief Interface for adding and editing breakpoint.
+ */
 /*!
-  \param pBreakpointTreeItem - pointer to BreakpointTreeItem
-  \param pBreakpointsTreeModel - pointer to BreakpointsTreeModel
-  */
+ * \brief BreakpointDialog::BreakpointDialog
+ * \param pBreakpointTreeItem - pointer to BreakpointTreeItem
+ * \param pBreakpointsTreeModel - pointer to BreakpointsTreeModel
+ */
 BreakpointDialog::BreakpointDialog(BreakpointTreeItem *pBreakpointTreeItem, BreakpointsTreeModel *pBreakpointsTreeModel)
-  : QDialog(pBreakpointsTreeModel->getBreakpointsTreeView(), Qt::WindowTitleHint)
+  : QDialog(pBreakpointsTreeModel->getBreakpointsTreeView())
 {
   setAttribute(Qt::WA_DeleteOnClose);
   setWindowTitle(QString(Helper::applicationName).append(" - ").append(tr("%1 Breakpoint").arg(pBreakpointTreeItem ? "Edit" : "Add")));
@@ -65,6 +71,7 @@ BreakpointDialog::BreakpointDialog(BreakpointTreeItem *pBreakpointTreeItem, Brea
   mpBrowseClassesButton->setAutoDefault(false);
   connect(mpBrowseClassesButton, SIGNAL(clicked()), SLOT(browseClasses()));
   mpBrowseFileSystemButton = new QPushButton(tr("File System"));
+  mpBrowseFileSystemButton->setEnabled(false);
   mpBrowseFileSystemButton->setAutoDefault(false);
   connect(mpBrowseFileSystemButton, SIGNAL(clicked()), SLOT(browseFileSystem()));
   // Create the line number label and text box
@@ -78,7 +85,7 @@ BreakpointDialog::BreakpointDialog(BreakpointTreeItem *pBreakpointTreeItem, Brea
   mpIgnoreCountSpinBox = new QSpinBox;
   mpIgnoreCountSpinBox->setRange(0, std::numeric_limits<int>::max());
   // create the condition label and text box
-  mpConditionLabel = new Label(tr("Condition:"));
+  mpConditionLabel = new Label(Helper::condition);
   mpConditionTextBox = new QLineEdit;
   mpConditionHintLabel = new Label(tr("* Use \"%1\" to set condition on simulation time.").arg("data->localData[0]->timeValue"));
   // if edit case then set the existing values
@@ -127,7 +134,7 @@ BreakpointDialog::BreakpointDialog(BreakpointTreeItem *pBreakpointTreeItem, Brea
   */
 void BreakpointDialog::browseClasses()
 {
-  MainWindow *pMainWindow = mpBreakpointsTreeModel->getBreakpointsTreeView()->getBreakpointsWidget()->getDebuggerMainWindow()->getMainWindow();
+  MainWindow *pMainWindow = MainWindow::instance();
   LibraryBrowseDialog *pLibraryBrowseDialog = new LibraryBrowseDialog(tr("Select Class"), mpFileNameTextBox, pMainWindow->getLibraryWidget());
   pLibraryBrowseDialog->exec();
 }
@@ -179,8 +186,7 @@ void BreakpointDialog::addOrEditBreakpoint()
                                                mpIgnoreCountSpinBox->value(), mpConditionTextBox->text());
     }
   } else {  /* if user has selected a class using Browse Classes button */
-    LibraryWidget *pLibraryWidget;
-    pLibraryWidget = mpBreakpointsTreeModel->getBreakpointsTreeView()->getBreakpointsWidget()->getDebuggerMainWindow()->getMainWindow()->getLibraryWidget();
+    LibraryWidget *pLibraryWidget = MainWindow::instance()->getLibraryWidget();
     LibraryTreeItem *pLibraryTreeItem = pLibraryWidget->getLibraryTreeModel()->findLibraryTreeItem(mpFileNameTextBox->text());
     if (pLibraryTreeItem) {
       if (!pLibraryTreeItem->isSaved()) {
@@ -204,7 +210,14 @@ void BreakpointDialog::addOrEditBreakpoint()
         }
       } else {  /* Edit Case */
         /* find the BreakpointMarker and update its filepath and lineNumber. */
-        pBreakpointMarker = mpBreakpointsTreeModel->findBreakpointMarker(mpBreakpointTreeItem->getFilePath(), mpBreakpointTreeItem->getLineNumber().toInt());
+        pBreakpointMarker = mpBreakpointsTreeModel->findBreakpointMarker(mpBreakpointTreeItem->getFilePath(),
+                                                                         mpBreakpointTreeItem->getLineNumber().toInt());
+        // first remove the marker
+        if (mpBreakpointTreeItem->getLibraryTreeItem()->getModelWidget() &&
+            mpBreakpointTreeItem->getLibraryTreeItem()->getModelWidget()->getEditor()) {
+          mpBreakpointTreeItem->getLibraryTreeItem()->getModelWidget()->getEditor()->getDocumentMarker()->removeMark(pBreakpointMarker);
+        }
+        // set new attributes for the breakpoint marker
         pBreakpointMarker->setFilePath(pLibraryTreeItem->getFileName());
         pBreakpointMarker->setLineNumber(lineNumber);
         pBreakpointMarker->setEnabled(mpEnableCheckBox->isChecked());
@@ -216,7 +229,9 @@ void BreakpointDialog::addOrEditBreakpoint()
         mpBreakpointsTreeModel->updateBreakpoint(mpBreakpointTreeItem, pLibraryTreeItem->getFileName(), lineNumber,
                                                  mpEnableCheckBox->isChecked(), mpIgnoreCountSpinBox->value(), mpConditionTextBox->text());
         if (pLibraryTreeItem->getModelWidget()) {
-          pLibraryTreeItem->getModelWidget()->getEditor()->getDocumentMarker()->removeMark(pBreakpointMarker);
+          if (!pLibraryTreeItem->getModelWidget()->getEditor()) {
+            pLibraryTreeItem->getModelWidget()->createModelWidgetComponents();
+          }
           pLibraryTreeItem->getModelWidget()->getEditor()->getDocumentMarker()->addMark(pBreakpointMarker, lineNumber);
         }
       }
