@@ -52,6 +52,7 @@
 #include <QMdiSubWindow>
 #include <QMdiArea>
 
+
 class OMCProxy;
 class TransformationsWidget;
 class LibraryWidget;
@@ -63,6 +64,9 @@ class GDBLoggerWidget;
 class DocumentationWidget;
 class PlotWindowContainer;
 class VariablesWidget;
+#if !defined(WITHOUT_OSG)
+class ThreeDViewer;
+#endif
 class BreakpointsWidget;
 class SimulationDialog;
 class TLMCoSimulationDialog;
@@ -73,6 +77,11 @@ class AboutOMEditDialog;
 class Label;
 class FileDataNotifier;
 class LibraryTreeItem;
+class GitCommands;
+class CommitChangesDialog;
+class TraceabilityInformationURI;
+class TraceabilityGraphViewWidget;
+
 
 class MainWindow : public QMainWindow
 {
@@ -101,16 +110,24 @@ public:
   PlotWindowContainer* getPlotWindowContainer() {return mpPlotWindowContainer;}
   VariablesWidget* getVariablesWidget() {return mpVariablesWidget;}
   QDockWidget* getVariablesDockWidget() {return mpVariablesDockWidget;}
+
+#if !defined(WITHOUT_OSG)
+  bool isThreeDViewerInitialized();
+  ThreeDViewer* getThreeDViewer();
+  QDockWidget* getThreeDViewerDockWidget() {return mpThreeDViewerDockWidget;}
+#endif
   SimulationDialog* getSimulationDialog() {return mpSimulationDialog;}
   TLMCoSimulationDialog* getTLMCoSimulationDialog() {return mpTLMCoSimulationDialog;}
   ModelWidgetContainer* getModelWidgetContainer() {return mpModelWidgetContainer;}
   WelcomePageWidget* getWelcomePageWidget() {return mpWelcomePageWidget;}
+  GitCommands* getGitCommands() {return mpGitCommands;}
+  CommitChangesDialog* getCommitChangesDialog() {return mpCommitChangesDialog;}
+  TraceabilityInformationURI* getTraceabilityInformationURI() {return mpTraceabilityInformationURI;}
   QStatusBar* getStatusBar() {return mpStatusBar;}
   QProgressBar* getProgressBar() {return mpProgressBar;}
   void showProgressBar() {mpProgressBar->setVisible(true);}
   void hideProgressBar() {mpProgressBar->setVisible(false);}
-  Label* getPointerXPositionLabel() {return mpPointerXPositionLabel;}
-  Label* getPointerYPositionLabel() {return mpPointerYPositionLabel;}
+  Label* getPositionLabel() {return mpPositionLabel;}
   QTabBar* getPerspectiveTabBar() {return mpPerspectiveTabbar;}
   QTimer* getAutoSaveTimer() {return mpAutoSaveTimer;}
   QAction* getSaveAction() {return mpSaveAction;}
@@ -124,6 +141,8 @@ public:
   QAction* getResetZoomAction() {return mpResetZoomAction;}
   QAction* getZoomInAction() {return mpZoomInAction;}
   QAction* getZoomOutAction() {return mpZoomOutAction;}
+  QAction* getCloseAllWindowsAction() {return mpCloseAllWindowsAction;}
+  QAction* getCloseAllWindowsButThisAction() {return mpCloseAllWindowsButThisAction;}
   QAction* getSimulateModelAction() {return mpSimulateModelAction;}
   QAction* getSimulateWithTransformationalDebuggerAction() {return mpSimulateWithTransformationalDebuggerAction;}
   QAction* getSimulateWithAlgorithmicDebuggerAction() {return mpSimulateWithAlgorithmicDebuggerAction;}
@@ -149,12 +168,20 @@ public:
   QAction* getImportFromOMNotebookAction() {return mpImportFromOMNotebookAction;}
   QAction* getImportNgspiceNetlistAction() {return mpImportNgspiceNetlistAction;}
   QAction* getConnectModeAction() {return mpConnectModeAction;}
+  QAction* getTransitionModeAction() {return mpTransitionModeAction;}
   QAction* getReSimulateModelAction() {return mpReSimulateModelAction;}
   QAction* getReSimulateSetupAction() {return mpReSimulateSetupAction;}
   QAction* getSimulationParamsAction() {return mpSimulationParamsAction;}
   QAction* getFetchInterfaceDataAction() {return mpFetchInterfaceDataAction;}
   QAction* getAlignInterfacesAction() {return mpAlignInterfacesAction;}
   QAction* getTLMSimulationAction() {return mpTLMCoSimulationAction;}
+  QAction* getLogCurrentFileAction() {return mpLogCurrentFileAction;}
+  QAction* getStageCurrentFileForCommitAction() {return mpStageCurrentFileForCommitAction;}
+  QAction* getUnstageCurrentFileFromCommitAction() {return mpUnstageCurrentFileFromCommitAction;}
+  QAction* getCommitFilesAction() {return mpCommitFilesAction;}
+  QAction* getRevertCommitAction() {return mpRevertCommitAction;}
+  QAction* getCleanWorkingDirectoryAction() {return mpCleanWorkingDirectoryAction;}
+  QAction* getTraceabilityQueryAction() {return mpTraceabilityQueryAction;}
   void addRecentFile(const QString &fileName, const QString &encoding);
   void updateRecentFileActions();
   void closeEvent(QCloseEvent *event);
@@ -175,7 +202,7 @@ public:
   void exportModelFMU(LibraryTreeItem *pLibraryTreeItem);
   void exportModelXML(LibraryTreeItem *pLibraryTreeItem);
   void exportModelFigaro(LibraryTreeItem *pLibraryTreeItem);
-  void fetchInterfaceData(LibraryTreeItem *pLibraryTreeItem);
+  void fetchInterfaceData(LibraryTreeItem *pLibraryTreeItem, QString singleModel=QString());
   void TLMSimulate(LibraryTreeItem *pLibraryTreeItem);
   void exportModelToOMNotebook(LibraryTreeItem *pLibraryTreeItem);
   void createOMNotebookTitleCell(LibraryTreeItem *pLibraryTreeItem, QDomDocument xmlDocument, QDomElement domElement);
@@ -213,14 +240,21 @@ private:
   PlotWindowContainer *mpPlotWindowContainer;
   VariablesWidget *mpVariablesWidget;
   QDockWidget *mpVariablesDockWidget;
+  TraceabilityGraphViewWidget *mpTraceabilityGraphViewWidget;
+#if !defined(WITHOUT_OSG)
+  ThreeDViewer *mpThreeDViewer;
+  QDockWidget *mpThreeDViewerDockWidget;
+#endif
   SimulationDialog *mpSimulationDialog;
   TLMCoSimulationDialog *mpTLMCoSimulationDialog;
   ModelWidgetContainer *mpModelWidgetContainer;
   WelcomePageWidget *mpWelcomePageWidget;
+  GitCommands *mpGitCommands;
+  CommitChangesDialog *mpCommitChangesDialog;
+  TraceabilityInformationURI *mpTraceabilityInformationURI;
   QStackedWidget *mpCentralStackedWidget;
   QProgressBar *mpProgressBar;
-  Label *mpPointerXPositionLabel;
-  Label *mpPointerYPositionLabel;
+  Label *mpPositionLabel;
   QTabBar *mpPerspectiveTabbar;
   QStatusBar *mpStatusBar;
   QTimer *mpAutoSaveTimer;
@@ -232,9 +266,9 @@ private:
   QAction *mpLoadModelicaLibraryAction;
   QAction *mpOpenResultFileAction;
   QAction *mpOpenTransformationFileAction;
-  // MetaModel File Actions
-  QAction *mpNewMetaModelFileAction;
-  QAction *mpOpenMetaModelFileAction;
+  // CompositeModel File Actions
+  QAction *mpNewCompositeModelFileAction;
+  QAction *mpOpenCompositeModelFileAction;
   QAction *mpLoadExternModelAction;
   QAction *mpOpenDirectoryAction;
   QAction *mpSaveAction;
@@ -285,6 +319,15 @@ private:
   // Debug Menu
   QAction *mpDebugConfigurationsAction;
   QAction *mpAttachDebuggerToRunningProcessAction;
+  // Git Menu
+  QAction *mpCreateGitRepositoryAction;
+  QAction *mpLogCurrentFileAction;
+  QAction *mpStageCurrentFileForCommitAction;
+  QAction *mpUnstageCurrentFileFromCommitAction;
+  QAction *mpCommitFilesAction;
+  QAction *mpRevertCommitAction;
+  QAction *mpCleanWorkingDirectoryAction;
+  QAction *mpTraceabilityQueryAction;
   // Tools Menu
   QAction *mpShowOMCLoggerWidgetAction;
   QAction *mpShowOpenModelicaCommandPromptAction;
@@ -303,6 +346,7 @@ private:
   QAction *mpModelicaDocumentationAction;
   QAction *mpModelicaByExampleAction;
   QAction *mpModelicaWebReferenceAction;
+  QAction *mpOpenModelicaTLMSimulatorDocumentationAction;
   QAction *mpAboutOMEditAction;
   // Toolbar Actions
   // Shapes Toolbar Actions
@@ -314,6 +358,7 @@ private:
   QAction *mpTextShapeAction;
   QAction *mpBitmapShapeAction;
   QAction *mpConnectModeAction;
+  QAction *mpTransitionModeAction;
   // Model Switcher Toolbar Actions
   QAction *mpModelSwitcherActions[MaxRecentFiles];
   // Plot Toolbar Actions
@@ -321,6 +366,8 @@ private:
   QAction *mpReSimulateSetupAction;
   QAction *mpNewPlotWindowAction;
   QAction *mpNewParametricPlotWindowAction;
+  QAction *mpNewArrayPlotWindowAction;
+  QAction *mpNewArrayParametricPlotWindowAction;
 #if !defined(WITHOUT_OSG)
   QAction *mpNewAnimationWindowAction;
 #endif
@@ -357,13 +404,13 @@ public slots:
   void loadModelicaLibrary();
   void showOpenResultFileDialog();
   void showOpenTransformationFileDialog();
-  void createNewMetaModelFile();
-  void openMetaModelFile();
+  void createNewCompositeModelFile();
+  void openCompositeModelFile();
   void loadExternalModels();
   void openDirectory();
   void loadSystemLibrary();
-  void readOutputFile(qint64 bytes);
-  void readErrorFile(qint64 bytes);
+  void writeOutputFileData(QString data);
+  void writeErrorFileData(QString data);
   void openRecentFile();
   void clearRecentFilesList();
   void undo();
@@ -412,6 +459,7 @@ public slots:
   void openModelicaDocumentation();
   void openModelicaByExample();
   void openModelicaWebReference();
+  void openOpenModelicaTLMSimulatorDocumentationAction();
   void openAboutOMEdit();
   void toggleShapesButton();
   void openRecentModelWidget();
@@ -422,6 +470,7 @@ public slots:
 private slots:
   void perspectiveTabChanged(int tabIndex);
   void documentationDockWidgetVisibilityChanged(bool visible);
+  void threeDViewerDockWidgetVisibilityChanged(bool visible);
   void autoSave();
   void switchToWelcomePerspectiveSlot();
   void switchToModelingPerspectiveSlot();
@@ -429,6 +478,14 @@ private slots:
   void switchToAlgorithmicDebuggingPerspectiveSlot();
   void showConfigureDialog();
   void showAttachToProcessDialog();
+  void createGitRepository();
+  void logCurrentFile();
+  void stageCurrentFileForCommit();
+  void unstageCurrentFileFromCommit();
+  void commitFiles();
+  void revertCommit();
+  void cleanWorkingDirectory();
+  void queryTraceabilityInformation();
 private:
   void createActions();
   void createToolbars();
@@ -440,7 +497,7 @@ private:
   void switchToAlgorithmicDebuggingPerspective();
   void closeAllWindowsButThis(QMdiArea *pMdiArea);
   void tileSubWindows(QMdiArea *pMdiArea, bool horizontally);
-  void fetchInterfaceDataHelper(LibraryTreeItem *pLibraryTreeItem);
+  void fetchInterfaceDataHelper(LibraryTreeItem *pLibraryTreeItem, QString singleModel=QString());
 protected:
   virtual void dragEnterEvent(QDragEnterEvent *event);
   virtual void dragMoveEvent(QDragMoveEvent *event);
