@@ -96,7 +96,8 @@ SimulationDialog::~SimulationDialog()
 void SimulationDialog::show(LibraryTreeItem *pLibraryTreeItem, bool isReSimulate, SimulationOptions simulationOptions)
 {
   /* restore the window geometry. */
-  if (OptionsDialog::instance()->getGeneralSettingsPage()->getPreserveUserCustomizations()) {
+  if (OptionsDialog::instance()->getGeneralSettingsPage()->getPreserveUserCustomizations() &&
+      Utilities::getApplicationSettings()->contains("SimulationDialog/geometry")) {
     restoreGeometry(Utilities::getApplicationSettings()->value("SimulationDialog/geometry").toByteArray());
   }
   mpLibraryTreeItem = pLibraryTreeItem;
@@ -124,7 +125,13 @@ void SimulationDialog::directSimulate(LibraryTreeItem *pLibraryTreeItem, bool la
 #else
   assert(false==launchAnimation);
 #endif
+  /* ticket:4440 OMEdit does not simulate
+   * Make sure we always simulate when directSimulate() is called.
+   */
+  bool simulateCheckBoxState = mpSimulateCheckBox->isChecked();
+  mpSimulateCheckBox->setChecked(true);
   simulate();
+  mpSimulateCheckBox->setChecked(simulateCheckBoxState);
 }
 
 /*!
@@ -170,12 +177,15 @@ void SimulationDialog::setUpForm()
   pSimulationIntervalGridLayout->setColumnStretch(1, 1);
   pSimulationIntervalGridLayout->addWidget(mpStartTimeLabel, 0, 0);
   pSimulationIntervalGridLayout->addWidget(mpStartTimeTextBox, 0, 1);
+  pSimulationIntervalGridLayout->addWidget(new Label(Helper::secs), 0, 2);
   pSimulationIntervalGridLayout->addWidget(mpStopTimeLabel, 1, 0);
   pSimulationIntervalGridLayout->addWidget(mpStopTimeTextBox, 1, 1);
+  pSimulationIntervalGridLayout->addWidget(new Label(Helper::secs), 1, 2);
   pSimulationIntervalGridLayout->addWidget(mpNumberofIntervalsRadioButton, 2, 0);
-  pSimulationIntervalGridLayout->addWidget(mpNumberofIntervalsSpinBox, 2, 1);
+  pSimulationIntervalGridLayout->addWidget(mpNumberofIntervalsSpinBox, 2, 1, 1, 2);
   pSimulationIntervalGridLayout->addWidget(mpIntervalRadioButton, 3, 0);
   pSimulationIntervalGridLayout->addWidget(mpIntervalTextBox, 3, 1);
+  pSimulationIntervalGridLayout->addWidget(new Label(Helper::secs), 3, 2);
   mpSimulationIntervalGroupBox->setLayout(pSimulationIntervalGridLayout);
   // Integration
   mpIntegrationGroupBox = new QGroupBox(tr("Integration"));
@@ -194,7 +204,7 @@ void SimulationDialog::setUpForm()
   if (currentIndex > -1) {
     mpMethodComboBox->setCurrentIndex(currentIndex);
   }
-  connect(mpMethodComboBox, SIGNAL(currentIndexChanged(QString)), SLOT(enableDasslOptions(QString)));
+  connect(mpMethodComboBox, SIGNAL(currentIndexChanged(QString)), SLOT(enableDasslIdaOptions(QString)));
   mpMehtodHelpButton = new QToolButton;
   mpMehtodHelpButton->setIcon(QIcon(":/Resources/icons/link-external.svg"));
   mpMehtodHelpButton->setToolTip(tr("Integration help"));
@@ -208,44 +218,45 @@ void SimulationDialog::setUpForm()
   QStringList jacobianMethods, jacobianMethodsDesc;
   MainWindow::instance()->getOMCProxy()->getJacobianMethods(&jacobianMethods, &jacobianMethodsDesc);
   mpJacobianComboBox = new QComboBox;
+  mpJacobianComboBox->addItem("");
+  mpJacobianComboBox->setItemData(0, "", Qt::ToolTipRole);
   mpJacobianComboBox->addItems(jacobianMethods);
   for (int i = 0 ; i < jacobianMethodsDesc.size() ; i++) {
-    mpJacobianComboBox->setItemData(i, jacobianMethodsDesc.at(i), Qt::ToolTipRole);
+    mpJacobianComboBox->setItemData(i + 1, jacobianMethodsDesc.at(i), Qt::ToolTipRole);
   }
   connect(mpJacobianComboBox, SIGNAL(currentIndexChanged(int)), SLOT(updateJacobianToolTip(int)));
-  updateJacobianToolTip(0);
-  // dassl options
-  mpDasslOptionsGroupBox = new QGroupBox(tr("DASSL Options"));
+  // dassl/ida options
+  mpDasslIdaOptionsGroupBox = new QGroupBox(tr("DASSL/IDA Options"));
   // no root finding
-  mpDasslRootFindingCheckBox = new QCheckBox(tr("Root Finding"));
-  mpDasslRootFindingCheckBox->setToolTip(tr("Activates the internal root finding procedure of dassl"));
-  mpDasslRootFindingCheckBox->setChecked(true);
+  mpRootFindingCheckBox = new QCheckBox(tr("Root Finding"));
+  mpRootFindingCheckBox->setToolTip(tr("Activates the internal root finding procedure of methods: dassl and ida."));
+  mpRootFindingCheckBox->setChecked(true);
   // no restart
-  mpDasslRestartCheckBox = new QCheckBox(tr("Restart After Event"));
-  mpDasslRestartCheckBox->setToolTip(tr("Activates the restart of dassl after an event is performed"));
-  mpDasslRestartCheckBox->setChecked(true);
+  mpRestartAfterEventCheckBox = new QCheckBox(tr("Restart After Event"));
+  mpRestartAfterEventCheckBox->setToolTip(tr("Activates the restart of the integration method after an event is performed, used by the methods: dassl, ida"));
+  mpRestartAfterEventCheckBox->setChecked(true);
   // initial step size
-  mpDasslInitialStepSizeLabel = new Label(tr("Initial Step Size:"));
-  mpDasslInitialStepSizeTextBox = new QLineEdit;
+  mpInitialStepSizeLabel = new Label(tr("Initial Step Size:"));
+  mpInitialStepSizeTextBox = new QLineEdit;
   // max step size
-  mpDasslMaxStepSizeLabel = new Label(tr("Maximum Step Size:"));
-  mpDasslMaxStepSizeTextBox = new QLineEdit;
+  mpMaxStepSizeLabel = new Label(tr("Maximum Step Size:"));
+  mpMaxStepSizeTextBox = new QLineEdit;
   // max integration order
-  mpDasslMaxIntegrationOrderLabel = new Label(tr("Maximum Integration Order:"));
-  mpDasslMaxIntegrationOrderSpinBox = new QSpinBox;
-  mpDasslMaxIntegrationOrderSpinBox->setValue(5);
-  // set the layout for DASSL options groupbox
-  QGridLayout *pDasslOptionsGridLayout = new QGridLayout;
-  pDasslOptionsGridLayout->setColumnStretch(1, 1);
-  pDasslOptionsGridLayout->addWidget(mpDasslRootFindingCheckBox, 0, 0, 1, 2);
-  pDasslOptionsGridLayout->addWidget(mpDasslRestartCheckBox, 1, 0, 1, 2);
-  pDasslOptionsGridLayout->addWidget(mpDasslInitialStepSizeLabel, 2, 0);
-  pDasslOptionsGridLayout->addWidget(mpDasslInitialStepSizeTextBox, 2, 1);
-  pDasslOptionsGridLayout->addWidget(mpDasslMaxStepSizeLabel, 3, 0);
-  pDasslOptionsGridLayout->addWidget(mpDasslMaxStepSizeTextBox, 3, 1);
-  pDasslOptionsGridLayout->addWidget(mpDasslMaxIntegrationOrderLabel, 4, 0);
-  pDasslOptionsGridLayout->addWidget(mpDasslMaxIntegrationOrderSpinBox, 4, 1);
-  mpDasslOptionsGroupBox->setLayout(pDasslOptionsGridLayout);
+  mpMaxIntegrationOrderLabel = new Label(tr("Maximum Integration Order:"));
+  mpMaxIntegrationOrderSpinBox = new QSpinBox;
+  mpMaxIntegrationOrderSpinBox->setValue(5);
+  // set the layout for DASSL/Ida options groupbox
+  QGridLayout *pDasslIdaOptionsGridLayout = new QGridLayout;
+  pDasslIdaOptionsGridLayout->setColumnStretch(1, 1);
+  pDasslIdaOptionsGridLayout->addWidget(mpRootFindingCheckBox, 0, 0, 1, 2);
+  pDasslIdaOptionsGridLayout->addWidget(mpRestartAfterEventCheckBox, 1, 0, 1, 2);
+  pDasslIdaOptionsGridLayout->addWidget(mpInitialStepSizeLabel, 2, 0);
+  pDasslIdaOptionsGridLayout->addWidget(mpInitialStepSizeTextBox, 2, 1);
+  pDasslIdaOptionsGridLayout->addWidget(mpMaxStepSizeLabel, 3, 0);
+  pDasslIdaOptionsGridLayout->addWidget(mpMaxStepSizeTextBox, 3, 1);
+  pDasslIdaOptionsGridLayout->addWidget(mpMaxIntegrationOrderLabel, 4, 0);
+  pDasslIdaOptionsGridLayout->addWidget(mpMaxIntegrationOrderSpinBox, 4, 1);
+  mpDasslIdaOptionsGroupBox->setLayout(pDasslIdaOptionsGridLayout);
   // set the layout for integration groupbox
   QGridLayout *pIntegrationGridLayout = new QGridLayout;
   pIntegrationGridLayout->setColumnStretch(1, 1);
@@ -256,10 +267,11 @@ void SimulationDialog::setUpForm()
   pIntegrationGridLayout->addWidget(mpToleranceTextBox, 1, 1, 1, 2);
   pIntegrationGridLayout->addWidget(mpJacobianLabel, 2, 0);
   pIntegrationGridLayout->addWidget(mpJacobianComboBox, 2, 1, 1, 2);
-  pIntegrationGridLayout->addWidget(mpDasslOptionsGroupBox, 3, 0, 1, 3);
+  pIntegrationGridLayout->addWidget(mpDasslIdaOptionsGroupBox, 3, 0, 1, 3);
   mpIntegrationGroupBox->setLayout(pIntegrationGridLayout);
   // Compiler Flags
-  mpCflagsLabel = new Label(tr("Compiler Flags (Optional):"));
+  mpCflagsLabel = new Label(tr("C/C++ Compiler Flags (Optional):"));
+  mpCflagsLabel->setToolTip(tr("Space separated list of C/C++ compiler flags"));
   mpCflagsTextBox = new QLineEdit;
   // Number of Processors
   mpNumberOfProcessorsLabel = new Label(tr("Number of Processors:"));
@@ -602,6 +614,7 @@ void SimulationDialog::initializeFields(bool isReSimulate, SimulationOptions sim
     }
     // if ignoreSimulationFlagsAnnotation flag is not set then read the __OpenModelica_simulationFlags annotation
     if (!OptionsDialog::instance()->getSimulationPage()->getIgnoreSimulationFlagsAnnotationCheckBox()->isChecked()) {
+      QMap<QString, QString> additionalSimulationFlags;
       // if the class has __OpenModelica_simulationFlags annotation then use its values.
       QList<QString> simulationFlags = MainWindow::instance()->getOMCProxy()->getAnnotationNamedModifiers(mClassName, "__OpenModelica_simulationFlags");
       foreach (QString simulationFlag, simulationFlags) {
@@ -610,10 +623,10 @@ void SimulationDialog::initializeFields(bool isReSimulate, SimulationOptions sim
           mpClockComboBox->setCurrentIndex(mpClockComboBox->findText(value));
         } else if (simulationFlag.compare("cpu") == 0) {
           mpCPUTimeCheckBox->setChecked(true);
-        } else if (simulationFlag.compare("dasslnoRestart") == 0) {
-          mpDasslRestartCheckBox->setChecked(false);
-        } else if (simulationFlag.compare("dasslnoRootFinding") == 0) {
-          mpDasslRootFindingCheckBox->setChecked(false);
+        } else if (simulationFlag.compare("noRestart") == 0) {
+          mpRestartAfterEventCheckBox->setChecked(false);
+        } else if (simulationFlag.compare("noRootFinding") == 0) {
+          mpRootFindingCheckBox->setChecked(false);
         } else if (simulationFlag.compare("emit_protected") == 0) {
           mpProtectedVariablesCheckBox->setChecked(true);
         } else if (simulationFlag.compare("f") == 0) {
@@ -625,7 +638,7 @@ void SimulationDialog::initializeFields(bool isReSimulate, SimulationOptions sim
         } else if (simulationFlag.compare("iit") == 0) {
           mpEquationSystemInitializationTimeTextBox->setText(value);
         } else if (simulationFlag.compare("initialStepSize") == 0) {
-          mpDasslInitialStepSizeTextBox->setText(value);
+          mpInitialStepSizeTextBox->setText(value);
         } else if (simulationFlag.compare("jacobian") == 0) {
           mpJacobianComboBox->setCurrentIndex(mpJacobianComboBox->findText(value));
         } else if (simulationFlag.compare("l") == 0) {
@@ -633,9 +646,9 @@ void SimulationDialog::initializeFields(bool isReSimulate, SimulationOptions sim
         } else if (simulationFlag.compare("ls") == 0) {
           mpLinearSolverComboBox->setCurrentIndex(mpLinearSolverComboBox->findText(value));
         } else if (simulationFlag.compare("maxIntegrationOrder") == 0) {
-          mpDasslMaxIntegrationOrderSpinBox->setValue(value.toInt());
+          mpMaxIntegrationOrderSpinBox->setValue(value.toInt());
         } else if (simulationFlag.compare("maxStepSize") == 0) {
-          mpDasslMaxStepSizeTextBox->setText(value);
+          mpMaxStepSizeTextBox->setText(value);
         } else if (simulationFlag.compare("nls") == 0) {
           mpNonLinearSolverComboBox->setCurrentIndex(mpNonLinearSolverComboBox->findText(value));
         } else if (simulationFlag.compare("noEquidistantTimeGrid") == 0) {
@@ -660,8 +673,24 @@ void SimulationDialog::initializeFields(bool isReSimulate, SimulationOptions sim
             }
             i++;
           }
+        } else { // put everything else in the Additional Simulation Flags textbox
+          additionalSimulationFlags.insert(simulationFlag, value);
         }
       }
+      QStringList additionalSimulationFlagsList;
+      QMapIterator<QString, QString> additionalSimulationFlagsIterator(additionalSimulationFlags);
+      additionalSimulationFlagsIterator.toFront();
+      while (additionalSimulationFlagsIterator.hasNext()) {
+        additionalSimulationFlagsIterator.next();
+        if (additionalSimulationFlagsIterator.value().compare("()") == 0) {
+          additionalSimulationFlagsList.append(QString("-%1").arg(additionalSimulationFlagsIterator.key()));
+        } else {
+          additionalSimulationFlagsList.append(QString("-%1=%2")
+                                               .arg(additionalSimulationFlagsIterator.key())
+                                               .arg(additionalSimulationFlagsIterator.value()));
+        }
+      }
+      mpAdditionalSimulationFlagsTextBox->setText(additionalSimulationFlagsList.join(" "));
     }
     mpCflagsTextBox->setEnabled(true);
     mpFileNameTextBox->setEnabled(true);
@@ -690,15 +719,15 @@ void SimulationDialog::initializeFields(bool isReSimulate, SimulationOptions sim
       mpJacobianComboBox->setCurrentIndex(currentIndex);
     }
     // no root finding
-    mpDasslRootFindingCheckBox->setChecked(simulationOptions.getDasslRootFinding());
+    mpRootFindingCheckBox->setChecked(simulationOptions.getRootFinding());
     // no restart
-    mpDasslRestartCheckBox->setChecked(simulationOptions.getDasslRestart());
+    mpRestartAfterEventCheckBox->setChecked(simulationOptions.getRestartAfterEvent());
     // initial step size
-    mpDasslInitialStepSizeTextBox->setText(simulationOptions.getDasslInitialStepSize());
+    mpInitialStepSizeTextBox->setText(simulationOptions.getInitialStepSize());
     // max step size
-    mpDasslMaxStepSizeTextBox->setText(simulationOptions.getDasslMaxStepSize());
+    mpMaxStepSizeTextBox->setText(simulationOptions.getMaxStepSize());
     // max integration order
-    mpDasslMaxIntegrationOrderSpinBox->setValue(simulationOptions.getDasslMaxIntegration());
+    mpMaxIntegrationOrderSpinBox->setValue(simulationOptions.getMaxIntegration());
     // Compiler Flags
     mpCflagsTextBox->setDisabled(true);
     // Number of Processors
@@ -800,10 +829,20 @@ void SimulationDialog::initializeFields(bool isReSimulate, SimulationOptions sim
  */
 bool SimulationDialog::translateModel(QString simulationParameters)
 {
+  // reset simulation setting
+  OptionsDialog::instance()->saveSimulationSettings();
+  // set the infoXMLOperations flag
+  if (OptionsDialog::instance()->getDebuggerPage()->getGenerateOperationsCheckBox()->isChecked()) {
+    MainWindow::instance()->getOMCProxy()->setCommandLineOptions("-d=infoXmlOperations");
+  }
   // check reset messages number before simulation option
   if (OptionsDialog::instance()->getMessagesPage()->getResetMessagesNumberBeforeSimulationCheckBox()->isChecked()) {
     MessagesWidget::instance()->resetMessagesNumber();
   }
+  // check clear messages browser before simulation option
+  if (OptionsDialog::instance()->getMessagesPage()->getClearMessagesBrowserBeforeSimulationCheckBox()->isChecked()) {
+    MessagesWidget::instance()->clearMessages();
+  }
   /* save the model before translating */
   if (OptionsDialog::instance()->getSimulationPage()->getSaveClassBeforeSimulationCheckBox()->isChecked() &&
       !mpLibraryTreeItem->isSaved() &&
@@ -823,6 +862,10 @@ bool SimulationDialog::translateModel(QString simulationParameters)
   bool result = MainWindow::instance()->getOMCProxy()->translateModel(mClassName, simulationParameters);
   // reset simulation setting
   OptionsDialog::instance()->saveSimulationSettings();
+  // set the infoXMLOperations flag
+  if (OptionsDialog::instance()->getDebuggerPage()->getGenerateOperationsCheckBox()->isChecked()) {
+    MainWindow::instance()->getOMCProxy()->setCommandLineOptions("-d=infoXmlOperations");
+  }
   return result;
 }
 
@@ -835,11 +878,11 @@ SimulationOptions SimulationDialog::createSimulationOptions()
   simulationOptions.setMethod(mpMethodComboBox->currentText());
   simulationOptions.setTolerance(mpToleranceTextBox->text());
   simulationOptions.setJacobian(mpJacobianComboBox->itemData(mpJacobianComboBox->currentIndex()).toString());
-  simulationOptions.setDasslRootFinding(mpDasslRootFindingCheckBox->isChecked());
-  simulationOptions.setDasslRestart(mpDasslRestartCheckBox->isChecked());
-  simulationOptions.setDasslInitialStepSize(mpDasslInitialStepSizeTextBox->text());
-  simulationOptions.setDasslMaxStepSize(mpDasslMaxStepSizeTextBox->text());
-  simulationOptions.setDasslMaxIntegration(mpDasslMaxIntegrationOrderSpinBox->value());
+  simulationOptions.setRootFinding(mpRootFindingCheckBox->isChecked());
+  simulationOptions.setRestartAfterEvent(mpRestartAfterEventCheckBox->isChecked());
+  simulationOptions.setInitialStepSize(mpInitialStepSizeTextBox->text());
+  simulationOptions.setMaxStepSize(mpMaxStepSizeTextBox->text());
+  simulationOptions.setMaxIntegration(mpMaxIntegrationOrderSpinBox->value());
   simulationOptions.setCflags(mpCflagsTextBox->text());
   simulationOptions.setNumberOfProcessors(mpNumberOfProcessorsSpinBox->value());
   simulationOptions.setBuildOnly(mpBuildOnlyCheckBox->isChecked());
@@ -905,28 +948,30 @@ SimulationOptions SimulationDialog::createSimulationOptions()
                          .arg("variableFilter").arg(simulationOptions.getVariableFilter()));
   simulationFlags.append(QString("-r=").append(simulationOptions.getResultFileName()));
   // jacobian
-  simulationFlags.append(QString("-jacobian=").append(mpJacobianComboBox->currentText()));
-  // dassl options
-  if (mpDasslOptionsGroupBox->isEnabled()) {
-    // dassl root finding
-    if (!mpDasslRootFindingCheckBox->isChecked()) {
-      simulationFlags.append("-dasslnoRootFinding");
+  if (!mpJacobianComboBox->currentText().isEmpty()) {
+    simulationFlags.append(QString("-jacobian=").append(mpJacobianComboBox->currentText()));
+  }
+  // dassl/ida options
+  if (mpDasslIdaOptionsGroupBox->isEnabled()) {
+    // root finding
+    if (!mpRootFindingCheckBox->isChecked()) {
+      simulationFlags.append("-noRootFinding");
     }
-    // dassl restart
-    if (!mpDasslRestartCheckBox->isChecked()) {
-      simulationFlags.append("-dasslnoRestart");
+    // restart after event
+    if (!mpRestartAfterEventCheckBox->isChecked()) {
+      simulationFlags.append("-noRestart");
     }
-    // dassl initial step size
-    if (!mpDasslInitialStepSizeTextBox->text().isEmpty()) {
-      simulationFlags.append(QString("-initialStepSize=").append(mpDasslInitialStepSizeTextBox->text()));
+    // initial step size
+    if (!mpInitialStepSizeTextBox->text().isEmpty()) {
+      simulationFlags.append(QString("-initialStepSize=").append(mpInitialStepSizeTextBox->text()));
     }
-    // dassl max step size
-    if (!mpDasslMaxStepSizeTextBox->text().isEmpty()) {
-      simulationFlags.append(QString("-maxStepSize=").append(mpDasslMaxStepSizeTextBox->text()));
+    // max step size
+    if (!mpMaxStepSizeTextBox->text().isEmpty()) {
+      simulationFlags.append(QString("-maxStepSize=").append(mpMaxStepSizeTextBox->text()));
     }
-    // dassl max step size
-    if (mpDasslMaxIntegrationOrderSpinBox->value() != 5) {
-      simulationFlags.append(QString("-maxIntegrationOrder=").append(QString::number(mpDasslMaxIntegrationOrderSpinBox->value())));
+    // max step size
+    if (mpMaxIntegrationOrderSpinBox->value() != 5) {
+      simulationFlags.append(QString("-maxIntegrationOrder=").append(QString::number(mpMaxIntegrationOrderSpinBox->value())));
     }
   }
   // emit protected variables
@@ -1013,6 +1058,17 @@ void SimulationDialog::createAndShowSimulationOutputWidget(SimulationOptions sim
   if (simulationOptions.isReSimulate() && simulationOptions.getLaunchAlgorithmicDebugger()) {
     showAlgorithmicDebugger(simulationOptions);
   } else {
+    /* ticket:4406 Option to automatically close Simulation Completed Window
+     * Close all completed SimulationOutputWidget windows
+     */
+    if (OptionsDialog::instance()->getSimulationPage()->getCloseSimulationOutputWidgetsBeforeSimulationCheckBox()->isChecked()) {
+      foreach (SimulationOutputWidget *pSimulationOutputWidget, mSimulationOutputWidgetsList) {
+        if (!(pSimulationOutputWidget->getSimulationProcessThread()->isCompilationProcessRunning() ||
+              pSimulationOutputWidget->getSimulationProcessThread()->isSimulationProcessRunning())) {
+          pSimulationOutputWidget->close();
+        }
+      }
+    }
     SimulationOutputWidget *pSimulationOutputWidget = new SimulationOutputWidget(simulationOptions);
     mSimulationOutputWidgetsList.append(pSimulationOutputWidget);
     int xPos = QApplication::desktop()->availableGeometry().width() - pSimulationOutputWidget->frameSize().width() - 20;
@@ -1089,66 +1145,68 @@ void SimulationDialog::saveSimulationFlagsAnnotation()
   // old simulation flags
   QString oldSimulationFlags = QString("annotate=%1").arg(MainWindow::instance()->getOMCProxy()->getSimulationFlagsAnnotation(mpLibraryTreeItem->getNameStructure()));
   // new simulation flags
-  QStringList simulationFlags;
+  QMap<QString, QString> simulationFlags;
   if (!mpClockComboBox->currentText().isEmpty()) {
-    simulationFlags.append(QString("%1=\"%2\"").arg("clock").arg(mpClockComboBox->currentText()));
+    simulationFlags.insert("clock", mpClockComboBox->currentText());
   }
   if (mpCPUTimeCheckBox->isChecked()) {
-    simulationFlags.append(QString("%1=\"()\"").arg("cpu"));
+    simulationFlags.insert("cpu", "()");
   }
-  if (!mpDasslRestartCheckBox->isChecked()) {
-    simulationFlags.append(QString("%1=\"()\"").arg("dasslnoRestart"));
+  if (!mpRestartAfterEventCheckBox->isChecked()) {
+    simulationFlags.insert("noRestart", "()");
   }
-  if (!mpDasslRootFindingCheckBox->isChecked()) {
-    simulationFlags.append(QString("%1=\"()\"").arg("dasslnoRootFinding"));
+  if (!mpRootFindingCheckBox->isChecked()) {
+    simulationFlags.insert("noRootFinding", "()");
   }
   if (mpProtectedVariablesCheckBox->isChecked()) {
-    simulationFlags.append(QString("%1=\"()\"").arg("emit_protected"));
+    simulationFlags.insert("emit_protected", "()");
   }
   if (!mpModelSetupFileTextBox->text().isEmpty()) {
-    simulationFlags.append(QString("%1=\"%2\"").arg("f").arg(mpModelSetupFileTextBox->text()));
+    simulationFlags.insert("f", mpModelSetupFileTextBox->text());
   }
   if (!mpEquationSystemInitializationFileTextBox->text().isEmpty()) {
-    simulationFlags.append(QString("%1=\"%2\"").arg("iif").arg(mpEquationSystemInitializationFileTextBox->text()));
+    simulationFlags.insert("iif", mpEquationSystemInitializationFileTextBox->text());
   }
   if (!mpInitializationMethodComboBox->currentText().isEmpty()) {
-    simulationFlags.append(QString("%1=\"%2\"").arg("iim").arg(mpInitializationMethodComboBox->currentText()));
+    simulationFlags.insert("iim", mpInitializationMethodComboBox->currentText());
   }
   if (!mpEquationSystemInitializationTimeTextBox->text().isEmpty()) {
-    simulationFlags.append(QString("%1=\"%2\"").arg("iit").arg(mpEquationSystemInitializationTimeTextBox->text()));
+    simulationFlags.insert("iit", mpEquationSystemInitializationTimeTextBox->text());
   }
-  if (!mpDasslInitialStepSizeTextBox->text().isEmpty()) {
-    simulationFlags.append(QString("%1=\"%2\"").arg("initialStepSize").arg(mpDasslInitialStepSizeTextBox->text()));
+  if (!mpInitialStepSizeTextBox->text().isEmpty()) {
+    simulationFlags.insert("initialStepSize", mpInitialStepSizeTextBox->text());
+  }
+  if (!mpJacobianComboBox->currentText().isEmpty()) {
+    simulationFlags.insert("jacobian", mpJacobianComboBox->currentText());
   }
-  simulationFlags.append(QString("%1=\"%2\"").arg("jacobian").arg(mpJacobianComboBox->currentText()));
   if (!mpLinearizationTimeTextBox->text().isEmpty()) {
-    simulationFlags.append(QString("%1=\"%2\"").arg("l").arg(mpLinearizationTimeTextBox->text()));
+    simulationFlags.insert("l", mpLinearizationTimeTextBox->text());
   }
   if (!mpLinearSolverComboBox->currentText().isEmpty()) {
-    simulationFlags.append(QString("%1=\"%2\"").arg("ls").arg(mpLinearSolverComboBox->currentText()));
+    simulationFlags.insert("ls", mpLinearSolverComboBox->currentText());
   }
-  if (mpDasslMaxIntegrationOrderSpinBox->value() != 5) {
-    simulationFlags.append(QString("%1=\"%2\"").arg("maxIntegrationOrder").arg(mpDasslMaxIntegrationOrderSpinBox->value()));
+  if (mpMaxIntegrationOrderSpinBox->value() != 5) {
+    simulationFlags.insert("maxIntegrationOrder", QString::number(mpMaxIntegrationOrderSpinBox->value()));
   }
-  if (!mpDasslMaxStepSizeTextBox->text().isEmpty()) {
-    simulationFlags.append(QString("%1=\"%2\"").arg("maxStepSize").arg(mpDasslMaxStepSizeTextBox->text()));
+  if (!mpMaxStepSizeTextBox->text().isEmpty()) {
+    simulationFlags.insert("maxStepSize", mpMaxStepSizeTextBox->text());
   }
   if (!mpNonLinearSolverComboBox->currentText().isEmpty()) {
-    simulationFlags.append(QString("%1=\"%2\"").arg("nls").arg(mpNonLinearSolverComboBox->currentText()));
+    simulationFlags.insert("nls", mpNonLinearSolverComboBox->currentText());
   }
   if (mpEquidistantTimeGridCheckBox->isEnabled() && !mpEquidistantTimeGridCheckBox->isChecked()) {
-    simulationFlags.append(QString("%1=\"()\"").arg("noEquidistantTimeGrid"));
+    simulationFlags.insert("noEquidistantTimeGrid", "()");
   }
   if (!mpStoreVariablesAtEventsCheckBox->isChecked()) {
-    simulationFlags.append(QString("%1=\"()\"").arg("noEventEmit"));
+    simulationFlags.insert("noEventEmit", "()");
   }
   if (!mpOutputVariablesTextBox->text().isEmpty()) {
-    simulationFlags.append(QString("%1=\"%2\"").arg("output").arg(mpOutputVariablesTextBox->text()));
+    simulationFlags.insert("output", mpOutputVariablesTextBox->text());
   }
   if (!mpResultFileName->text().isEmpty()) {
-    simulationFlags.append(QString("%1=\"%2\"").arg("r").arg(mpResultFileName->text()));
+    simulationFlags.insert("r", mpResultFileName->text());
   }
-  simulationFlags.append(QString("%1=\"%2\"").arg("s").arg(mpMethodComboBox->currentText()));
+  simulationFlags.insert("s", mpMethodComboBox->currentText());
   QStringList logStreams;
   int i = 0;
   while (QLayoutItem* pLayoutItem = mpLoggingGroupLayout->itemAt(i)) {
@@ -1161,9 +1219,29 @@ void SimulationDialog::saveSimulationFlagsAnnotation()
     i++;
   }
   if (logStreams.size() > 0) {
-    simulationFlags.append(QString("%1=\"%2\"").arg("lv").arg(logStreams.join(",")));
+    simulationFlags.insert("lv", logStreams.join(","));
   }
-  QString newSimulationFlags = QString("__OpenModelica_simulationFlags(%1)").arg(simulationFlags.join(","));
+  QStringList additionalSimulationFlags = mpAdditionalSimulationFlagsTextBox->text().split(" ", QString::SkipEmptyParts);
+  foreach (QString additionalSimulationFlag, additionalSimulationFlags) {
+    additionalSimulationFlag = additionalSimulationFlag.trimmed();
+    if (additionalSimulationFlag.startsWith('-')) {
+      additionalSimulationFlag.remove(0, 1);
+    }
+    QStringList nameValueList = additionalSimulationFlag.split("=", QString::SkipEmptyParts);
+    if (nameValueList.size() < 2) {
+      simulationFlags.insert(nameValueList.at(0), "()");
+    } else {
+      simulationFlags.insert(nameValueList.at(0), nameValueList.at(1));
+    }
+  }
+  QStringList simulationFlagsList;
+  QMapIterator<QString, QString> simulationFlagsIterator(simulationFlags);
+  simulationFlagsIterator.toFront();
+  while (simulationFlagsIterator.hasNext()) {
+    simulationFlagsIterator.next();
+    simulationFlagsList.append(QString("%1=\"%2\"").arg(simulationFlagsIterator.key(), simulationFlagsIterator.value()));
+  }
+  QString newSimulationFlags = QString("__OpenModelica_simulationFlags(%1)").arg(simulationFlagsList.join(","));
   // if we have ModelWidget for class then put the change on undo stack.
   if (mpLibraryTreeItem->getModelWidget()) {
     UpdateClassSimulationFlagsAnnotationCommand *pUpdateClassSimulationFlagsAnnotationCommand;
@@ -1214,6 +1292,8 @@ void SimulationDialog::performSimulation()
   }
   MainWindow::instance()->getOMCProxy()->setCommandLineOptions("+profiling=" + mpProfilingComboBox->currentText());
   simulationOptions = createSimulationOptions();
+  // change the cursor to Qt::WaitCursor
+  QApplication::setOverrideCursor(Qt::WaitCursor);
   // show the progress bar
   MainWindow::instance()->getStatusBar()->showMessage(tr("Translating %1.").arg(mClassName));
   MainWindow::instance()->getProgressBar()->setRange(0, 0);
@@ -1222,6 +1302,8 @@ void SimulationDialog::performSimulation()
   // hide the progress bar
   MainWindow::instance()->hideProgressBar();
   MainWindow::instance()->getStatusBar()->clearMessage();
+  // restore the cursor
+  QApplication::restoreOverrideCursor();
   mIsReSimulate = false;
   if (isTranslationSuccessful) {
     // check if we can compile using the target compiler
@@ -1311,9 +1393,11 @@ void SimulationDialog::simulationProcessFinished(SimulationOptions simulationOpt
     pOMCProxy->closeSimulationResultFile();
     if (list.size() > 0) {
       if (OptionsDialog::instance()->getSimulationPage()->getSwitchToPlottingPerspectiveCheckBox()->isChecked()) {
+        bool showPlotWindow = true;
 #if !defined(WITHOUT_OSG)
         // if simulated with animation then open the animation directly.
         if (mpLaunchAnimationCheckBox->isChecked()) {
+          showPlotWindow = false;
           if (simulationOptions.getResultFileName().endsWith(".mat")) {
             MainWindow::instance()->getPlotWindowContainer()->addAnimationWindow(MainWindow::instance()->getPlotWindowContainer()->subWindowList().isEmpty());
             AnimationWindow *pAnimationWindow = MainWindow::instance()->getPlotWindowContainer()->getCurrentAnimationWindow();
@@ -1325,9 +1409,19 @@ void SimulationDialog::simulationProcessFinished(SimulationOptions simulationOpt
             MessagesWidget::instance()->addGUIMessage(MessageItem(MessageItem::Modelica, "", false, 0, 0, 0, 0, msg, Helper::scriptingKind,
                                                                   Helper::notificationLevel));
           }
+        } else {
+          showPlotWindow = true;
         }
 #endif
         MainWindow::instance()->getPerspectiveTabBar()->setCurrentIndex(2);
+        if (showPlotWindow) {
+          OMPlot::PlotWindow *pPlotWindow = MainWindow::instance()->getPlotWindowContainer()->getTopPlotWindow();
+          if (pPlotWindow) {
+            MainWindow::instance()->getPlotWindowContainer()->setTopPlotWindowActive();
+          } else {
+            MainWindow::instance()->getPlotWindowContainer()->addPlotWindow(MainWindow::instance()->getPlotWindowContainer()->subWindowList().isEmpty());
+          }
+        }
       } else {
         // stay in current perspective and show variables browser
         MainWindow::instance()->getVariablesDockWidget()->show();
@@ -1393,13 +1487,13 @@ void SimulationDialog::updateMethodToolTip(int index)
  * Enables/disables the Dassl options group box
  * \param method
  */
-void SimulationDialog::enableDasslOptions(QString method)
+void SimulationDialog::enableDasslIdaOptions(QString method)
 {
-  if (method.compare("dassl") == 0) {
-    mpDasslOptionsGroupBox->setEnabled(true);
+  if (method.compare("dassl") == 0 || method.compare("ida") == 0) {
+    mpDasslIdaOptionsGroupBox->setEnabled(true);
     mpEquidistantTimeGridCheckBox->setEnabled(true);
   } else {
-    mpDasslOptionsGroupBox->setEnabled(false);
+    mpDasslIdaOptionsGroupBox->setEnabled(false);
     mpEquidistantTimeGridCheckBox->setEnabled(false);
   }
 }
@@ -1519,7 +1613,7 @@ void SimulationDialog::simulate()
       }
       if ((mpLibraryTreeItem->getModelWidget() && mpSaveExperimentAnnotationCheckBox->isChecked()) ||
           mpSaveSimulationFlagsAnnotationCheckBox->isChecked()) {
-        mpLibraryTreeItem->getModelWidget()->getUndoStack()->beginMacro("Simulation settings");
+        mpLibraryTreeItem->getModelWidget()->beginMacro("Simulation settings");
       }
       if (mpSaveExperimentAnnotationCheckBox->isChecked()) {
         saveExperimentAnnotation();
@@ -1529,7 +1623,7 @@ void SimulationDialog::simulate()
       }
       if ((mpLibraryTreeItem->getModelWidget() && mpSaveExperimentAnnotationCheckBox->isChecked()) ||
           mpSaveSimulationFlagsAnnotationCheckBox->isChecked()) {
-        mpLibraryTreeItem->getModelWidget()->getUndoStack()->endMacro();
+        mpLibraryTreeItem->getModelWidget()->endMacro();
       }
       if (mpSimulateCheckBox->isChecked()) {
         performSimulation();
