@@ -38,6 +38,7 @@
 #include "Modeling/LibraryTreeWidget.h"
 
 #include <QApplication>
+#include <QDesktopWidget>
 #include <QGridLayout>
 #include <QStylePainter>
 #include <QPainter>
@@ -149,7 +150,6 @@ FileDataNotifier::FileDataNotifier(const QString fileName)
 {
   mFile.setFileName(fileName);
   mStop = false;
-  mBytesAvailable = 0;
 }
 
 /*!
@@ -164,17 +164,6 @@ void FileDataNotifier::exit(int retcode)
   QThread::exit(retcode);
 }
 
-/*!
- * \brief FileDataNotifier::read
- * Reads the bytes from the file.
- * \param maxlen
- * \return
- */
-QByteArray FileDataNotifier::read(qint64 maxlen)
-{
-  return mFile.read(maxlen);
-}
-
 /*!
  * \brief FileDataNotifier::run
  * Reimplentation of QThread::run().
@@ -190,9 +179,8 @@ void FileDataNotifier::run()
         break;
       }
       // if file has bytes available to read.
-      if (mFile.bytesAvailable() > mBytesAvailable) {
-        mBytesAvailable = mFile.bytesAvailable();
-        emit bytesAvailable(mFile.bytesAvailable());
+      if (mFile.bytesAvailable() > 0) {
+        emit sendData(QString(mFile.readAll()));
       }
       Sleep::sleep(1);
     }
@@ -469,12 +457,12 @@ QSettings* Utilities::getApplicationSettings()
 
 
 /*!
- * \brief Utilities::parseMetaModelText
- * Parses the MetaModel text against the schema.
+ * \brief Utilities::parseCompositeModelText
+ * Parses the CompositeModel text against the schema.
  * \param pMessageHandler
  * \param contents
  */
-void Utilities::parseMetaModelText(MessageHandler *pMessageHandler, QString contents)
+void Utilities::parseCompositeModelText(MessageHandler *pMessageHandler, QString contents)
 {
   QFile schemaFile(QString(":/Resources/XMLSchema/tlmModelDescription.xsd"));
   schemaFile.open(QIODevice::ReadOnly);
@@ -813,6 +801,25 @@ QGenericMatrix<3,3, double> Utilities::getRotationMatrix(QGenericMatrix<3,1,doub
   return R;
 }
 
+#ifdef WIN32
+QString Utilities::getGDBPath()
+{
+#if defined(__MINGW32__) && !defined(__MINGW64__)
+  const char *sgdb = "/tools/msys/mingw32/bin/gdb.exe";
+#endif
+#if defined(__MINGW64__)
+  const char *sgdb = "/tools/msys/mingw64/bin/gdb.exe";
+#endif
+  const char *OMDEV = getenv("OMDEV");
+  if (QString(OMDEV).isEmpty()) {
+    return QString(Helper::OpenModelicaHome).append(sgdb);
+  } else {
+    QString qOMDEV = QString(OMDEV).replace("\\", "/");
+    return QString(qOMDEV).append(sgdb);
+  }
+}
+#endif
+
 Utilities::FileIconProvider::FileIconProviderImplementation *instance()
 {
   static Utilities::FileIconProvider::FileIconProviderImplementation theInstance;
@@ -865,3 +872,135 @@ QIcon Utilities::FileIconProvider::icon(const QFileInfo &info)
 {
   return instance()->icon(info);
 }
+
+bool Utilities::containsWord(QString text, int index, QString keyword, bool checkParenthesis)
+{
+  if (index + keyword.length() > text.length()) {
+    return false;
+  }
+  QString textToMatch = text.mid(index, keyword.length());
+  QRegExp keywordRegExp("\\b" + keyword + "\\b");
+  if (keywordRegExp.indexIn(textToMatch) != -1 && (index + keyword.length() == text.length() ||
+                                                   text[index + keyword.length()].isSpace() ||
+                                                   (checkParenthesis && text[index + keyword.length()] == '('))) {
+    return true;
+  }
+  return false;
+}
+
+/*!
+ * \brief Utilities::convertMMToPixel
+ * Converts the value from mm to pixels
+ * pixel = (dpi * mm / 1 inch)
+ * 1 inch is 25.4
+ * \param value
+ * \return
+ */
+qreal Utilities::convertMMToPixel(qreal value)
+{
+  return (QApplication::desktop()->screen()->logicalDpiX() * value) / 25.4;
+}
+
+/*!
+ * \brief Utilities::maxi
+ * This function gives the maximum
+ * \param arr
+ * \param n
+ * \return
+ */
+float Utilities::maxi(float arr[],int n) {
+  float m = 0;
+  for (int i = 0; i < n; ++i)
+    if (m < arr[i])
+      m = arr[i];
+  return m;
+}
+
+/*!
+ * \brief Utilities::mini
+ * This function gives the minimum
+ * \param arr
+ * \param n
+ * \return
+ */
+float Utilities::mini(float arr[], int n) {
+  float m = 1;
+  for (int i = 0; i < n; ++i)
+    if (m > arr[i])
+      m = arr[i];
+  return m;
+}
+
+/*!
+ * \brief liang_barsky_clipper
+ * Liang–Barsky algorithm to find the intersection point.
+ * Returns a list of points. The first point is when line is coming from outside and intersects the rectangle.
+ * The second point in the list is when line is comming from inside and intersects the rectangle.
+ * \param xmin
+ * \param ymin
+ * \param xmax
+ * \param ymax
+ * \param x1
+ * \param y1
+ * \param x2
+ * \param y2
+ * \return
+ */
+QList<QPointF> Utilities::liangBarskyClipper(float xmin, float ymin, float xmax, float ymax, float x1, float y1, float x2, float y2) {
+  // defining variables
+  float p1 = -(x2 - x1);
+  float p2 = -p1;
+  float p3 = -(y2 - y1);
+  float p4 = -p3;
+
+  float q1 = x1 - xmin;
+  float q2 = xmax - x1;
+  float q3 = y1 - ymin;
+  float q4 = ymax - y1;
+
+  float posarr[5], negarr[5];
+  int posind = 1, negind = 1;
+  posarr[0] = 1;
+  negarr[0] = 0;
+
+  if ((p1 == 0 && q1 < 0) || (p3 == 0 && q3 < 0)) {
+      qDebug() << "Line is parallel to clipping window!";
+      return QList<QPointF>();
+  }
+  if (p1 != 0) {
+    float r1 = q1 / p1;
+    float r2 = q2 / p2;
+    if (p1 < 0) {
+      negarr[negind++] = r1; // for negative p1, add it to negative array
+      posarr[posind++] = r2; // and add p2 to positive array
+    } else {
+      negarr[negind++] = r2;
+      posarr[posind++] = r1;
+    }
+  }
+  if (p3 != 0) {
+    float r3 = q3 / p3;
+    float r4 = q4 / p4;
+    if (p3 < 0) {
+      negarr[negind++] = r3;
+      posarr[posind++] = r4;
+    } else {
+      negarr[negind++] = r4;
+      posarr[posind++] = r3;
+    }
+  }
+
+  float xn1, yn1, xn2, yn2;
+  float rn1, rn2;
+  rn1 = maxi(negarr, negind); // maximum of negative array
+  rn2 = mini(posarr, posind); // minimum of positive array
+
+  xn1 = x1 + p2 * rn1;
+  yn1 = y1 + p4 * rn1; // computing new points
+
+  xn2 = x1 + p2 * rn2;
+  yn2 = y1 + p4 * rn2;
+
+//  qDebug() << x1 << y1 << xn1 << yn1 << x2 << y2 << xn2 << yn2;
+  return QList<QPointF>() << QPointF(xn1, yn1) << QPointF(xn2, yn2);
+}
