@@ -120,7 +120,7 @@ namespace IAEX
     const char *omhome = getenv("OPENMODELICAHOME");
   #ifdef WIN32
     if (!omhome) {
-      QMessageBox::critical(0, "OMNotebook", "OPENMODELICAHOME not set", "OK");
+      QMessageBox::critical(0, tr("Error"), tr("OPENMODELICAHOME not set"), "OK");
       app_->quit();
       exit(1);
     }
@@ -142,7 +142,7 @@ namespace IAEX
     mainWindow = new QMainWindow();
     QDir dir;
 
-    // when last window closed, the applicaiton should quit also
+    // when last window closed, the application should quit also
     QObject::connect(app_, SIGNAL(lastWindowClosed()), app_, SLOT(quit()));
 
     //Create a commandCenter.
@@ -156,7 +156,7 @@ namespace IAEX
 
     if(!QDir().exists(openmodelica))
     {
-      QMessageBox::critical( 0, "OpenModelica Error", "The environment variable OPENMODELICAHOME="+openmodelica+" is not a valid directory" );
+      QMessageBox::critical( 0, "OpenModelica Error", tr("The environment variable OPENMODELICAHOME=%1 is not a valid directory").arg(openmodelica) );
       exit(1);
     }
 
@@ -164,23 +164,22 @@ namespace IAEX
     OmcInteractiveEnvironment *env = OmcInteractiveEnvironment::getInstance();
     env->evalExpression("setCommandLineOptions(\"+d=shortOutput\")");
     QString cmdLine = env->getResult();
-    cout << "Set shortOutput flag: " << cmdLine.toStdString() << std::endl;
+    //cout << "Set shortOutput flag: " << cmdLine.toStdString() << std::endl;
     QString tmpDir = OmcInteractiveEnvironment::TmpPath();
     if (!QDir().exists(tmpDir)) QDir().mkdir(tmpDir);
     tmpDir = QDir(tmpDir).canonicalPath();
-    cout << "Temp.Dir " << tmpDir.toStdString() << std::endl;
+    //cout << "Temp.Dir " << tmpDir.toStdString() << std::endl;
     QString cdCmd = "cd(\"" + tmpDir + "\")";
     env->evalExpression(cdCmd);
     QString cdRes = env->getResult();
     cdRes.remove("\"");
     if (0 != tmpDir.compare(cdRes)) {
-      QMessageBox::critical( 0, "OpenModelica Error", "Could not create or cd to temp-dir\nCommand:\n  "+tmpDir+"\nReturned:\n  "+cdRes);
+      QMessageBox::critical( 0, "OpenModelica Error", tr("Could not create or cd to temp-dir\nCommand:\n  %1\nReturned:\n  %2").arg(tmpDir).arg(cdRes));
       exit(1);
     }
 
     // 2005-12-17 AF, Create instance (load styles) of stylesheet
     // 2006-04-10 AF, use environment variable to find stylesheet.xml
-    Stylesheet *sheet;
     try
     {
       QString stylesheetfile;
@@ -189,11 +188,11 @@ namespace IAEX
       else
         stylesheetfile = openmodelica + "/share/omnotebook/stylesheet.xml";
 
-      sheet = Stylesheet::instance( stylesheetfile );
+      Stylesheet::instance( stylesheetfile );
     }
     catch( exception &e )
     {
-      QMessageBox::warning( 0, "Error", e.what(), "OK" );
+      QMessageBox::warning( 0, tr("Error"), e.what(), "OK" );
       exit(-1);
     }
 
@@ -213,7 +212,7 @@ namespace IAEX
     {
       QString msg = e.what();
       msg += "\nCould not create command completion class, exiting OMNotebook";
-      QMessageBox::warning( 0, "Error", msg, "OK" );
+      QMessageBox::warning( 0, tr("Error"), msg, "OK" );
       std::exit(-1);
     }
 
@@ -312,8 +311,7 @@ namespace IAEX
     {
       if( !dir.remove( removeList_.at(i) ))
       {
-        QString msg = "Could not remove temporary image " + removeList_.at(i) + " from harddrive.";
-        QMessageBox::warning( 0, "Warning", msg, "OK" );
+        QMessageBox::warning( 0, tr("Warning"), tr("Could not remove temporary image %1 from harddrive.").arg(removeList_.at(i)), "OK" );
       }
     }
   }
@@ -400,7 +398,7 @@ namespace IAEX
    * \author Ingemar Axelsson and Anders Fernström
    * \date 2006-05-03 (update)
    *
-   * \brief Open an file, and display the content of hte file
+   * \brief Open an file, and display the content of the file
    *
    * 2005-09-22 AF, added the filename to the NotebookWindow() call
    * 2005-10-11 AF, Porting, added resize call, so all cells get the
@@ -428,7 +426,7 @@ namespace IAEX
       DocumentView *v = new NotebookWindow(d, filename);
       add(v);
 
-      // 2006-01-31 AF, Open window minimized insted of normal
+      // 2006-01-31 AF, Open window minimized instead of normal
 
       //v->showMinimized();
 
@@ -446,6 +444,14 @@ namespace IAEX
       v->raise();  // for MacOS
       v->activateWindow(); // for Windows
 
+      vector<DocumentView *> windowViews = documentViewList();
+      vector<DocumentView *>::iterator v_iter = windowViews.begin();
+      while( v_iter != windowViews.end() )
+      {
+        ((NotebookWindow *)*v_iter)->updateWindowMenu();
+        ++v_iter;
+      }
+
       QDesktopWidget dw;
       v->move(0, 0);
       v->resize(dw.geometry().width(),dw.geometry().height());
@@ -466,7 +472,7 @@ namespace IAEX
   * \date 2006-01-16
   *
   * \brief Add filename to a list of temporary files that should
-  * be deleted when the applicaiton quits.
+  * be deleted when the application quits.
   */
   void CellApplication::removeTempFiles(QString filename)
   {
@@ -477,7 +483,7 @@ namespace IAEX
   * \author Anders Fernström
   * \date 2006-01-27
   *
-  * \brief returns list of all current doucment views
+  * \brief returns list of all current document views
   */
   vector<DocumentView *> CellApplication::documentViewList()
   {
@@ -516,6 +522,12 @@ namespace IAEX
       else
         ++dv_iter;
     }
+    dv_iter = views_.begin();
+    while( dv_iter != views_.end() )
+    {
+      ((NotebookWindow *)*dv_iter)->updateWindowMenu();
+      ++dv_iter;
+    }
   }
 
   /*!
