@@ -186,7 +186,7 @@ namespace IAEX
     delete factory_;
 
     // 2006-01-16 AF (update), remove all images in memory and add
-    // the temporary images to removelist in the main applicaiton
+    // the temporary images to removelist in the main application
     QHash<QString, QImage*>::iterator i_iter = images_.begin();
     while( i_iter != images_.end() )
     {
@@ -393,11 +393,11 @@ namespace IAEX
   void CellDocument::cursorDeleteCell()
   {
     executeCommand(new DeleteSelectedCellsCommand());
-    qDebug("cursorDeleteCell");
+    //qDebug("cursorDeleteCell");
 
     //emit cursorChanged(); //This causes an untracable segfault.
 
-    qDebug("Finished");
+    //qDebug("Finished");
   }
 
   /*!
@@ -666,7 +666,7 @@ namespace IAEX
 
     // save the image temporary to the harddrive
     QImageWriter writer( name, "png" );
-    writer.setDescription( "Temporary OMNotebook image" );
+    writer.setDescription( tr("Temporary OMNotebook image") );
     writer.setQuality( 100 );
     writer.write( *image );
 
@@ -828,7 +828,7 @@ namespace IAEX
           // TO BIG
           if( height > (scrollBottom-scrollTop) )
           {
-            qDebug( "TO BIG" );
+            qDebug( "TOO BIG" );
             // cell so big that it span over entire viewarea
             return;
           }
@@ -854,7 +854,7 @@ namespace IAEX
               pos = 0;
 
             // set new scrollvalue
-            cout << "UP: old(" << scroll_->verticalScrollBar()->value() << "), new(" << pos << ")" << endl;
+            //cout << "UP: old(" << scroll_->verticalScrollBar()->value() << "), new(" << pos << ")" << endl;
             scroll_->verticalScrollBar()->setValue( pos );
           }
           // DOWN
@@ -1081,8 +1081,8 @@ namespace IAEX
 //  void CellDocument::anchorClicked(const QUrl *link)
   {
     // 2006-02-10 AF, check if path is empty
-    fprintf(stderr, "received link: %s\n", link->toString().toStdString().c_str());
-    fflush(stderr); fflush(stdout);
+    //fprintf(stderr, "received link: %s\n", link->toString().toStdString().c_str());
+    //fflush(stderr); fflush(stdout);
     if( !link->path().isEmpty() )
     {
       // 2005-12-05 AF, check if filename exists, otherwise use work dir
@@ -1471,4 +1471,3 @@ namespace IAEX
 
 
 };
-
